#define DIRECTDRAW_VERSION 0x300

#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>
#include "HostGfx.h"
#include "host_internal.h"

//Right now, this only affects this file...
#define mbENABLE_FULL_SCREEN

//This is too slow for general use.
//#define CHECK_PIXEL_VALUES


//If defined, align HostGfxState to the given number of bits.
#define HOSTGFX_STATE_ALIGNMENT_BITS (5)

#ifdef HOSTGFX_STATE_ALIGNMENT_BITS
# define HOSTGFX_STATE_ALIGNMENT_BYTES (1<<(HOSTGFX_STATE_ALIGNMENT_BITS))
#endif

//If defined, put palettes in a malloc'ed area, with each palette
//being aligned to a 1<<PALETTE_ALIGNMENT_BITS-byte boundary. If not defined,
//they're directly in the HostGfxState.
//#define PALETTE_ALIGNMENT_BITS (5)

#ifdef PALETTE_ALIGNMENT_BITS
# define PALETTE_ALIGNMENT_BYTES (1<<(PALETTE_ALIGNMENT_BITS))
# define PALETTE_ALIGN(N) (((N)+PALETTE_ALIGNMENT_BYTES-1)&~(PALETTE_ALIGNMENT_BYTES-1))
#endif

//////////////////////////////////////////////////////////////////////////

//the 'list' returned if there are no refresh rates.
static const int empty_refresh_rate_list=0;

//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////

enum {
	FULL_SCREEN_WIDTH=800,
	FULL_SCREEN_HEIGHT=600,
	MAX_FAILED_STARTS=5,
};

//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// HostGfx
//
enum {
	max_num_back_buffers=3
};

struct HostGfxState {
	//palettes
#ifdef PALETTE_ALIGNMENT_BITS
	byte *lookup_8bit;//[8];
	word *lookup_16bit;//[8];
	dword *lookup_32bit;//[8];
	char *palette_data;
#else
	dword lookup_32bit[8];
	word lookup_16bit[8];
	byte lookup_8bit[8];
#endif
	byte lookup_24bit[8][4];
	float palette[8][3];

	//Owner. Used by DirectDraw. Must be toplevel window.
	HWND owner;
	//Target. Display is shown in this window, so could have
	//child window here.
	HWND target;
	IDirectDraw2 *ddraw;
	IDirectDrawSurface2 *primary;
	DDSURFACEDESC primary_desc;
	IDirectDrawSurface2 *back_buffers[max_num_back_buffers];
	int num_back_buffers;
	DDSURFACEDESC back_buffer_desc;
	IDirectDrawClipper *primary_clipper;
	IDirectDrawPalette *primary_palette;
	int cpu_buffer,gpu_buffer;
	int buffers_ok;
	//HostGfx_BeebScreenType beeb_screen_type;
	//HostGfx_ScanlinesMode scanlines_mode;//bool vertical_2x;

	int full_screen;
	int full_screen_refresh_rate;

	unsigned present_count;

	DDCAPS halcaps;

	//ends with 0.
	int *refresh_rates;

	//this is bumped each time a start fails for a particular buffer size.
	//each time the Start fails, this value is bumped.
	//if failures happen too often, the system will stop trying to start itself until
	//the screen size changes or we go into full screen.
	//
	//we accept a few failures to cope with transient problems. in no way has this
	//anything to do with my having rather lost track of exactly what gets called
	//from where and how many times the init may get run.
	//
	//if we go into full screen, but that fails, the system tries again in windowed
	//mode, so the process restarts. 
	//
	//in the failure situation, 'present' will ignore the buffer and instead draw
	//a message on the target window using standard Win32 functions.
	int num_failed_starts;

	int last_present_w,last_present_h;

	//pointer to real start of the heap block this structure lives in, in
	//case it's different from the start of the structure.
	void *real_address;

	//clip list from the clipper.
	RGNDATA *clip_list;
	DWORD clip_list_size;

	const char **special_effects_names;
};

typedef void (*CopyRect8Fn)(HostGfxState *state,byte *dest,unsigned dest_pitch,const byte *src,int srcy,int width,int num_whole_lines,
	int line_height,int gap_height,unsigned src_pitch);

typedef void (*PresentFn)(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints);

struct HostGfx_SpecialEffect {
	CopyRect8Fn copyrect8;
	PresentFn convert8;
	PresentFn convert16;
	PresentFn convert24;
	PresentFn convert32;
	char *name;
};
typedef struct HostGfx_SpecialEffect HostGfx_SpecialEffect;

extern const HostGfx_SpecialEffect special_effects[];//forward declaration
extern const int num_special_effects;//forward declaration

//////////////////////////////////////////////////////////////////////////

static void GetBitfield(DWORD value,int *shift,int *count) {
	*shift=0;
	*count=0;
	if(value!=0) {
		while(!(value&1)) {
			++*shift;
			value>>=1;
		}
		while(value&1) {
			++*count;
			value>>=1;
		}
	}
}

//static __inline int HostGfx_Shift(HostGfxState *state) {
//	HOST_ASSERT(!state->full_screen);
//	return state->scanlines_mode==HostGfx_DOUBLE?1:0;
//}

static void HostGfx_RefreshPalette(HostGfxState *state) {
	DWORD *masks[3];
	PALETTEENTRY rgb_palette[8];
	DWORD lookup_24bit[8];
	int i;

	masks[0]=&state->primary_desc.ddpfPixelFormat.dwRBitMask;
	masks[1]=&state->primary_desc.ddpfPixelFormat.dwGBitMask;
	masks[2]=&state->primary_desc.ddpfPixelFormat.dwBBitMask;

	for(i=0;i<8;++i) {
		int j;

		state->lookup_32bit[i]=0;
		state->lookup_16bit[i]=0;
		lookup_24bit[i]=0;
		for(j=0;j<3;++j) {
			int shift=0,count=0;
			byte elem;

			GetBitfield(*masks[j],&shift,&count);
			HOST_ASSERT(state->palette[i][j]>=0.f&&state->palette[i][j]<=1.f);

			elem=(byte)(state->palette[i][j]*255.f);

			state->lookup_32bit[i]|=(elem>>(8-count))<<shift;
			lookup_24bit[i]|=(elem>>(8-count))<<shift;
			state->lookup_16bit[i]|=(elem>>(8-count))<<shift;

			state->lookup_24bit[i][j]=(elem>>(8-count))<<shift;
		}

		rgb_palette[i].peRed=(byte)(state->palette[i][0]*255.f);
		rgb_palette[i].peGreen=(byte)(state->palette[i][1]*255.f);
		rgb_palette[i].peBlue=(byte)(state->palette[i][2]*255.f);
		rgb_palette[i].peFlags=PC_NOCOLLAPSE;

		state->lookup_24bit[i][0]=(byte)lookup_24bit[i];
		state->lookup_24bit[i][1]=(byte)(lookup_24bit[i]>>8);
		state->lookup_24bit[i][2]=(byte)(lookup_24bit[i]>>16);
	}

	//8 bit palette is fixed.
	//http://www.compuphase.com/palette.htm
	//I just use 0 for black and 249->255 for the rest -- they're fixed.
	state->lookup_8bit[0]=0;
	state->lookup_8bit[1]=249;
	state->lookup_8bit[2]=250;
	state->lookup_8bit[3]=251;
	state->lookup_8bit[4]=252;
	state->lookup_8bit[5]=253;
	state->lookup_8bit[6]=254;
	state->lookup_8bit[7]=255;

	if(state->primary_palette) {
		IDirectDrawPalette_SetEntries(state->primary_palette,0,HostGfx_FULL_SCREEN_PALETTE_BASE,8,rgb_palette);
	}
}

#define RECT_WIDTH(R) ((R)->right-(R)->left)
#define RECT_HEIGHT(R) ((R)->bottom-(R)->top)

static void GetRectIntersection(RECT *dest,const RECT *r1,const RECT *r2) {
	//IntersectRect(dest,r1,r2);
	int x_disjoint,y_disjoint;

	dest->left=0;
	dest->right=0;
	dest->top=0;
	dest->bottom=0;

	//early outs
	x_disjoint=r1->left>=r2->right||r2->left>=r1->right;
	y_disjoint=r1->top>=r2->bottom||r2->top>=r1->bottom;
	if(x_disjoint||y_disjoint) {
		return;
	}

	//plonk
	dest->left=max(r1->left,r2->left);
	dest->right=min(r1->right,r2->right);
	dest->top=max(r1->top,r2->top);
	dest->bottom=min(r1->bottom,r2->bottom);
}

static void DrawRect(byte *dest_bitmap,unsigned dest_pitch,const RECT *dest_rect,byte outline,int interior) {
	int y;
	byte *dest;

	if(RECT_WIDTH(dest_rect)>0&&RECT_HEIGHT(dest_rect)>0) {
		dest=dest_bitmap+dest_rect->top*dest_pitch+dest_rect->left;

		//draw the box
		for(y=0;y<RECT_HEIGHT(dest_rect);++y) {
			if(y==0||y==RECT_HEIGHT(dest_rect)-1) {
				memset(dest,outline,RECT_WIDTH(dest_rect));
			} else {
				dest[0]=outline;
				if(RECT_WIDTH(dest_rect)>2&&interior>=0) {
					memset(dest+1,interior,RECT_WIDTH(dest_rect)-2);
				}
				dest[RECT_WIDTH(dest_rect)-1]=outline;
			}
			dest+=dest_pitch;
		}
	}
}

//clears the intersection of r1 and r2. dest_bitmap is 8pp.
static void HostGfx_ClearRectIntersection(byte *dest_bitmap,unsigned dest_pitch,const RECT *r1,const RECT *r2) {
	RECT dest_rect;

	GetRectIntersection(&dest_rect,r1,r2);

	if(RECT_WIDTH(&dest_rect)>0&&RECT_HEIGHT(&dest_rect)>0) {
		int y;
		byte *dest;

		dest=dest_bitmap+dest_rect.top*dest_pitch+dest_rect.left;
		for(y=0;y<RECT_HEIGHT(&dest_rect);++y) {
			memset(dest,0,RECT_WIDTH(&dest_rect));
			dest+=dest_pitch;
		}
	}
}

//HostGfx_CopyRect8(state,dest,dest_pitch,src,RECT_WIDTH(dest_rect),num_whole_lines,gap_height,src_pitch);
static void HostGfx_CopyRect8(HostGfxState *state,byte *dest,unsigned dest_pitch,const byte *src,int srcy,int width,int num_whole_lines,
	int line_height,int gap_height,unsigned src_pitch)
{
	int y;

	for(y=0;y<num_whole_lines;++y) {
		int subline;

		for(subline=0;subline<line_height;++subline) {
			memcpy(dest,src,width);
			dest+=dest_pitch;
		}

		//this might pop us over the end of the buffer, but we won't actually write to that,
		//so it's ok.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

//for full screen only. draws directly to front buffer, so must clip manually against the clipper's clip list.
//as an additional complication, the source bitmap is automatically centred horizontally.
//
//TODO take into account present_hints, by clearing areas that are in the clip list but outside the src bitmap.
static void HostGfx_Copy8(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints,CopyRect8Fn copy_rect_fn)
{
	const RGNDATAHEADER *clip_list_hdr;
	DWORD clip_list_size;
	HRESULT hr;
	int src_line_dest_height=line_height+gap_height;
	RECT entire_dest_rect;
	int entire_dest_dx,entire_dest_dy;
	RECT tidy_top,tidy_bottom,tidy_left,tidy_right;

	HOST_ASSERT(y_first<=gap_height);

	entire_dest_dx=(FULL_SCREEN_WIDTH-src_width)/2;
	entire_dest_dy=y_first;

	entire_dest_rect.left=entire_dest_dx;
	entire_dest_rect.top=entire_dest_dy;
	entire_dest_rect.right=entire_dest_rect.left+src_width;
	entire_dest_rect.bottom=entire_dest_rect.top+src_height*src_line_dest_height;

	hr=IDirectDrawClipper_GetClipList(state->primary_clipper,0,0,&clip_list_size);
	if(FAILED(hr)) {
		dprintf(__FUNCTION__ ": IDirectDrawClipper_GetClipList (get clip list size) failed.\n");
		return;
	}

	if(clip_list_size>state->clip_list_size) {
		RGNDATA *new_clip_list=realloc(state->clip_list,clip_list_size);
		if(new_clip_list) {
			dprintf(__FUNCTION__ ": clip list now %u bytes, was %u\n",clip_list_size,state->clip_list_size);
			state->clip_list=new_clip_list;
			state->clip_list_size=clip_list_size;
		}
	}

	//docs are unclear, will this update *lpdwSize? don't want it too (might be larger than current
	//clip list)
	hr=IDirectDrawClipper_GetClipList(state->primary_clipper,0,state->clip_list,&clip_list_size);
	if(FAILED(hr)) {
		dprintf(__FUNCTION__ ": IDirectDrawClipper_GetClipList (get clip list itself) failed.\n");
		return;
	}

	clip_list_hdr=&state->clip_list->rdh;
	if(clip_list_hdr->iType==RDH_RECTANGLES) {
		DWORD i;
		const RECT *rects=(RECT *)state->clip_list->Buffer;

		if(present_hints&HostGfx_PRESENT_HINT_TIDY) {
			//Top (and top left/top right corners)
			tidy_top.left=0;
			tidy_top.right=FULL_SCREEN_WIDTH;
			tidy_top.top=0;
			tidy_top.bottom=entire_dest_rect.top;

			//Bottom (and bottom left/bottom right corners)
			tidy_bottom.left=0;
			tidy_bottom.right=FULL_SCREEN_WIDTH;
			tidy_bottom.top=entire_dest_rect.bottom;
			tidy_bottom.bottom=FULL_SCREEN_HEIGHT;

			//Left
			tidy_left.left=0;
			tidy_left.right=entire_dest_rect.left;
			tidy_left.top=entire_dest_rect.top;
			tidy_left.bottom=entire_dest_rect.bottom;

			//Right
			tidy_right.left=entire_dest_rect.right;
			tidy_right.right=FULL_SCREEN_WIDTH;
			tidy_right.top=entire_dest_rect.top;
			tidy_right.bottom=entire_dest_rect.bottom;
		}

		//		dprintf(__FUNCTION__ ": Gtc=%u nCount=%u\n",GetTickCount(),clip_list_hdr->nCount);
		for(i=0;i<clip_list_hdr->nCount;++i) {
			RECT dest_rect;
			static byte value=0;

			//			DrawRect(dest_bitmap,dest_pitch,&rects[i],2,4);
			GetRectIntersection(&dest_rect,&rects[i],&entire_dest_rect);
			if(dest_rect.right-dest_rect.left>0&&dest_rect.bottom-dest_rect.top>0) {
				int num_whole_lines;
				const byte *src;
				byte *dest;
				int src_dx,src_dy;
				int dest_dx,dest_dy;
				int y;

				//Fill this rect in then.
				//This was going to be more complicated (correct handling of partial lines, etc.),
				//but it ended up dead simple instead.

				//Determine source
				src_dx=dest_rect.left-entire_dest_dx;
				src_dy=(dest_rect.top-entire_dest_dy)/src_line_dest_height;

				src=src_bitmap;
				src+=src_dx;
				src+=src_dy*src_pitch;

				//And dest
				dest_dx=dest_rect.left;
				dest_dy=dest_rect.top;

				dest=dest_bitmap;
				dest+=dest_dx;
				dest+=dest_dy*dest_pitch;

				//this isn't _quite_ right :)
				//I think this is what DirectDraw does anyway, though, judging by the weird artefacts
				//when moving dialogs over a stretched bitmap. However, I'm potentially leaving a few
				//lines unwritten at the bottom of the destination rectangle, which DirectDraw doesn't.
				//Fortunately, only a cosmetic thing.
				num_whole_lines=(dest_rect.bottom-dest_rect.top)/src_line_dest_height;

				(*copy_rect_fn)(state,dest,dest_pitch,src,src_dy,RECT_WIDTH(&dest_rect),num_whole_lines,line_height,gap_height,src_pitch);

				//and tidy the gaps, if there are any.
				if((present_hints&HostGfx_PRESENT_HINT_TIDY)&&gap_height>0) {
					dest=dest_bitmap;
					dest+=dest_dx;
					dest+=dest_dy*dest_pitch;

					for(y=0;y<num_whole_lines;++y) {
						int gapline;

						dest+=line_height*dest_pitch;

						for(gapline=0;gapline<gap_height;++gapline) {
							memset(dest,0,RECT_WIDTH(&dest_rect));
							dest+=dest_pitch;
						}
					}
				}
			}

			//tidy up the edges, if the present hints say so.
			if(present_hints&HostGfx_PRESENT_HINT_TIDY) {
				HostGfx_ClearRectIntersection(dest_bitmap,dest_pitch,&tidy_top,&rects[i]);
				HostGfx_ClearRectIntersection(dest_bitmap,dest_pitch,&tidy_bottom,&rects[i]);
				HostGfx_ClearRectIntersection(dest_bitmap,dest_pitch,&tidy_left,&rects[i]);
				HostGfx_ClearRectIntersection(dest_bitmap,dest_pitch,&tidy_right,&rects[i]);
			}
		}
	}
}

static void HostGfx_Convert8(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const unsigned char *src;
	char *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {

		//Line
		for(i=0;i<line_height;++i) {
			for(x=0;x<src_width;++x) {
				dest[x]=state->lookup_8bit[src[x]];
			}
			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

static void HostGfx_Convert16(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const unsigned char *src;
	char *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {

		//Line
		for(i=0;i<line_height;++i) {
			for(x=0;x<src_width;++x) {
				((word *)dest)[x]=state->lookup_16bit[src[x]];
			}
			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

static void HostGfx_Convert24(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const unsigned char *src;
	char *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {

		//Line
		for(i=0;i<line_height;++i) {
			byte *pixel=dest;

			for(x=0;x<src_width;++x) {
				pixel[0]=state->lookup_24bit[src[x]][0];
				pixel[1]=state->lookup_24bit[src[x]][1];
				pixel[2]=state->lookup_24bit[src[x]][2];

				pixel+=3;
			}
			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

static void HostGfx_Convert32(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const byte *src;
	byte *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {

		//Line
		for(i=0;i<line_height;++i) {
			for(x=0;x<src_width;++x) {
				((dword *)dest)[x]=state->lookup_32bit[src[x]];
			}
			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

static const byte tv_blurmasks[5]={1,2,4,1,2,};

//HostGfx_CopyRect8(state,dest,dest_pitch,src,RECT_WIDTH(dest_rect),num_whole_lines,gap_height,src_pitch);
static void HostGfx_CopyRect8_Tv(HostGfxState *state,byte *dest,unsigned dest_pitch,const byte *src,int srcy,int width,int num_whole_lines,
	int line_height,int gap_height,unsigned src_pitch)
{
	int y;

	for(y=0;y<num_whole_lines;++y) {
		int subline;

		for(subline=0;subline<line_height;++subline) {
			const byte *bms;
			int x;

			bms=&tv_blurmasks[(y+state->present_count)%3];

			dest[0]=HostGfx_FULL_SCREEN_PALETTE_BASE|(src[0]&(bms[0]|bms[1]))|(src[1]&bms[2]);
			for(x=1;x<width-1;++x) {
				dest[x]=HostGfx_FULL_SCREEN_PALETTE_BASE|(src[x-1]&bms[0])|(src[x]&bms[1])|(src[x+1]&bms[2]);
			}
			dest[x]=HostGfx_FULL_SCREEN_PALETTE_BASE|(src[x-1]&bms[0])|(src[x]&(bms[1]|bms[2]));

			dest+=dest_pitch;
		}

		//this might pop us over the end of the buffer, but we won't actually write to that,
		//so it's ok.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}


static void HostGfx_Convert8_Tv(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
							 const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
							 int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const unsigned char *src;
	char *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {
		//Line
		for(i=0;i<line_height;++i) {
			const byte *bms;

			bms=&tv_blurmasks[(y+state->present_count)%3];

			dest[0]=state->lookup_8bit[(src[0]&(bms[0]|bms[1]))|(src[1]&bms[2])];
			for(x=1;x<src_width-1;++x) {
				dest[x]=state->lookup_8bit[(src[x-1]&bms[0])|(src[x]&bms[1])|(src[x+1]&bms[2])];
			}
			dest[x]=state->lookup_8bit[(src[x-1]&bms[0])|(src[x+1]&(bms[1]|bms[2]))];

			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

static void HostGfx_Convert16_Tv(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
							  const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
							  int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const unsigned char *src;
	char *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {
		//Line
		for(i=0;i<line_height;++i) {
			const byte *bms;

			bms=&tv_blurmasks[(y+state->present_count)%3];

			((word *)dest)[0]=state->lookup_16bit[(src[0]&(bms[0]|bms[1]))|(src[1]&bms[2])];
			for(x=1;x<src_width-1;++x) {
				((word *)dest)[x]=state->lookup_16bit[(src[x-1]&bms[0])|(src[x]&bms[1])|(src[x+1]&bms[2])];
			}
			((word *)dest)[x]=state->lookup_16bit[(src[x-1]&bms[0])|(src[x+1]&(bms[1]|bms[2]))];

			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

static void HostGfx_Convert32_Tv(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
							  const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
							  int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const byte *src;
	byte *dest;

	HostGfx_RefreshPalette(state);

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {
		//Line
		for(i=0;i<line_height;++i) {
			const byte *bms;

			bms=&tv_blurmasks[(/*y+*/state->present_count)%3];

			((dword *)dest)[0]=state->lookup_32bit[(src[0]&(bms[0]|bms[1]))|(src[1]&bms[2])];
			for(x=1;x<src_width-1;++x) {
				((dword *)dest)[x]=state->lookup_32bit[(src[x-1]&bms[0])|(src[x]&bms[1])|(src[x+1]&bms[2])];
			}
			((dword *)dest)[x]=state->lookup_32bit[(src[x-1]&bms[0])|(src[x+1]&(bms[1]|bms[2]))];

			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

static void HostGfx_Convert32_Test(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const byte *src;
	byte *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {

		//Line
		for(i=0;i<line_height;++i) {
			for(x=0;x<src_width;++x) {
				((dword *)dest)[x]=state->lookup_32bit[src[x]];
			}
			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}

static void HostGfx_Convert16_Test(HostGfxState *state,byte *dest_bitmap,unsigned dest_pitch,
	const byte *src_bitmap,int src_width,unsigned src_pitch,int src_height,
	int y_first,int line_height,int gap_height,unsigned present_hints)
{
	int num_lines;
	int y,i,x;
	const unsigned char *src;
	char *dest;

	dest=(char *)dest_bitmap+y_first*dest_pitch;
	src=src_bitmap;

	num_lines=src_height*(line_height+gap_height);
	for(y=0;y<src_height;++y) {

		//Line
		for(i=0;i<line_height;++i) {
			for(x=0;x<src_width;++x) {
				((word *)dest)[x]=state->lookup_16bit[src[x]];
			}
			dest+=dest_pitch;
		}

		//TODO no tidy in windowed mode yet.
		dest+=gap_height*dest_pitch;

		src+=src_pitch;
	}
}



//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////


static IDirectDrawSurface2 *NewSurface2(HostGfxState *state,DDSURFACEDESC *desc) {
	HRESULT hr;
	IDirectDrawSurface *surface;
	IDirectDrawSurface2 *surface2;
	DDBLTFX fx;

	hr=DXCHK(IDirectDraw2_CreateSurface(state->ddraw,desc,&surface,0));
	if(FAILED(hr)) {
		return 0;
	}

	hr=DXCHK(IDirectDrawSurface_QueryInterface(surface,&IID_IDirectDrawSurface2,&surface2));
	IDirectDrawSurface_Release(surface);
	if(FAILED(hr)) {
		return 0;
	}

	if(!(desc->ddsCaps.dwCaps&DDSCAPS_PRIMARYSURFACE)) {
		memset(&fx,0,sizeof fx);
		fx.dwSize=sizeof fx;
		fx.dwFillColor=0;
		DXCHK(IDirectDrawSurface2_Blt(surface2,0,0,0,DDBLT_COLORFILL|DDBLT_WAIT,&fx));
	}

	return surface2;
}

static void HostGfx_StopInternal(HostGfxState *state) {
	unsigned i;

	HOST_ASSERT(state);

	if(state->ddraw) {
		if(state->full_screen) {
			IDirectDraw_RestoreDisplayMode(state->ddraw);
		}
		IDirectDraw_SetCooperativeLevel(state->ddraw,0/*state->owner*/,DDSCL_NORMAL);
	}

	dprintf("HostGfx_StopInternal: Release: primary_clipper=0x%p\n",
		state->primary_clipper);
	SAFE_RELEASE(state->primary_clipper);

	dprintf("HostGfx_StopInternal: Release: primary_palette=0x%p\n",
		state->primary_palette);
	SAFE_RELEASE(state->primary_palette);

	for(i=0;i<max_num_back_buffers;++i) {
		dprintf("HostGfx_StopInternal: Release: back_buffers[%u]=0x%p\n",
			i,state->back_buffers[i]);
		SAFE_RELEASE(state->back_buffers[i]);
	}

	dprintf("HostGfx_StopInternal: Release: primary=0x%p\n",state->primary);
	SAFE_RELEASE(state->primary);

	dprintf("HostGfx_StopInternal: Release: ddraw=0x%p\n",state->ddraw);
	SAFE_RELEASE(state->ddraw);

	state->num_back_buffers=0;
	state->buffers_ok=0;
}

static int HostGfx_CreateClipper(HostGfxState *state) {//Clipper
	HRESULT hr;

	dprintf("HostGfx_CreateClipper: CreateClipper.\n");
	hr=IDirectDraw2_CreateClipper(state->ddraw,0,&state->primary_clipper,0);
	if(FAILED(hr)) {
		return 0;
	}

	dprintf("HostGfx_CreateClipper: Set clipper window.\n");
	hr=IDirectDrawClipper_SetHWnd(state->primary_clipper,0,state->target);
	if(FAILED(hr)) {
		return 0;
	}

	dprintf("HostGfx_CreateClipper: Set clipper on primary surface.\n");
	hr=IDirectDrawSurface2_SetClipper(state->primary,state->primary_clipper);
	if(FAILED(hr)) {
		return 0;
	}

	return 1;
}

static HRESULT HostGfx_StartWindowedInternal(HostGfxState *state,int w,int h) {
	DDSURFACEDESC sd;
	HRESULT hr;
	unsigned i;

	dprintf("HostGfx_StartWindowedInternal: SetCooperativeLevel.\n");
	hr=IDirectDraw2_SetCooperativeLevel(state->ddraw,state->owner,DDSCL_NORMAL);
	if(FAILED(hr)) {
		return hr;
	}

	//Primary surface and its description
	memset(&sd,0,sizeof sd);
	sd.dwSize=sizeof sd;
	sd.dwFlags=DDSD_CAPS;
	sd.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE;
	dprintf("HostGfx_StartInternal: Create primary surface.\n");
	state->primary=NewSurface2(state,&sd);
	if(!state->primary) {
		return E_FAIL;
	}
	memset(&state->primary_desc,0,sizeof state->primary_desc);
	state->primary_desc.dwSize=sizeof state->primary_desc;
	DXCHK(IDirectDrawSurface2_GetSurfaceDesc(state->primary,&state->primary_desc));

	//Back surfaces, if wanted.
	state->num_back_buffers=0;
	if(w<0||h<0) {
		dprintf("HostGfx_StartWindowedInternal: invalid dimensions, not creating back buffers.\n");
	} else {
		for(i=0;i<max_num_back_buffers;++i) {
			memset(&sd,0,sizeof sd);
			sd.dwSize=sizeof sd;
			sd.dwFlags=DDSD_CAPS|DDSD_WIDTH|DDSD_HEIGHT;
			sd.ddsCaps.dwCaps=DDSCAPS_OFFSCREENPLAIN;
			if(state->halcaps.dwCaps&DDCAPS_BLT) {
				sd.ddsCaps.dwCaps|=DDSCAPS_VIDEOMEMORY;
			} else {
				sd.ddsCaps.dwCaps|=DDSCAPS_SYSTEMMEMORY;
			}
			sd.dwWidth=w;
			sd.dwHeight=h;//<<HostGfx_Shift(state);
			dprintf("HostGfx_StartWindowedInternal: Create %ux%u back buffer %u/%u.\n",
				sd.dwWidth,sd.dwHeight,1+i,max_num_back_buffers);
			state->back_buffers[i]=NewSurface2(state,&sd);
			if(!state->back_buffers[i]) {
				dprintf("HostGfx_StartWindowedInternal:\tthis one failed\n");
				if(state->num_back_buffers==0) {
					dprintf("HostGfx_StartWindowedInternal:\tno back buffers created, failed overall.\n");
					hr=E_FAIL;
				}
				break;
			}
			++state->num_back_buffers;
		}
		dprintf("HostGfx_StartWindowedInternal: created %d back buffers\n",
			state->num_back_buffers);
	}

	if(!HostGfx_CreateClipper(state)) {
		goto stop_error;
	}

	dprintf("HostGfx_StartWindowedInternal: ok.\n");

	return S_OK;

stop_error:
	dprintf("HostGfx_StartWindowedInternal: failed: %s.\n",DXGetErrorString(hr));
	SAFE_RELEASE(state->primary_clipper);
	for(i=0;i<max_num_back_buffers;++i) {
		SAFE_RELEASE(state->back_buffers[i]);
	}
	SAFE_RELEASE(state->primary);
	return E_FAIL;
}

static HRESULT HostGfx_StartFullScreenInternal(HostGfxState *state,int w,int h) {
	DDSURFACEDESC sd;
	HRESULT hr;
	PALETTEENTRY palette[256];
	//	unsigned i;
	//DDSCAPS dscaps;
	DDBLTFX bltfx;
	HDC screen_dc;
	DWORD coop;

	//Set video mode
	dprintf("HostGfx_StartFullScreenInternal: SetCooperativeLevel.\n");
	coop=DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN;
	hr=IDirectDraw_SetCooperativeLevel(state->ddraw,state->owner,coop);
	if(FAILED(hr)) {
		goto stop_error;
	}
	dprintf("HostGfx_StartFullScreenInternal: SetDisplayMode, %dHz.\n",
		state->full_screen_refresh_rate);
	hr=IDirectDraw2_SetDisplayMode(state->ddraw,FULL_SCREEN_WIDTH,FULL_SCREEN_HEIGHT,8,
		state->full_screen_refresh_rate,0);
	if(FAILED(hr)) {
		goto stop_error;
	}

	//Primary surface and its description.
	//I want to avoid catering for flipping etc., so I don't bother
	memset(&sd,0,sizeof sd);
	sd.dwSize=sizeof sd;
	sd.dwFlags=DDSD_CAPS|DDSD_BACKBUFFERCOUNT;
	sd.dwBackBufferCount=1;
	sd.ddsCaps.dwCaps=DDSCAPS_PRIMARYSURFACE|DDSCAPS_COMPLEX|DDSCAPS_FLIP;
	dprintf("HostGfx_StartFullScreenInternal: Create primary surface.\n");
	state->primary=NewSurface2(state,&sd);
	if(!state->primary) {
		hr=E_FAIL;
		goto stop_error;
	}

	//Set that as the front buffer.
	state->back_buffers[0]=state->primary;
	IDirectDrawSurface2_AddRef(state->back_buffers[0]);

	//Clear the buffer that's going to be used.
	memset(&bltfx,0,sizeof bltfx);
	bltfx.dwSize=sizeof bltfx;
	bltfx.dwFillColor=0;
	DXCHK(IDirectDrawSurface2_Blt(state->back_buffers[0],0,0,0,
		DDBLT_COLORFILL|DDBLT_WAIT,&bltfx));

	//Here the palette is retrieved, and 8 entries reset for use by the emulator,
	//then set as the current palette. The hope is that the GDI can draw onto the
	//primary surface, allowing use of normal Windows menu bar and status bar if
	//the user wants this.

	//This seems to be the best way to get the system palette.
	//http://groups.google.com/groups?q=default+palette+directdraw&hl=en&lr=&ie=UTF-8&oe=UTF-8&selm=38F675F48B36D4118E4D00508B5EBA37BE6BBB%40cpmsftmsgv21.microsoft.com&rnum=10
	//(was the first useful one from google)
	dprintf("HostGfx_StartFullScreenInternal: determining default palette\n");
	dprintf("HostGfx_StartFullScreenInternal:     IDirectDrawSurface2_GetDC...\n");
	hr=IDirectDrawSurface2_GetDC(state->primary,&screen_dc);
	if(FAILED(hr)) {
		goto stop_error;
	}
	dprintf("HostGfx_StartFullScreenInternal:     GetSystemPaletteEntries...\n");
	if(GetSystemPaletteEntries(screen_dc,0,256,palette)!=256) {
		hr=E_FAIL;
		//but the DC should still be released.
	}
	IDirectDrawSurface2_ReleaseDC(state->primary,screen_dc);
	//ShowPalette(stdout,"system palette",palette);
	if(FAILED(hr)) {
		goto stop_error;
	}

	//	//Set the 8 entries that the emulator uses.
	//	for(i=0;i<8;++i) {
	//		PALETTEENTRY *pe=&palette[HostGfx_FULL_SCREEN_PALETTE_BASE+i];
	//		pe->peRed=i&1?255:0;
	//		pe->peGreen=i&2?255:0;
	//		pe->peBlue=i&4?255:0;
	//		pe->peFlags=PC_NOCOLLAPSE;
	//	}
	hr=IDirectDraw_CreatePalette(state->ddraw,DDPCAPS_8BIT|DDPCAPS_ALLOW256,palette,
		&state->primary_palette,0);
	if(FAILED(hr)) {
		goto stop_error;
	}

	//It's not clear what to do if this should fail. So hope it doesn't.
	IDirectDrawSurface_SetPalette(state->primary,state->primary_palette);

	HostGfx_RefreshPalette(state);

	if(!HostGfx_CreateClipper(state)) {
		goto stop_error;
	}

	dprintf("HostGfx_StartFullScreenInternal: ok.\n");
	return S_OK;

stop_error:
	dprintf("HostGfx_StartFullScreenInternal: fail: %s\n",DXGetErrorString(hr));
	IDirectDraw2_RestoreDisplayMode(state->ddraw);
	IDirectDraw2_SetCooperativeLevel(state->ddraw,state->owner,DDSCL_NORMAL);
	SAFE_RELEASE(state->primary_palette);
	SAFE_RELEASE(state->back_buffers[0]);
	SAFE_RELEASE(state->primary);
	return hr;
}

static HRESULT FAR PASCAL StoreRefreshRatesEnumModesCallback(LPDDSURFACEDESC lpDDSurfaceDesc,
															 LPVOID lpContext)
{
	HostGfxState *state=lpContext;
	DWORD mask=DDSD_WIDTH|DDSD_HEIGHT|DDSD_REFRESHRATE;

	if((lpDDSurfaceDesc->dwFlags&mask)==mask) {
		if(lpDDSurfaceDesc->dwWidth==FULL_SCREEN_WIDTH&&lpDDSurfaceDesc->dwHeight==FULL_SCREEN_HEIGHT) {
			int rate=lpDDSurfaceDesc->dwRefreshRate;
			if(rate!=0) {
				int num=0;
				int add=1;

				if(state->refresh_rates) {
					for(num=0;state->refresh_rates[num]!=0;++num) {
						if(state->refresh_rates[num]==rate) {
							add=0;
							break;
						}
					}
				}

				if(add) {
					int *p=realloc(state->refresh_rates,(num+2)*sizeof *state->refresh_rates);
					if(p) {
						state->refresh_rates=p;
						state->refresh_rates[num]=rate;
						state->refresh_rates[num+1]=0;
					}
				}
			}
		}
	}

	return DDENUMRET_OK;
}

static int HostGfx_StartInternal(HostGfxState *state,int w,int h) {
	IDirectDraw *ddraw;
	HRESULT hr;

	state->present_count=0;

	dprintf("HostGfx_StartInternal: w=%d h=%d\n",w,h);

	//DirectDraw object
	dprintf("HostGfx_StartInternal: starting DirectDraw.\n");
	hr=DirectDrawCreate(0,&ddraw,0);
	if(FAILED(hr)) {
		IDirectDraw_Release(ddraw);
		goto stop_error;
	}
	dprintf("HostGfx_StartInternal: QueryInterface for IDirectDraw2.\n");
	hr=IDirectDraw_QueryInterface(ddraw,&IID_IDirectDraw2,&state->ddraw);
	IDirectDraw_Release(ddraw);
	if(FAILED(hr)) {
		goto stop_error;
	}

	dprintf("HostGfx_StartInternal: GetCaps.\n");
	memset(&state->halcaps,0,sizeof state->halcaps);
	state->halcaps.dwSize=sizeof state->halcaps;
	hr=IDirectDraw2_GetCaps(state->ddraw,&state->halcaps,0);
	if(FAILED(hr)) {
		goto stop_error;
	}

	if(state->full_screen) {
		hr=HostGfx_StartFullScreenInternal(state,w,h);
		if(FAILED(hr)) {
			//HostGfx_StopInternal(state);
			state->full_screen=0;
		}
	}

	if(!state->full_screen) {
		hr=HostGfx_StartWindowedInternal(state,w,h);
	}
	if(FAILED(hr)) {
		goto stop_error;
	}

	if(state->back_buffers[0]) {
		memset(&state->back_buffer_desc,0,sizeof state->back_buffer_desc);
		state->back_buffer_desc.dwSize=sizeof state->back_buffer_desc;
		IDirectDrawSurface2_GetSurfaceDesc(state->back_buffers[0],
			&state->back_buffer_desc);
		state->buffers_ok=1;
		state->cpu_buffer=0;
		state->gpu_buffer=state->num_back_buffers-1;//num_back_buffers-1;
	}

	HostGfx_RefreshPalette(state);

	hr=IDirectDraw2_EnumDisplayModes(state->ddraw,DDEDM_REFRESHRATES,0,state,
		&StoreRefreshRatesEnumModesCallback);
	if(FAILED(hr)) {
		free(state->refresh_rates);
		state->refresh_rates=0;
	}

	//	HostGfx_SetScreenType(state,HostGfx_COLOUR);
	//	HostGfx_SetScanlinesMode(state,HostGfx_SINGLE);

	return 1;

stop_error:
	dprintf("HostGfx_StartInternal: fail: %s\n",DXGetErrorString(hr));
	HostGfx_StopInternal(state);
	return 0;
}

HostGfxState *HostGfx_Start(HWND owner,HWND target) {
	HostGfxState *new_state;
	unsigned i;
#ifdef HOSTGFX_STATE_ALIGNMENT_BITS
	{
		unsigned aligned;
		void *real_address;

		real_address=malloc(sizeof *new_state+HOSTGFX_STATE_ALIGNMENT_BYTES-1);
		aligned=(unsigned)real_address;
		aligned+=HOSTGFX_STATE_ALIGNMENT_BYTES-1;
		aligned&=~(HOSTGFX_STATE_ALIGNMENT_BYTES-1);
		new_state=(HostGfxState *)aligned;
		new_state->real_address=real_address;
	}
#else
	new_state=malloc(sizeof *new_state);//HostGfxState;
	new_state->real_address=new_state;
#endif

	//	dprintf("HostGfx_Start: owner window \"%s\"",owner->GetTitle().c_str());
	new_state->owner=owner;
	new_state->target=target;
	new_state->ddraw=0;
	new_state->primary=0;
	for(i=0;i<max_num_back_buffers;++i) {
		new_state->back_buffers[i]=0;
	}
	new_state->num_back_buffers=0;
	new_state->primary_clipper=0;
	new_state->primary_palette=0;
	new_state->buffers_ok=0;
	new_state->full_screen=0;
	new_state->full_screen_refresh_rate=0;

	for(i=0;i<8;++i) {
		new_state->palette[i][0]=(float)(i&1);
		new_state->palette[i][1]=(float)((i>>1)&1);
		new_state->palette[i][2]=(float)((i>>2)&1);
	}

#ifdef PALETTE_ALIGNMENT_BITS
	{
		unsigned size8,size16,size32,size,base;

		size8=PALETTE_ALIGN(8*sizeof new_state->lookup_8bit[0]);
		size16=PALETTE_ALIGN(8*sizeof new_state->lookup_16bit[0]);
		size32=PALETTE_ALIGN(8*sizeof new_state->lookup_32bit[0]);

		size=size8+size16+size32+PALETTE_ALIGNMENT_BYTES-1;

		new_state->palette_data=malloc(size);
		base=PALETTE_ALIGN((unsigned)new_state->palette_data);
		new_state->lookup_8bit=(byte *)base;
		new_state->lookup_16bit=(word *)(base+size8);
		new_state->lookup_32bit=(dword *)(base+size8+size16);

		HOST_ASSERT((char *)&new_state->lookup_32bit[8]<=new_state->palette_data+size);
	}
#endif

	new_state->refresh_rates=0;

	new_state->num_failed_starts=0;

	new_state->clip_list=0;
	new_state->clip_list_size=0;

	if(!HostGfx_StartInternal(new_state,-1,-1)) {
		free(new_state);//TODO this should be HostGfx_Stop...
		return 0;
	}

	new_state->special_effects_names=malloc((num_special_effects+1)*sizeof(const char *));
	for(i=0;i<(unsigned)num_special_effects;++i) {
		new_state->special_effects_names[i]=special_effects[i].name;
	}
	new_state->special_effects_names[i]=0;

	dprintf("HostGfx_Start: ok.\n");
	return new_state;
}

void HostGfx_Stop(HostGfxState *state) {
	if(state) {
		HostGfx_StopInternal(state);

		free(state->clip_list);
		free(state->refresh_rates);

#ifdef PALETTE_ALIGNMENT_BITS
		free(state->palette_data);
#endif
		free((void *)state->special_effects_names);//complains about const pointer without the cast

		//must be last.
		free(state->real_address);
	}
}

static void HostGfx_DrawStartFailedMsg(HostGfxState *state) {
	static const char *msg="Unable to start graphics.";
	HDC dc;

	dc=GetDC(state->target);
	if(dc) {
		int index;
		RECT client;

		index=SaveDC(dc);

		GetClientRect(state->target,&client);

		FillRect(dc,&client,GetStockBrush(BLACK_BRUSH));

		SetBkMode(dc,TRANSPARENT);
		SetTextColor(dc,RGB(255,255,255));
		SetTextAlign(dc,TA_CENTER|TA_BASELINE);
		TextOut(dc,client.right/2,client.bottom/2,msg,strlen(msg));

		RestoreDC(dc,index);
		ReleaseDC(state->target,dc);
	}
}

//TODO this logic is becoming twisted, all this asynchronous setting up business is simply
//far too much!
void HostGfx_Present(HostGfxState *state,int effect_index,const unsigned char *bitmap_3bit,int bitmap_w,int bitmap_h,
	unsigned pitch,int y_first,int line_height,int gap_height,unsigned present_hints)
{
	DDSURFACEDESC sd;
	HRESULT hr;
	int right_width,right_height;
	int h,w;
	DWORD lock_flags;
	const HostGfx_SpecialEffect *effect;

	lock_flags=DDLOCK_WRITEONLY|DDLOCK_WAIT|DDLOCK_SURFACEMEMORYPTR;
#if (defined HOST_DEBUG)&&!(defined DX3_ONLY)
	lock_flags|=DDLOCK_NOSYSLOCK;
#endif


	HOST_ASSERT(y_first<=gap_height);
	w=bitmap_w;
	h=bitmap_h*(line_height+gap_height);
	//if we're confronted with something different from last time, that resets
	//our failed start count.
	if(w!=state->last_present_w||h!=state->last_present_h) {
		state->last_present_w=w;
		state->last_present_h=h;
		state->num_failed_starts=0;
	}

	//if we failed to start too many times, draw a message, and don't
	//bother trying again, yet.
	if(!state->buffers_ok&&state->num_failed_starts>=MAX_FAILED_STARTS) {
		HostGfx_DrawStartFailedMsg(state);
		return;
	}

	//we may need to recreate the buffers because we're being asked to display
	//a differently-sized buffer in windowed mode.
	right_width=1;
	right_height=1;
	if(state->buffers_ok) {
		if(!state->full_screen) {
			right_width=w==(int)state->back_buffer_desc.dwWidth;
			right_height=h==(int)state->back_buffer_desc.dwHeight;
		}
	}

	if(!state->buffers_ok||!right_width||!right_height) {
		dprintf("HostGfx_Present: settings change, restarting DirectDraw.\n");
		HostGfx_StopInternal(state);
		HostGfx_StartInternal(state,w,h);
		//Anything might have happened, so drop out without doing anything so
		//that next time Present is called the emulator can check if full screen
		//succeeded. Also bump the number of failed starts if that didn't work.
		if(!state->buffers_ok) {
			++state->num_failed_starts;
		}
		return;
	}

	if(present_hints&HostGfx_PRESENT_HINT_SYNC) {
		IDirectDraw_WaitForVerticalBlank(state->ddraw,DDWAITVB_BLOCKBEGIN,0);
	}

	state->num_failed_starts=0;

	memset(&sd,0,sizeof sd);
	sd.dwSize=sizeof sd;
	hr=IDirectDrawSurface2_Lock(state->back_buffers[state->cpu_buffer],0,&sd,lock_flags,0);
	if(FAILED(hr)) {
		state->buffers_ok=0;
		return;
	}

	if(effect_index<1||effect_index>=num_special_effects) {
		effect=&special_effects[0];
	} else {
		effect=&special_effects[effect_index];
	}

	if(state->full_screen) {
		HostGfx_Copy8(state,sd.lpSurface,sd.lPitch,bitmap_3bit,bitmap_w,pitch,bitmap_h,y_first,line_height,gap_height,present_hints,
			effect->copyrect8);
	} else {
		//		bbcVideo::SetPixelFixedBits(0);
		switch(state->primary_desc.ddpfPixelFormat.dwRGBBitCount) {
		case 8:
			(*effect->convert8)(state,sd.lpSurface,sd.lPitch,bitmap_3bit,bitmap_w,pitch,bitmap_h,y_first,line_height,gap_height,present_hints);
			break;
		case 15:
		case 16:
			(*effect->convert16)(state,sd.lpSurface,sd.lPitch,bitmap_3bit,bitmap_w,pitch,bitmap_h,y_first,line_height,gap_height,present_hints);
			break;
		case 24:
			(*effect->convert24)(state,sd.lpSurface,sd.lPitch,bitmap_3bit,bitmap_w,pitch,bitmap_h,y_first,line_height,gap_height,present_hints);
			break;
		case 32:
			(*effect->convert32)(state,sd.lpSurface,sd.lPitch,bitmap_3bit,bitmap_w,pitch,bitmap_h,y_first,line_height,gap_height,present_hints);
			break;
		}
	}
	IDirectDrawSurface2_Unlock(state->back_buffers[state->cpu_buffer],0);

	++state->present_count;

	if(state->full_screen) {
		//IDirectDrawSurface_Flip(state->primary,0,DDFLIP_WAIT);
		//		ValidateRect(state->owner,0);
	} else {
		POINT top_left={0,0};
		RECT dest;

		state->gpu_buffer=(state->gpu_buffer+1)%state->num_back_buffers;
		state->cpu_buffer=(state->cpu_buffer+1)%state->num_back_buffers;
		//this was previously HostGfx_ShowWindowed.
		//		if(!state->buffers_ok) {
		//			return;
		//		}
		//		//HostGfx_ShowWindowed called if cfg_.video.full_screen, which doesn't imply
		//		//state->full_screen or indeed state->buffers_ok, so drop out if any of the
		//		//required surfaces are currently unavailable.
		//		if(!state->primary) {
		//			return;
		//		}
		//		if(!state->back_buffers[state->gpu_buffer]) {
		//			return;
		//		}
		GetClientRect(state->target,&dest);
		if(dest.right>0&&dest.bottom>0) {
			ClientToScreen(state->target,&top_left);
			OffsetRect(&dest,top_left.x,top_left.y);
			IDirectDrawSurface7_Blt(state->primary,&dest,state->back_buffers[state->gpu_buffer],0,DDBLT_WAIT,0);
		}
	}
	ValidateRect(state->target,0);
}

void HostGfx_SetPalette(HostGfxState *state,const float new_palette[8][3]) {
	memmove(&state->palette,new_palette,sizeof state->palette);
	HostGfx_RefreshPalette(state);
}

//void HostGfx_SetScanlinesMode(HostGfxState *state,
//	HostGfx_ScanlinesMode new_scanlines_mode)
//{
//	if(state->scanlines_mode!=new_scanlines_mode) {
//		state->buffers_ok=0;
//	}
//	state->scanlines_mode=new_scanlines_mode;
//}
//
void HostGfx_SetFullScreen(HostGfxState *state,int full_screen,
						   int full_screen_refresh_rate)
{
#ifdef mbENABLE_FULL_SCREEN
	int restart=0;

	if(full_screen!=state->full_screen||
		(state->full_screen&&
		full_screen_refresh_rate!=state->full_screen_refresh_rate))
	{
		HostGfx_StopInternal(state);
		restart=1;
	}
	state->full_screen=full_screen;
	if(state->full_screen) {
		state->full_screen_refresh_rate=full_screen_refresh_rate;
	}
	if(restart) {
		HostGfx_StartInternal(state,-1,-1);
	}
#else
	state->full_screen=0;
#endif
}

int HostGfx_IsFullScreen(HostGfxState *state) {
	return state->buffers_ok&&state->full_screen;
}

void HostGfx_Cls(HostGfxState *state) {
	if(state->full_screen) {
		DDBLTFX fx;

		memset(&fx,0,sizeof fx);
		fx.dwSize=sizeof fx;
		fx.dwFillColor=0;
		IDirectDrawSurface2_Blt(state->primary,0,0,0,DDBLT_COLORFILL|DDBLT_WAIT,&fx);
	}
}

const int *HostGfx_FullScreenRefreshRates(HostGfxState *state) {
	if(state->refresh_rates) {
		return state->refresh_rates;
	} else {
		return &empty_refresh_rate_list;
	}
}

const char **HostGfx_SpecialEffects(HostGfxState *state) {
	return state->special_effects_names;
}

const HostGfx_SpecialEffect special_effects[]={
	{
		&HostGfx_CopyRect8,
		&HostGfx_Convert8,
		&HostGfx_Convert16,
		&HostGfx_Convert24,
		&HostGfx_Convert32,
		"No effect",
	},
//	{
//		&HostGfx_CopyRect8_Tv,
//		&HostGfx_Convert8_Tv,
//		&HostGfx_Convert16_Tv,
//		&HostGfx_Convert24,
//		&HostGfx_Convert32_Tv,
//		"TV effect",
//	},
//#ifdef HOST_DEBUG
//	{
//		&HostGfx_CopyRect8,
//		&HostGfx_Convert8,
//		&HostGfx_Convert16_Test,
//		&HostGfx_Convert24,
//		&HostGfx_Convert32_Test,
//		"Test effect",
//	},
//#endif
};
const int num_special_effects=sizeof special_effects/sizeof special_effects[0];
