#ifndef HOSTGFX_H_
#define HOSTGFX_H_

#ifdef __cplusplus
extern "C" {
#endif

//////////////////////////////////////////////////////////////////////////
// Graphics
struct HostGfxState;
typedef struct HostGfxState HostGfxState;

//Create a HostGfxState that can be used to draw into the client area of
//the specified window.
HostGfxState *HostGfx_Start(HWND owner,HWND target);

//Destroy the given HostGfxState.
void HostGfx_Stop(HostGfxState *state);

//Set new display contents. Bitmap is 3 bit FFFFFBGR format. FFFFF are
//the fixed bits that you know are suitable for the current display.
//
//Pops up on screen ASAP.
//Should clip against anything on top of the window, if required.
//
//y_first is Y coordinate of first scanline.
//line_height is height of each source line. data duplicated on each.
//gap_height is height of gap (if any) between source line.
//present_hints are hints about what you want.
//
//y_first must be <= gap_height. it can move lines only into the gaps.
//
//effect is index into special effects array.
enum HostGfx_PresentHints {
	//If set, the emulator wants a tidy display. If not present,
	//optimize for speed.
	HostGfx_PRESENT_HINT_TIDY=1,

	//If set, emulator wants present to sync to vsync, if this is possible.
	HostGfx_PRESENT_HINT_SYNC=2,
};
void HostGfx_Present(HostGfxState *state,int effect_index,const unsigned char *bitmap_3bit,int w,int h,
					 unsigned pitch,int y_first,int line_height,int gap_height,unsigned present_hints);

//Sets the palette. Each palette entry is a float triplet. 1.f is full on that colour,
//0.f is none, or anything inbetween. Of course values may end up quantized, but that's
//not your problem.
void HostGfx_SetPalette(HostGfxState *state,const float new_palette[8][3]);//RGB

//Request full screen mode. This may or may not succeed, possibly immediately or maybe
//some time in the future. Check HostGfx_IsFullScreen to see what's going on.
void HostGfx_SetFullScreen(HostGfxState *state,int full_screen,
						   int full_screen_refresh_rate);

//Get list of refresh rates. This always ends with 0, indicating a default rate.
const int *HostGfx_FullScreenRefreshRates(HostGfxState *state);

//Whether full screen or not.
int HostGfx_IsFullScreen(HostGfxState *state);

//Bits 2-0 of this value should be clear.
//It is the base of the 8 palette entries the emulator will use.
//This way the system palette can be left as-is, allowing the display
//of system widgets in full screen mode.
enum HostGfx_Config {
	HostGfx_FULL_SCREEN_PALETTE_BASE=128,
};
typedef enum HostGfx_Config HostGfx_Config;

//Clear the screen.
void HostGfx_Cls(HostGfxState *state);

//Retrieve list of names of special effects.
//null pointer indicates end of list.
//always returns at least one name.
const char **HostGfx_SpecialEffects(HostGfxState *state);

#ifdef __cplusplus
}
#endif

#endif
