#ifndef HOST_INTERNAL_H_
#define HOST_INTERNAL_H_

#define CINTERFACE
#include <unknwn.h>

//Whether to use the dxerr lib to get text descriptions of errors.
#define USE_DXERR

#ifdef USE_DXERR
# ifdef DX9_SDK
#  include <dxerr9.h>
#  define DXGetErrorString(N) DXGetErrorString9(N)
#  pragma comment(lib,"dxerr9.lib")
# else
#  include <dxerr8.h>
#  define DXGetErrorString(N) DXGetErrorString8(N)
#  pragma comment(lib,"dxerr8.lib")
# endif
#else
# define DXGetErrorString(N) ("DX9_ERR disabled -- no description available")
#endif

#ifdef HOST_DEBUG
#define HOST_ASSERT(X)\
	do {\
		if(!(X)) {\
			__asm {\
				int 3\
			}\
		}\
	} while(0)
#else
#define HOST_ASSERT(X)
#endif

#define SAFE_RELEASE(V)\
	do {\
		if(V) {\
			(V)->lpVtbl->Release(V);\
			(V)=0;\
		}\
	} while(0)

#define DXCHK(X) RealDXCHK(X,#X,__FILE__,__LINE__,__FUNCTION__)

//static void *offsetwith(void *base,int offset) {
//	return (char *)base+offset;
//}
#define offsetwith(base,offset) ((void *)(((char *)(base))+(offset)))
typedef signed char int8;
typedef unsigned char byte;

typedef signed short int16;
typedef unsigned short word;

typedef signed long int32;
typedef unsigned long dword;

HRESULT RealDXCHK(HRESULT hr,const char *what,const char *file,unsigned line,
	const char *function);

int dprintf(const char *fmt,...);

#endif