import sys
import xml.sax
import textwrap

def print_list_entry(indent,first_line_prefix,lines,outf):
    for i in range(len(lines)):
        output=indent
        
        if i==0:
            output+=first_line_prefix
        else:
            output+=len(first_line_prefix)*" "

        output+=" "
        output+=lines[i]

        print >>outf,output        

class UnorderedList:
    def __init__(self,outf):
        self.outf=outf
        
    def print_item(self,indent,lines):
        print_list_entry(indent,"*",lines,self.outf)

class OrderedList:
    def __init__(self,outf):
        self.num=1
        self.outf=outf

    def print_item(self,indent,lines):
        print_list_entry(indent,str(self.num)+".",lines,self.outf)
        self.num+=1
        
class Handler(xml.sax.handler.ContentHandler):
    def __init__(self,outf):
        xml.sax.handler.ContentHandler.__init__(self)
        self.contents=[]
        self.left_margin=0
        self.right_margin=74
        self.sec_indent=2
        self.list_handlers_stack=[]

        self.outf=outf

##    def start_xml(self,attrs):
##        pass
##    def end_xml(self):
##        pass

    def start_item(self,attrs):
        self.contents.append("")
    def end_item(self):
        lines=textwrap.wrap(self.contents[-1],self.right_margin-self.left_margin-2)
        self.list_handlers_stack[-1].print_item(self.indent(),lines)
        del self.contents[-1]

    def list_ending(self):
        del self.list_handlers_stack[-1]
        print >>self.outf

    def start_numlist(self,attrs):
        self.list_handlers_stack.append(OrderedList(self.outf))
    def end_numlist(self):
        self.list_ending()
    
    def start_list(self,attrs):
        self.list_handlers_stack.append(UnorderedList(self.outf))
    def end_list(self):
        self.list_ending()

    def start_p(self,attrs):
        self.contents.append("")
    def end_p(self):
        l=self.contents[-1].replace("\t","").strip()
        lines=textwrap.wrap(l,self.right_margin-self.left_margin)
        for line in lines:
            print >>self.outf,self.indent()+line
        print >>self.outf
        del self.contents[-1]

    def indent(self):
        return self.left_margin*" "
    
    def start_sec(self,attrs):
        attrs_names_lc=[attr.lower() for attr in attrs.getNames()]
        for name in attrs.getNames():
            if name.lower().startswith("title"):
                print >>self.outf,self.indent()+attrs.getValue(name)
        print >>self.outf
        self.left_margin+=self.sec_indent
    def end_sec(self):
        self.left_margin-=self.sec_indent
        assert self.left_margin>=0
    
    def startElement(self,name,attrs):
        try:
            getattr(self,"start_"+name.lower())(attrs)
        except AttributeError:
            pass

    def endElement(self,name):
        try:
            getattr(self,"end_"+name.lower())()
        except AttributeError:
            pass

    def characters(self,content):
        if len(self.contents)>0:
            self.contents[-1]+=content

def fmt(input_name,output_name):
    outf=open(output_name,"wt")
    
    parser=xml.sax.make_parser()
    parser.setContentHandler(Handler(outf))

    parser.parse(input_name)

    outf.close()

    return True    
