#include "pch.h"
#include "bbcAdc.h"
#include "bbcSystemVIA.h"
#include "bbcComputer.h"

int bbcAdc::next_stop_at_;
bool bbcAdc::busy_;
bool bbcAdc::is_8bit_;
int bbcAdc::input_channel_;
float bbcAdc::channels_[4];
t65::Word bbcAdc::last_conversion_;
bool bbcAdc::conversion_complete_;

void bbcAdc::Init() {
	bbcAdc::Reset();
}

void bbcAdc::Reset() {
	unsigned i;
	
	busy_=true;
	is_8bit_=false;
	input_channel_=0;
	last_conversion_.w=0;
	conversion_complete_=false;
	for(i=0;i<4;++i) {
		channels_[i]=0.f;
	}
}

void bbcAdc::Update(int cycles) {
	if(busy_) {
		if(cycles-next_stop_at_<0) {//bbcComputer::cycles<next_stop_at_) {
			bbcComputer::SetNextStop(next_stop_at_);
			return;
		} else {
			busy_=false;
			conversion_complete_=true;
			last_conversion_.w=t65::word(channels_[input_channel_]*65535.f)&0xFFF0;
			bbc_system_via.CB1(false);
		}
	}
}

t65::byte bbcAdc::Read(t65::word addr) {
	t65::byte v=0;

	switch(addr&3) {
	case 0:
		//Status
		v=input_channel_&3;
		if(is_8bit_) {
			v|=8;
		}
		if(last_conversion_.w&0x8000) {
			v|=32;
		}
		if(last_conversion_.w&0x4000) {
			v|=16;
		}
		if(!busy_) {
			v|=64;
		}
		if(!conversion_complete_) {
			v|=128;
		}
		//busy_=false;//TODO is this correct?
		return v;
	case 1:
		//MSB of conversion
		//busy_=false;
		return last_conversion_.h;
	case 2:
		//LSB of conversion
		//busy_=false;
		return last_conversion_.l;
	}
	return 0;
}

void bbcAdc::Write(t65::word addr,t65::byte val) {
	if((addr&3)==0) {
		input_channel_=val&3;
		is_8bit_=!!(val&8);
		busy_=true;
		conversion_complete_=false;
		//AUG p429, 8bit takes 4ms (8000 cycles) and 12bit 10ms (20000 cycles)
		next_stop_at_=bbcComputer::Cycles2MHz()+(is_8bit_?8000:20000);
		bbcComputer::SetNextStop(next_stop_at_);
	}
}

#ifdef bbcDEBUG_PANELS
void bbcAdc::InitDebugPanel(bbcDebugPanel *panel) {
	panel->SetSize(20,6);
}

void bbcAdc::UpdateDebugPanel(bbcDebugPanel *panel) {
	for(int i=0;i<4;++i) {
		panel->Print(0,i,"ch%d=%f",i,channels_[i]);
	}
	panel->Print(0,4,"input=%d",input_channel_);
	panel->Print(0,5,"last_conversion=&%04X",last_conversion_);
}
#endif
