#ifndef bbcDEBUGTRACE_H_
#define bbcDEBUGTRACE_H_

#ifdef bbcDEBUG_TRACE

#include "bbcVIA.h"
#include <list>
#include <vector>

class bbcDebugTrace {
public:
	bbcDebugTrace();
	~bbcDebugTrace();

	typedef bool (*ProgressFn)(int percent_done,void *context);
	bool WriteTrace(FILE *out,ProgressFn progress_fn,void *context) const;

	struct Instr {
		t65::Word pc;
		t65::byte bytes[3];
		t65::byte a,x,y,p,s;//10 bytes
	};

	struct Irq {
		int irqflags;
		bbcVIA::TraceState sys,usr;
		t65::byte fdc_status;//about 40 (!!!!!) bytes total
	};

	enum Type {
		ENT_IRQ,
		ENT_INSTR,
	};

	struct Ent {
		Type type;
		int cycles;
		int next_stop;
		union {
			Instr instr;
			Irq irq;
		};
	};
public:
	Instr *NewInstr();
	Irq *NewIrq();

	unsigned size() const;
protected:
private:
	typedef std::vector<Ent> Chunk;
	typedef std::list<Chunk> Chunks;
	unsigned chunk_index_;
	Chunks chunks_;
	unsigned num_entries_;
	Ent *NewEnt(Type type);

	void AddChunk();
	void WriteViaState(FILE *out,int indent,const char *via_name,const bbcVIA::TraceState *state) const;
};

inline unsigned bbcDebugTrace::size() const {
	return num_entries_;
}

inline bbcDebugTrace::Instr *bbcDebugTrace::NewInstr() {
	return &this->NewEnt(ENT_INSTR)->instr;
}

inline bbcDebugTrace::Irq *bbcDebugTrace::NewIrq() {
	return &this->NewEnt(ENT_IRQ)->irq;
}

#endif//bbcDEBUG_TRACE

#endif
