#include "pch.h"
#include "bbcEventRecord.h"
#include "bbcComputer.h"

//////////////////////////////////////////////////////////////////////////
//
bbcEventHandler::bbcEventHandler(unsigned data_size_in,const char *id_in):
data_size(data_size_in),
id(real_id_)
{
	BASSERT(id_in[0]!=0&&id_in[1]!=0&&id_in[2]!=0&&id_in[3]!=0&&id_in[4]==0);
	strcpy(real_id_,id_in);
}

bbcEventHandler::~bbcEventHandler() {
}

//////////////////////////////////////////////////////////////////////////
//
static const unsigned chunk_size=1048576;

bbcEventRecord::bbcEventRecord() {
	this->AddChunk();
}

bbcEventRecord::~bbcEventRecord() {
	for(ChunkList::iterator chunk=chunks_.begin();chunk!=chunks_.end();++chunk) {
		delete[] chunk->data;
	}
}

void *bbcEventRecord::AddEvent(const bbcEventHandler *handler) {
	//If no handler specified, just return null. this allows for disabling
	//events if you store handler pointers in variables. obviously return
	//value now has to be checked :(
	if(!handler) {
		return 0;
	}

	Chunk *chunk=&chunks_.back();

	//will it fit? if not, create a new one. if there's still space left in this
	//chunk, add terminator.
	unsigned event_size=sizeof(bbcEventInfo)+handler->data_size;
	if(chunk->end+event_size>chunk_size) {
		//add new chunk.
		chunk=this->AddChunk();
	}

	//add space for this event.
	void *p=&chunk->data[chunk->end];
	chunk->end+=event_size;

	//fill in the bbcEventInfo.
	bbcEventInfo *info=static_cast<bbcEventInfo *>(p);
	info->handler=handler;
	info->cycles=bbcComputer::cpucycles;
	info->next_stop=bbcComputer::next_stop;

	//call the handler to fill it in.
	void *data=info+1;
	info->handler->FillEventData(data);

	//and return pointer to the data part.
	return data;
}

bbcEventRecord::Chunk *bbcEventRecord::AddChunk() {
	Chunk chunk;

	chunk.data=new t65::byte[chunk_size];
	chunk.end=0;

	chunks_.push_back(chunk);

	return &chunks_.back();
}

void bbcEventRecord::Clear() {
	for(ChunkList::const_iterator chunk=chunks_.begin();chunk!=chunks_.end();++chunk) {
		delete[] chunk->data;
	}
	chunks_.clear();
}

bool bbcEventRecord::SaveEventsText(FILE *out,ProgressFn progress_fn,void *context) {
	unsigned chunk_index=0;
	std::vector<char> output;
	unsigned write_size=1048576;//minimum number of bytes to write in one go
	float chunkamt=1.f/chunks_.size();
	unsigned amount_written=0;

	output.reserve(write_size);
	(*progress_fn)(0,0,context);
	for(ChunkList::const_iterator chunk=chunks_.begin();chunk!=chunks_.end();++chunk,++chunk_index) {
		unsigned i=0;
		while(i<chunk->end) {
			const bbcEventInfo *info=reinterpret_cast<const bbcEventInfo *>(&chunk->data[i]);
			BASSERT(info->handler);
			BASSERT(i+info->handler->data_size<=chunk_size);
			const void *event=info+1;
			char prefix[31];

			//each event starts with the event type id, cycle count and next stop.
			int indent=sprintf(prefix,"%s %-10u(@%-10u): ",info->handler->id,info->cycles,info->next_stop);//30 chars
			output.insert(output.end(),prefix,prefix+indent);

			//turn that into a string
			info->handler->AppendEventStr(info,event,indent,&output);

			//advance to next event.
			i+=sizeof(bbcEventInfo)+info->handler->data_size;

			//if we've got enough, or we've reached the end of a chunk, write the text.
			if(output.size()>=write_size||i>=chunk->end) {
				float percent=chunk_index/float(chunks_.size())+i/float(chunk->end)*chunkamt;

				if(!(*progress_fn)(int(percent*100.f),amount_written,context)) {
					break;
				}

				if(fwrite(&output[0],1,output.size(),out)!=output.size()) {
					return false;
				}
				amount_written+=output.size();

				printf("Saving events output: just wrote %u bytes. on chunk %u/%u\n",output.size(),chunk_index,chunks_.size());

				output.clear();
				output.reserve(write_size);
			}
		}

		output.clear();
	}

	bool ok=chunk==chunks_.end();
	this->Clear();
	return ok;
}
