#ifndef bbcMODELGENERIC_H_
#define bbcMODELGENERIC_H_

#include "bbcModel.h"

//////////////////////////////////////////////////////////////////////////
// more templates. This creates a suitable derived class for a specific model, with the
// minimum of hassle. (though it does look a bit scary.)
template<class Sim,unsigned Tflags>
class bbcModelGeneric:
	public bbcModel
{
public:
	bbcModelGeneric();
	void Init() const;
	void InitialiseCpuState() const;
	void Reset6502() const;
	void Run() const;
	t65::Word Disassemble(t65::Word base,char *opcode_buf,char *operand_buf,t65::byte *bytes_buf) const;
	int DisassembleBytes(const t65::byte *bytes,t65::Word bytes_addr,char *opcode_buf,char *operand_buf) const;
	const bbcMmioFnInit *ExtraMmioFns() const;
	t65::byte ReadByte(t65::Word addr) const;

	void GetInstrInfo(t65::byte op,const char **mnemonic,const char **addrmode,unsigned *count) const;

	static const bbcModelGeneric<Sim,Tflags> model;
private:
	static t65TYPENAME Sim::MachineType the_machine_;
#ifdef bbcDEBUG_TRACE
	bbcDebugTrace::Instr *AddInstrTrace() const;
#endif
};


template<class Sim,unsigned Tflags>
const bbcModelGeneric<Sim,Tflags> bbcModelGeneric<Sim,Tflags>::model;

template<class Sim,unsigned Tflags>
t65TYPENAME Sim::MachineType bbcModelGeneric<Sim,Tflags>::the_machine_;

template<class Sim,unsigned Tflags>
bbcModelGeneric<Sim,Tflags>::bbcModelGeneric():
bbcModel(Tflags)
{
}

template<class Sim,unsigned Tflags>
int bbcModelGeneric<Sim,Tflags>::DisassembleBytes(const t65::byte *bytes,
	t65::Word bytes_addr,char *opcode_buf,char *operand_buf) const
{
	return Sim::DisassemblerType::Disassemble(bytes_addr,bytes,opcode_buf,
		operand_buf);
}

template<class Sim,unsigned Tflags>
void bbcModelGeneric<Sim,Tflags>::Init() const {
	Sim::Init();
	for(unsigned i=0;i<16;++i) {
		Sim::CPUType::Config::romsel_write_fn(0x30+i,0);//reset ROMSEL
	}
}

template<class Sim,unsigned Tflags>
void bbcModelGeneric<Sim,Tflags>::InitialiseCpuState() const {
	Sim::InitialiseCpuState(the_machine_);
}

template<class Sim,unsigned Tflags>
void bbcModelGeneric<Sim,Tflags>::Reset6502() const {
	Sim::Reset6502(the_machine_);
}

#ifdef bbcDEBUG_TRACE
template<class Sim,unsigned Tflags>
bbcDebugTrace::Instr *bbcModelGeneric<Sim,Tflags>::AddInstrTrace() const {
	typedef t65TYPENAME Sim::MachineType Machine;
	bbcDebugTrace::Instr *ins=Machine::trace->NewInstr();
	t65::Word addr=ins->pc=bbcComputer::cpustate.pc;
	Sim::ByteReader br;
	ins->bytes[0]=br(addr);
	++addr.w;
	ins->bytes[1]=br(addr);
	++addr.w;
	ins->bytes[2]=br(addr);

	return ins;
}
#endif

template<class Sim,unsigned Tflags>
void bbcModelGeneric<Sim,Tflags>::Run() const {
	typedef t65TYPENAME Sim::MachineType Machine;
	typedef t65TYPENAME Sim::CPUType Cpu;

	//Set to true if we run any instructions with I enabled this time round.
	//Then, if an interrupt is required, and I is set, but irqs_ever_enabled
	//is true, we know to do the interrupt anyway since the last instruction
	//that ran with I clear was a SEI or something.
	//
	//TODO This might not be _quite_ right, though. If an SEI is run just at
	//the right moment, an IRQ could occur just after the SEI. But maybe this
	//happens on a real Beeb too?
	bool irqs_ever_enabled=false;

	if(Machine::cpustate.p&t65::I_MASK) {
		//Interrupts disabled. Run until next stop.
		do {
			BASSERT((Machine::cpustate.p&t65::I_MASK));
#ifdef bbcDEBUG_TRACE
			bbcDebugTrace::Instr *instr=0;
			if(Machine::trace) {
				instr=AddInstrTrace();
			}
#elif (defined bbcDEBUG_TRACE_EVENTS)
			bbcModelCpuEventHandler::Data *instr=0;
			if(bbcComputer::trace_cpu_events) {
				instr=static_cast<bbcModelCpuEventHandler::Data *>(bbcComputer::event_record.AddEvent(&bbcModelCpuEventHandler::handler));
				instr->pc=bbcComputer::cpustate.pc;
				t65::Word addr=instr->pc;
				instr->bytes[0]=Sim::ByteReader()(addr);
				++addr.w;
				instr->bytes[1]=Sim::ByteReader()(addr);
				++addr.w;
				instr->bytes[2]=Sim::ByteReader()(addr);
			}
#endif
			Sim::RunSingleInstruction(the_machine_);
#if (defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS)
			if(instr) {
				instr->a=Machine::cpustate.a;
				instr->p=Machine::cpustate.p;
				instr->x=Machine::cpustate.x;
				instr->y=Machine::cpustate.y;
				instr->s=Machine::cpustate.s.l;
			}
#endif
		} while(int(Machine::cpucycles-Machine::next_stop)<0);
	}

	//9/3/04 trying to fix empire strikes back (again) -- run another instruction
	//if IRQs just reenabled.
	if(!(Machine::cpustate.p&t65::I_MASK)) {
		irqs_ever_enabled=true;
		//interrupts enabled. Run until next stop or until interrupt happens. Will
		//stop straight away if irq_flags_ set.
		do {
#ifdef bbcDEBUG_TRACE
			bbcDebugTrace::Instr *instr=0;
			if(Machine::trace) {
				instr=AddInstrTrace();
			}
#elif (defined bbcDEBUG_TRACE_EVENTS)
			bbcModelCpuEventHandler::Data *instr=0;
			if(bbcComputer::trace_cpu_events) {
				instr=static_cast<bbcModelCpuEventHandler::Data *>(bbcComputer::event_record.AddEvent(&bbcModelCpuEventHandler::handler));
				instr->pc=bbcComputer::cpustate.pc;
				t65::Word addr=instr->pc;
				instr->bytes[0]=Sim::ByteReader()(addr);
				++addr.w;
				instr->bytes[1]=Sim::ByteReader()(addr);
				++addr.w;
				instr->bytes[2]=Sim::ByteReader()(addr);
			}
#endif
			Sim::RunSingleInstruction(the_machine_);
			//BASSERT(!Machine::irq_flags_||int(Machine::cycles-Machine::next_stop)<0);
#if (defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS)
			if(instr) {
				instr->a=Machine::cpustate.a;
				instr->p=Machine::cpustate.p;
				instr->x=Machine::cpustate.x;
				instr->y=Machine::cpustate.y;
				instr->s=Machine::cpustate.s.l;
			}
#endif
		} while(!Machine::irq_flags_&&int(Machine::cpucycles-Machine::next_stop)<0);
	}

	// 8/3/04 same for both types of main loop now.
	if(Machine::irq_flags_) {
#ifdef bbcDEBUG_TRACE
		if(Machine::trace) {
			Machine::AddIrqTrace();
		}
#elif (defined bbcDEBUG_TRACE_EVENTS)
		if(bbcComputer::trace_cpu_events) {
			bbcComputer::event_record.AddEvent(&bbcModelIrqEventHandler::handler);
		}
#endif
		if(Machine::irq_flags_&IRQ_NMI) {
			Machine::irq_flags_&=~IRQ_NMI;
			Cpu::NMI(the_machine_);
		} else if(!(Machine::cpustate.p&t65::I_MASK)||irqs_ever_enabled) {
			Cpu::IRQ(the_machine_);
		}
	}
}

template<class Sim,unsigned Tflags>
const bbcMmioFnInit *bbcModelGeneric<Sim,Tflags>::ExtraMmioFns() const {
	static const bbcMmioFnInit mmio_fns_[]={
		{0xfe30,16,Sim::CPUType::Config::romsel_read_fn,Sim::CPUType::Config::romsel_write_fn},
		{0,0},
	};

	return mmio_fns_;
}

template<class Sim,unsigned Tflags>
t65::Word bbcModelGeneric<Sim,Tflags>::Disassemble(t65::Word base,char *opcode_buf,char *operand_buf,t65::byte *bytes_buf) const {
	return Sim::Disassemble(the_machine_,base,opcode_buf,operand_buf,bytes_buf);
}

template<class Sim,unsigned Tflags>
t65::byte bbcModelGeneric<Sim,Tflags>::ReadByte(t65::Word addr) const {
	return Sim::ByteReader()(addr);
}

template<class Sim,unsigned Tflags>
void bbcModelGeneric<Sim,Tflags>::GetInstrInfo(t65::byte op,const char **mnemonic,const char **addrmode,unsigned *count) const {
	const t65::InstrInfo *instr=&Sim::DisassemblerType::dis_table[op];

	*mnemonic=instr->mnemonic;
	*addrmode=instr->addr_mode_name;
#ifdef t65_COUNT_INSTRUCTION_FREQUENCIES
	*count=Sim::CPUType::instr_freqs[op];
#else
	*count=0;
#endif
}

#endif