#include "pch.h"
#include "bbcComputer.h"
#include "bbcSystemVIA.h"
#include "bbcSound.h"
#include "bbcKeys.h"
#include "bbcKeyboardMatrix.h"
#include "bbcCmos.h"
#include <time.h>

bbcSystemVIA bbc_system_via;

const t65::word bbcSystemVIA::wraptbl_[4]={
    0x4000,
    0x2000,
    0x5000,
    0x2800,
};

bbcSystemVIA::bbcSystemVIA():
bbcVIA(IRQ_SYSVIA,"SysVIA"),
cmos_(0),
cmos_op_read_(false),
cmos_enabled_(false),
cmos_data_strobe_(false),
cmos_address_(false)
{
	latch_=0;

	joystick_buttons_[0]=false;
	joystick_buttons_[1]=false;
}

// The top 4 input bits on PB are for the joystick and speech system.
// %1111 means 'do nothing'.
void bbcSystemVIA::ReadingPb() {
	//speech -- do nothing
	if(!cmos_) {
		pb_|=0xC0;
	}

	//AUG doesn't say which way round these are :)
	pb_&=~0x30;
	if(!joystick_buttons_[0]) {
		pb_|=0x10;
	}
	if(!joystick_buttons_[1]) {
		pb_|=0x20;
	}
}

void bbcSystemVIA::ReadingPa() {
	if(cmos_) {
		pa_=bbcCmos::DataPins();
	}
	if(!(latch_&8)) {
		pa_&=0x7F;
		if(bbcKeyboardMatrix::ManualScanFlag()) {
			pa_|=0x80;
		}
	}
}

// Presumably multiple devices can be written to?
void bbcSystemVIA::WrittenPa() {
	this->SlowWrite(pa_);
}

// Port B on the system VIA is attached to the addressable latch. PB[012]
// supply the bit number, 0-7, and PB3 supplies the new value for that
// bit.
// This code is based on the code from James Bonfield's emulator.
void bbcSystemVIA::WrittenPb() {
	latched_pb_=pb_;
	
	t65::byte old_latch=latch_;

	//Firstly, handle latch.
	t65::byte mask=1<<(pb_&7);
	latch_&=~mask;
	if(pb_&8) {
		latch_|=mask;
	}

	//Set keyboard mode from that
	bbcKeyboardMatrix::SetWriteEnabled(!(latch_&8));

	bbcVideo::SetScreenWrapSize(wraptbl_[(latch_>>4)&3]);

	this->SlowWrite(pa_);
}

void bbcSystemVIA::SlowWrite(t65::byte value) {
	//run clock chip
	if(cmos_) {
		bbcCmos::SetAs(!!(latched_pb_&128));
		bbcCmos::SetDs(!!(latch_&4));
		bbcCmos::SetCe(!(latched_pb_&64));
		bbcCmos::SetRw(!!(latch_&2));
		bbcCmos::SetDataPins(value);
	}

	//sound
	if(!(latch_&1)) {
		bbcSound::Write(value);
	}

	//keyboard
	if(!(latch_&8)) {
		bbcKeyboardMatrix::SetInput(value);

		if(bbcKeyboardMatrix::NeedsIrq()) {
			this->CA2(true);
		}
	}
}

//	cmos_enabled_=!!(pb_&0x40);
//	if(cmos_&&cmos_enabled_&&(pb_&0x80)) {
//		cmos_address_=pb_&63;
//		printf("CMOS: address now +0x%02X (+%d)\n",cmos_address_,cmos_address_);
//	} else {
//
//		t65::byte mask=1<<(pb_&7);
//		latch_&=~mask;
//		if(pb_&8) {
//			latch_|=mask;
//		}
//
//		if(!(latch_&1)) {
//			bbcSound::Write(pa_);
//		}
//
//		if(!(latch_&2)) {
//			if(cmos_enabled_&&cmos_data_strobe_) {
//				this->CmosWrite(cmos_address_,pa_);
//				printf("CMOS: +0x%02X (+%d): write 0x%02X (%d)\n",cmos_address_,cmos_address_,pa_,pa_);
//			}
//		}
//
//		cmos_data_strobe_=!!((latch_^old)&4);
//
//		if(latch_&8) {
//			bbcKeyboardMatrix::SetWriteEnabled(false);
//		} else {
//			bbcKeyboardMatrix::SetWriteEnabled(true);
//			bbcKeyboardMatrix::SetInput(pa_);
//		}
//
//		bbcVideo::SetScreenWrapSize(wraptbl_[(latch_>>4)&3]);
//	}
//}
//	cmos_enabled_=!!(pb_&(1<<6));
//	bool old=latch_[pb_&7];
//	if(cmos_&&cmos_enabled_&&(pb_&0x80)) {
//		cmos_address_=pb_&63;
//		printf("CMOS: address now +0x%02X (+%d)\n",cmos_address_,cmos_address_);
//	} else {
//		latch_[pb_&7]=!!(pb_&8);
//		switch(pb_&7) {
//		case 0:     // "Write enable to the sound generator IC"
//			if(!latch_[0]) {
//				bbcSound::Write(pa_);
//				//                soundhardware->write(PA);
//			}
//			break;
//		case 1:     // "Read select on the speech processor"
//			if(!latch_[1]&&cmos_enabled_&&cmos_data_strobe_) {
//				this->CmosWrite(cmos_address_,pa_);
//				printf("CMOS: +0x%02X (+%d): write 0x%02X (%d)\n",cmos_address_,cmos_address_,pa_,pa_);
//			}
//			break;
//		case 2:     // "Write select on the speech processor"
//			cmos_data_strobe_=latch_[2]!=old;
//			break;
//		case 3:     // "Keyboard write enable" (0 = en 1 = dis)
//			if(latch_[3]) {
//				//dis
//				bbcKeyboardMatrix::SetWriteEnabled(false);
//			} else {
//				//en -- maybe new input
//				bbcKeyboardMatrix::SetWriteEnabled(true);
//				bbcKeyboardMatrix::SetInput(pa_);
//			}
//			break;
//		case 4:
//		case 5:     // Select screen wrap size
//			//			bbc_video_hardware.set_screen_wrap_size(wraptbl[(latch[4]?1:0)|(latch[5]?2:0)]);
//			{
//				unsigned v=(latch_[4]?1:0)|(latch_[5]?2:0);
//				bbcVideo::SetScreenWrapSize(wraptbl_[v]);
//			}
//			break;
//		case 6:     // Caps lock LED status
//			capslock_led_=!latch_[6];
//			break;
//		case 7:     // Shift lock LED status
//			shiftlock_led_=!latch_[7];
//			break;
//		}
//	}
//}

bool bbcSystemVIA::CapslockLedLit() {
	return !(latch_&64);
}

bool bbcSystemVIA::ShiftlockLedLit() {
	return !(latch_&128);
}


//this is the keyboard scan type affair -- it only occurs when the keyboard is
//not enabled for write.
//causes a +ve active edge on CA2 if the keyboard wants it.
//don't ask me how i know it's a +ve one -- i think i looked at the relevant
//via register to work out what's required to get the interrupt to fire.
//
// fix -- keystate updated at any time, only when disabled for write will
// the interrupt fire though. [19/3/2003]
void bbcSystemVIA::SetKeyState(bbcKey key,bool state) {
	bbcKeyboardMatrix::SetKeyState(key,state);
//	if(latch[3]&&KeyboardMatrix::NeedsIrq()) {
//		CA2(true);
//	}
}

//	if(latch[3]) {
//		if(KeyboardMatrix::SetKeyState(key,state)) {
//			CA2(true);
//		}
//	}
//}

void bbcSystemVIA::DoKeyboardIrq() {
	if(bbcKeyboardMatrix::NeedsIrq()) {
		this->CA2(true);
	}
}

void bbcSystemVIA::SetJoystickButtonState(int joystick,bool state) {
	BASSERT(joystick==0||joystick==1);
	joystick_buttons_[joystick]=state;
}

void bbcSystemVIA::SetCmos(t65::byte *cmos) {
	cmos_=cmos;
}

//static t65::byte BCD(t65::byte v) {
//	return v/10%10*16+v%10;
//}
//
//t65::byte bbcSystemVIA::CmosRead(t65::byte address) {
//	if(address<10) {
//		//TODO, this is possible too system-dependent for beebm_base...
//		//this code comes from from BeebEm
//		time_t SysTime;
//		time( &SysTime );
//		struct tm *CurTime = localtime( &SysTime );
//		switch(address) {
//		case 0x00:
//			return(BCD(CurTime->tm_sec));
//		case 0x02:
//			return(BCD(CurTime->tm_min));
//		case 0x04:
//			return(BCD(CurTime->tm_hour));
//		case 0x06:
//			return(BCD((CurTime->tm_wday)+1));
//		case 0x07:
//			return(BCD(CurTime->tm_mday));
//		case 0x08:
//			return(BCD((CurTime->tm_mon)+1));
//		case 0x09:
//			return(BCD((CurTime->tm_year)-10));
//		}
//	} else if(address>=14) {
//		if(address-14<bbcComputer::cmos_size) {
//			return cmos_[address-14];
//		}
//	}
//	return 0;
//}
//
//void bbcSystemVIA::CmosWrite(t65::byte address,t65::byte value) {
//	if(address>=14&&address-14<bbcComputer::cmos_size) {
//		cmos_[address-14]=value;
//	}
//}

t65::byte bbcSystemVIA::MmioRead(t65::word addr) {
	return bbc_system_via.Read(addr);
}

void bbcSystemVIA::MmioWrite(t65::word addr,t65::byte val) {
	bbc_system_via.Write(addr,val);
}
