#ifndef mbCONFIG_H_
#define mbCONFIG_H_

#include <bbcComputer.h>
#include "mbKeymap.h"
#include "Host.h"
#include <map>

//the wxWindows config type this program uses
//wxFIle Config, I cannot stand the registry!!
typedef wxFileConfig mbWxConfig;

enum mbMonitorType {
	mbMONITOR_COLOUR=0,
	mbMONITOR_BNW,
	mbMONITOR_AMBER,
	mbMONITOR_GREEN,

	mbMONITOR_NUM_TYPES,
};

enum mbScanlinesMode {
	mbSCANLINES_SINGLE,
	mbSCANLINES_INTERLACE,
	mbSCANLINES_DOUBLE,
};

//////////////////////////////////////////////////////////////////////////
// ROM configuration
struct mbRomsConfig {
	//Configuration for a ROM slot
	struct Rom {
		wxString filename;
		bool ram;

		Rom();
	};
	Rom roms[16];
	Rom master128_roms[7];
	wxString os_b;
	wxString os_bplus;
	wxString os_bplus128;
	wxString os_master128;
	wxRect dlg_rect;
	int fs_slot;
	std::map<wxString,wxString> fs_roms;

	mbRomsConfig();
	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
private:
	void LoadRom(mbWxConfig *cfg,const wxString &key,Rom *rom);
	void SaveRom(mbWxConfig *cfg,const wxString &key,const Rom *rom) const;
};

//////////////////////////////////////////////////////////////////////////
// Sound configuration
struct mbSoundConfig {
	bool enabled;
	bool stereo;
	int hz,bits;
	int len_frames;
	bool is_stereo_left[4],is_stereo_right[4];
	wxRect dlg_rect;
	int volume;

	mbSoundConfig();
	
	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
// Keyboard configuration
struct mbKeyboardConfig {
	bool keylinks[8];
	mbKeymap::C keymaps;
	wxString selected_keymap;
	wxRect dlg_rect;

	mbKeyboardConfig();
	
	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
//
struct mbVideoConfig {
	static const int max_frame_skip;//min is 1, of course

	int width,height;
	int frame_skip;
	int cleanup_frequency;
	bool full_screen;
	int full_screen_refresh_rate;
	bool full_screen_vsync;
	mbScanlinesMode full_screen_scanlines;
	mbScanlinesMode windowed_scanlines;
	mbMonitorType monitor_type;
	wxRect dlg_rect;
	int video_effect;
	
	mbVideoConfig();
	
	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
//
struct mbMiscConfig {
	bool limit_speed;
	wxString disc_interface;
	bool show_status_bar;
	bool show_menu;
	bool fast_forward_disc_access;
	wxString model;
	wxRect window_rect;
	bool window_is_maximized;
	wxRect hardware_dlg_rect;//hardly the best place for it, but I suppose it kind of goes with the model and disc interface settings.

	mbMiscConfig();
	
	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
//
struct mbQuickstartConfig {
	wxRect dialog_pos;
	std::vector<wxFileName> dirs;
	
	mbQuickstartConfig();

	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
//
struct mbJoysticksConfig {
	enum Axis {
		X,Y,Z,RX,RY,RZ,INVALID_AXIS,
	};
	static float HostInp_JoyState::*const hostinp_axes[6];

	struct BeebStick {
		wxString device_name;
		bool enabled;
		//int joystick_index;
		Axis x_axis;
		Axis y_axis;
		int button;

		BeebStick();
	};

	BeebStick joysticks[2];

	mbJoysticksConfig();

	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
//
struct mbCmosConfig {
	t65::byte contents[bbcComputer::cmos_size];

	mbCmosConfig();

	void Load(mbWxConfig *cfg);
	void Save(mbWxConfig *cfg) const;
};

//////////////////////////////////////////////////////////////////////////
//
struct mbConfig {
	mbRomsConfig roms;
	mbSoundConfig sound;
	mbKeyboardConfig keyboard;
	mbVideoConfig video;
	mbMiscConfig misc;
	mbQuickstartConfig quickstart;
	mbJoysticksConfig joysticks;
	mbCmosConfig cmos;
	wxString file_name;
	bool read_only;
	
	mbConfig();
	void SetFilename(const wxString &config_file_name,bool is_read_only);
	void Load(const wxString &config_file_name);
	void Save() const;
	void GetConfig(mbWxConfig *cfg) const;
};

#endif
