#include "pch.h"
#include "mbHardwareDialog.h"

enum {
	id_fsel=wxID_HIGHEST,
	id_interface_radio,
	id_model_radio,
};

mbHardwareDialog::Model::Model():
model(0),
model_os(0),
os_file_name(0),
select_file(0),
radio(0)
{
}

mbHardwareDialog::ModelRom::ModelRom():
file_name(0),
select_file(0),
rom(0),
slot(-1)
{
}

mbHardwareDialog::Interface::Interface():
iface(0),
radio(0),
file_name(0),
select_file(0)
{
}

void mbHardwareDialog::Interface::Enable(wxControl *w,bool enabled) {
	if(w) {
		w->Enable(enabled);
	}
}

void mbHardwareDialog::Interface::SetEnabled(bool enabled) {
	this->Enable(this->radio,enabled);
	this->Enable(this->file_name,enabled);
	this->Enable(this->select_file,enabled);
}

wxBoxSizer *mbHardwareDialog::CreateTextCtrlFsel(const wxString &caption,wxTextCtrl **text,wxButton **button) {
	wxBoxSizer *sz=new wxBoxSizer(wxHORIZONTAL);

	*text=new wxTextCtrl(this,-1,"");
	*button=new wxButton(this,id_fsel,"...",wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);

	sz->Add(new wxStaticText(this,-1,caption),0,wxLEFT|wxRIGHT|wxALIGN_CENTRE_VERTICAL,2);
	sz->Add(*text,1,wxLEFT|wxRIGHT,2);
	sz->Add(*button,0,wxLEFT|wxRIGHT,2);

	return sz;
}

mbHardwareDialog::mbHardwareDialog(wxWindow *parent,const mbMiscConfig &cfg,const mbRomsConfig &roms_cfg):
wxDialog(parent,-1,"Hardware",wxDefaultPosition,wxDefaultSize,wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER),
cfg_(cfg),
roms_cfg_(roms_cfg),
b_(0),
bplus_(0)
{
	static const unsigned tlr_gaps=wxTOP|wxLEFT|wxRIGHT;
	static const int border_size=6;
	unsigned i;

	//models
	std::vector<const mbModel *> models;
	mbModelGetAll(&models);
	wxStaticBox *models_box=new wxStaticBox(this,-1,"Model");
	wxStaticBoxSizer *models_box_sz=new wxStaticBoxSizer(models_box,wxVERTICAL);
	for(i=0;i<models.size();++i) {
		const mbModel *src=models[i];
		Model *dest=&*models_.insert(models_.end(),Model());

		dest->model=src;
		dest->model_os=&(roms_cfg_.*dest->model->os_name);

		dest->radio=new wxRadioButton(this,id_model_radio,dest->model->long_name,wxDefaultPosition,wxDefaultSize,wxRB_SINGLE);
		models_box_sz->Add(dest->radio,0,wxALL,4);

		//see if the OS is the same as one we've already added -- if so, just add in a static rather
		//than effectively duplicate a control.
		{
			unsigned j;

			//-1 because current model already in there -- this is a hack in the interests of usability.
			for(j=0;j<models_.size()-1;++j) {
				if(models_[j].model_os==dest->model_os) {
					break;
				}
			}

			if(j<models_.size()-1) {
				//yeah, we've seen this one.
				wxString caption=wxString::Format("(OS ROM as %s)",models_[j].model->long_name);
				models_box_sz->Add(new wxStaticText(this,-1,caption),0,wxGROW|wxALL,4);
			} else {
				wxBoxSizer *os_sz=this->CreateTextCtrlFsel("OS:",&dest->os_file_name,&dest->select_file);
				models_box_sz->Add(os_sz,0,wxGROW|wxALL,4);
			}
		}
		
		//mbRomsConfig::Rom *roms[16];
		//dest->model->GetRomsPtrs(roms_cfg_,roms);
		for(int j=0;j<16;++j) {
			if(dest->model->b_rom_mask&(1<<j)) {
				continue;
			}
			ModelRom *rom=&*dest->model_roms.insert(dest->model_roms.end(),ModelRom());
			rom->rom=src->CfgRomPtr(roms_cfg_,j);	//roms[j];
			rom->slot=j;
			wxBoxSizer *sz=this->CreateTextCtrlFsel(wxString::Format("%X",j),&rom->file_name,&rom->select_file);
			models_box_sz->Add(sz,0,wxGROW|wxALL,4);
		}
	}

	//disc interfaces
	std::vector<const mbDiscInterface *> ifs;
	mbDiscInterfaceGetAll(&ifs);
	wxStaticBox *interfaces_box=new wxStaticBox(this,-1,"Disc interface");
	wxStaticBoxSizer *interfaces_box_sz=new wxStaticBoxSizer(interfaces_box,wxVERTICAL);
	for(i=0;i<ifs.size();++i) {
		Interface *iface=&*interfaces_.insert(interfaces_.end(),Interface());

		iface->iface=ifs[i];
		
		iface->radio=new wxRadioButton(this,id_interface_radio,iface->iface->long_name,wxDefaultPosition,
			wxDefaultSize,wxRB_SINGLE);
		interfaces_box_sz->Add(iface->radio,0,wxALL,4);

		if(iface->iface->iface) {//hmmm
			iface->select_file=new wxButton(this,id_fsel,"...",wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
			iface->file_name=new wxTextCtrl(this,-1,"");
			
			wxBoxSizer *sz=new wxBoxSizer(wxHORIZONTAL);
			sz->Add(new wxStaticText(this,-1,"Filing system ROM:"),0,wxLEFT|wxRIGHT|wxALIGN_CENTRE_VERTICAL,2);
			sz->Add(iface->file_name,1,wxLEFT|wxRIGHT,2);
			sz->Add(iface->select_file,0,wxLEFT|wxRIGHT,2);
			
			interfaces_box_sz->Add(sz,0,wxGROW|wxALL,4);
		}
	}

	//bloody thing won't lay itself out properly.
//	wxStaticBox *models_box=new wxStaticBox(this,-1,"Model");
//	wxStaticBoxSizer *models_box_sz=new wxStaticBoxSizer(models_box,wxVERTICAL);
//	b_=new wxRadioButton(models_box,-1,"Model B");
//	bplus_=new wxRadioButton(models_box,-1,"Model B+");
//	models_box_sz->Add(b_,0,wxALL,4);
//	models_box_sz->Add(bplus_,0,wxALL,4);
	

	wxBoxSizer *btns_sz;
	wxButton *ok=new wxButton(this,wxID_OK,"Apply and hard reset");
	wxButton *cancel=new wxButton(this,wxID_CANCEL,"Cancel");
	ok->SetDefault();
	btns_sz=new wxBoxSizer(wxHORIZONTAL);
	btns_sz->Add(ok,0,wxALL|wxALIGN_RIGHT,border_size);
	btns_sz->Add(cancel,0,wxALL|wxALIGN_RIGHT,border_size);

	wxBoxSizer *top_sz=new wxBoxSizer(wxHORIZONTAL);
	top_sz->Add(models_box_sz,1,wxALL,border_size);
	top_sz->Add(interfaces_box_sz,1,wxALL,border_size);

	wxBoxSizer *all_sz=new wxBoxSizer(wxVERTICAL);
	all_sz->Add(top_sz,1,wxGROW|wxALL,border_size);
//	all_sz->Add(models_box_sz,0,wxGROW|wxALL,border_size);
	all_sz->Add(btns_sz,0,wxALIGN_RIGHT);

	this->SetSizer(all_sz);
	all_sz->SetSizeHints(this);

	this->RefreshDialog();

	this->SetSize(cfg_.hardware_dlg_rect);
}

void mbHardwareDialog::GetResult(mbMiscConfig *cfg,mbRomsConfig *roms_cfg) const {
	*cfg=cfg_;
	*roms_cfg=roms_cfg_;
}

void mbHardwareDialog::GetFselDetails(wxObject *src,wxTextCtrl **ctrl,wxString **string,wxString *caption) {
	BASSERT(src);
	BASSERT(ctrl);
	BASSERT(string);
	BASSERT(caption);

	unsigned i;
	
	for(i=0;i<interfaces_.size();++i) {
		Interface *f=&interfaces_[i];
		
		if(src==f->select_file) {
			*ctrl=f->file_name;
			*string=&roms_cfg_.fs_roms[f->iface->short_name];
			*caption=wxString::Format("Select ROM for %s",f->iface->long_name);
			return;
		}
	}

	for(i=0;i<models_.size();++i) {
		Model *m=&models_[i];

		if(src==m->select_file) {
			*ctrl=m->os_file_name;
			*string=m->model_os;
			*caption=wxString::Format("Select OS for %s",m->model->long_name);
			return;
		}

		for(unsigned j=0;j<m->model_roms.size();++j) {
			ModelRom *mr=&m->model_roms[j];

			if(src==mr->select_file) {
				*ctrl=mr->file_name;
				*string=&mr->rom->filename;
				*caption=wxString::Format("Select ROM %d for %s",mr->slot,m->model->long_name);
				return;
			}
		}
	}
	BASSERT(false);
}

void mbHardwareDialog::OnFsel(wxCommandEvent &event) {
	wxTextCtrl *ctrl;
	wxString *string;
	wxString caption;

	this->GetFselDetails(event.GetEventObject(),&ctrl,&string,&caption);

	wxString old_dir,old_name;
	wxFileName old=wxFileName::FileName(*string);
	if(old.IsOk()) {
		old_name=old.GetFullName();
		old_dir=old.GetPath();
	}
	wxFileDialog fd(this,caption,old_dir,old_name,"All files|*.*",wxOPEN|wxHIDE_READONLY);
	if(fd.ShowModal()==wxID_OK) {
		*string=fd.GetPath();
		this->RefreshDialog();
	}
}

void mbHardwareDialog::RefreshDialog() {
	unsigned i;

	for(i=0;i<interfaces_.size();++i) {
		Interface *f=&interfaces_[i];

		if(f->file_name) {
			f->file_name->SetValue(roms_cfg_.fs_roms[f->iface->short_name]);
		}
		f->radio->SetValue(cfg_.disc_interface==f->iface->short_name);

		f->SetEnabled(false);
	}

	const Model *selected=0;
	for(i=0;i<models_.size();++i) {
		const Model *m=&models_[i];

		if(m->os_file_name) {
			BASSERT(m->model_os);
			m->os_file_name->SetValue(*m->model_os);
		}
		for(unsigned j=0;j<m->model_roms.size();++j) {
			const ModelRom *mr=&m->model_roms[j];

			if(mr->file_name) {
				BASSERT(mr->rom);
				mr->file_name->SetValue(mr->rom->filename);
			}
		}
		if(cfg_.model.CmpNoCase(m->model->short_name)==0) {
			m->radio->SetValue(true);
			selected=m;
		} else {
			m->radio->SetValue(false);
		}
	}

	// Disable any interfaces that aren't valid for the current model.
	for(i=0;selected->model->valid_disc_interfaces[i];++i) {
		for(unsigned j=0;j<interfaces_.size();++j) {
			if(stricmp(interfaces_[j].iface->short_name,selected->model->valid_disc_interfaces[i])==0) {
				interfaces_[j].SetEnabled(true);
				break;
			}
		}
	}
}

void mbHardwareDialog::OnInterfaceRadio(wxCommandEvent &event) {
	for(unsigned i=0;i<interfaces_.size();++i) {
		const Interface *f=&interfaces_[i];

		if(event.GetEventObject()==f->radio) {
			cfg_.disc_interface=f->iface->short_name;
			this->RefreshDialog();
			break;
		}
	}
}

void mbHardwareDialog::OnModelRadio(wxCommandEvent &event) {
	for(unsigned i=0;i<models_.size();++i) {
		const Model *m=&models_[i];

		if(event.GetEventObject()==m->radio) {
			cfg_.model=m->model->short_name;
			this->RefreshDialog();
			break;
		}
	}
}

void mbHardwareDialog::OnOk(wxCommandEvent &event) {
	if(roms_cfg_.fs_slot>=0) {
		const wxString &rom=roms_cfg_.fs_roms[cfg_.disc_interface];

		roms_cfg_.roms[roms_cfg_.fs_slot].filename=rom;
	}
	cfg_.hardware_dlg_rect=this->GetRect();
	event.Skip();
}

BEGIN_EVENT_TABLE(mbHardwareDialog,wxDialog)
	EVT_BUTTON(id_fsel,mbHardwareDialog::OnFsel)
	EVT_RADIOBUTTON(id_interface_radio,mbHardwareDialog::OnInterfaceRadio)
	EVT_RADIOBUTTON(id_model_radio,mbHardwareDialog::OnModelRadio)
	EVT_BUTTON(wxID_OK,mbHardwareDialog::OnOk)
END_EVENT_TABLE()
