#include "pch.h"
#include "mbMainFrame.h"
#include "twArgs.h"
#include "mbModel.h"
#include <bbcComputer.h>
#include <bbcKeyboardMatrix.h>
#include <bbcSystemVIA.h>
#include <bbcSound.h>
#include <bbcFdd.h>
#include <bbcUserVIA.h>
#include <bbcSystemVIA.h>
#include <bbc1770.h>
#include "mbMisc.h"
#include "mbQuickstart.h"
#include "mbRomsConfigDialog.h"
#include "mbSoundConfigDialog.h"
#include "mbKeyboardConfigDialog.h"
#include "mbJoysticksConfigDialog.h"
#ifdef BEEB_DEBUG_SAVE_CHALLENGER_RAM
#include <bbc1770OpusChallenger.h>
#endif
#include "mbDiscFormat.h"
#include "mbHardwareDialog.h"
#include <bbcAdc.h>
#include "mbVideoConfigDialog.h"
#include <bbcUef.h>
#include "mbKeys.h"

//////////////////////////////////////////////////////////////////////////
//Frequences of various update things
//
//input -- how often HostInp is polled
//sys -- how often the message loop is run
//sound -- how often sound update is done
//
//These are given in frequency per _emulated_ second. This is what you want,
//but it may bear little relation to real time.
enum {
	update_input_freq=200,
#ifdef bbcDEBUG_ENABLE
	//Upping this value doesn't make a big difference until the debug panels are
	//activated...
	update_sys_freq=100,
#else
	update_sys_freq=25,
#endif
	update_sound_freq=50,

	update_input_cycles=2000000/update_input_freq,
	update_sys_cycles=2000000/update_sys_freq,
	update_sound_cycles=2000000/update_sound_freq,
};


//////////////////////////////////////////////////////////////////////////
//Crapadelic keyboard press playback!
//#define mbFAKE_KEYPRESSES

#ifdef mbFAKE_KEYPRESSES
enum FakeActionType {
	FAKE_KEY_DOWN,
	FAKE_KEY_UP,

	//these don't have a time_cycles.
	FAKE_START_DISASSEMBLY,
	FAKE_STOP_DISASSEMBLY,
};

struct FakeAction {
	FakeActionType type;
	int cycles;//when to do it
	t65::byte bbc_code;
};

static const int std_step=2000000/10;

#define FAKE_KEY_TAP(START,KEY)\
	{ FAKE_KEY_DOWN, (START), (KEY) },\
	{ FAKE_KEY_UP, (START)+std_step, (KEY) }

static const FakeAction fake_actions[]={
	//*.
	{ FAKE_KEY_DOWN,	2000000+std_step*0,		bbcKEY_SHIFT, },
	{ FAKE_KEY_DOWN,	2000000+std_step*1,		bbcKEY_COLON, },
	{ FAKE_KEY_UP,		2000000+std_step*2,		bbcKEY_COLON, },
	{ FAKE_KEY_UP,		2000000+std_step*3,		bbcKEY_SHIFT, },
	FAKE_KEY_TAP(2000000+std_step*4,bbcKEY_STOP),
	FAKE_KEY_TAP(2000000+std_step*6,bbcKEY_RETURN),

	//*COMPACT
	{ FAKE_KEY_DOWN,	5000000+std_step*0,		bbcKEY_SHIFT, },
	{ FAKE_KEY_DOWN,	5000000+std_step*1,		bbcKEY_COLON, },
	{ FAKE_KEY_UP,		5000000+std_step*2,		bbcKEY_COLON, },
	{ FAKE_KEY_UP,		5000000+std_step*3,		bbcKEY_SHIFT, },
	FAKE_KEY_TAP(5000000+std_step*4,bbcKEY_C),
	FAKE_KEY_TAP(5000000+std_step*6,bbcKEY_O),
	FAKE_KEY_TAP(5000000+std_step*8,bbcKEY_M),
	FAKE_KEY_TAP(5000000+std_step*10,bbcKEY_P),
	FAKE_KEY_TAP(5000000+std_step*12,bbcKEY_A),
	FAKE_KEY_TAP(5000000+std_step*14,bbcKEY_C),
	FAKE_KEY_TAP(5000000+std_step*16,bbcKEY_T),
	FAKE_KEY_TAP(5000000+std_step*18,bbcKEY_RETURN),
	//{ FAKE_START_DISASSEMBLY, 5000000+std_step*19, },//on release of RETURN
};
static const unsigned num_fake_actions=sizeof fake_actions/sizeof fake_actions[0];
#endif

//////////////////////////////////////////////////////////////////////////
//
static const char *window_title="model-b";

//////////////////////////////////////////////////////////////////////////
//TODO the drive handling sucks. UiUnloadDrive similarly.

#ifdef bbcDEBUG_ENABLE
//Enable the File|Test item
#define ENABLE_TEST
#endif

#define AUTO_PAINT() this->SetAutoPaint(true)

//////////////////////////////////////////////////////////////////////////
//Id offsets for each drive
enum {
	subid_drive=0,
	subid_drive_filename,
	subid_drive_load,
	subid_drive_save,
	subid_drive_save_as,
	subid_drive_unload,
	subid_drive_num,//MUST BE LAST
};

const mbMainFrame::DriveCommandMfn mbMainFrame::drive_subactions[subid_drive_num]={
	0,
	0,
	&mbMainFrame::OnDriveLoad,
	&mbMainFrame::OnDriveSave,
	&mbMainFrame::OnDriveSaveAs,
	&mbMainFrame::OnDriveUnload,
};

//////////////////////////////////////////////////////////////////////////

enum {
	id_status_bar=wxID_HIGHEST+1,
		
	id_exit,
	id_quickstart,
	id_hard_reset,
#ifdef ENABLE_TEST
	id_test,
#endif
	id_toggle_sound,
	id_toggle_limit_speed,
	id_toggle_status_bar,
	id_toggle_fast_forward_disc_access,
//	id_toggle_vertical_2x,
	id_toggle_full_screen,
	id_show_about_box,

	id_roms_dialog,
	id_sound_dialog,
	id_keymap_dialog,
	id_joysticks_dialog,
	
	id_keymaps_submenu,
	
	id_resolution_submenu,

	id_resolutions_begin,
	id_resolution_0=id_resolutions_begin,
	id_resolution_1,
	id_resolution_2,
	id_resolution_3,
	id_resolutions_end,
	
#if (defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS)
	id_start_disassembly,
	id_reboot_with_disassembly,
	id_stop_disassembly,
#endif

#ifdef BEEB_DEBUG_SAVE_CHALLENGER_RAM
	id_save_challenger_ram,
#endif
#ifdef bbcDEBUG_ENABLE
	id_disassemble_ram,
#endif
	
	id_screenshot_save_as,

#ifdef bbcDEBUG_PANELS
	id_toggle_dbgpanel_sysvia,
	id_dbgpanel_sysvia,
	id_toggle_dbgpanel_usrvia,
	id_dbgpanel_usrvia,
	id_toggle_dbgpanel_video,
	id_dbgpanel_video,
	id_toggle_dbgpanel_adc,
	id_dbgpanel_adc,
#endif
#ifdef bbcDEBUG_VIDEO
	id_toggle_showall,
	id_log_video_frames,
#ifdef bbcDEBUG_TRACE
	id_log_video_frames_disasm,
#endif
	id_toggle_show_t1_timeouts,

#ifdef t65_COUNT_INSTRUCTION_FREQUENCIES
	id_show_instruction_frequencies,
	id_show_instruction_frequencies_sorted,
#endif

#ifdef bbcDEBUG_COUNT_MMIO_ACCESSES
	id_show_sheila_access_counts_by_addr,
	id_show_sheila_access_counts_by_count,
#endif

#endif
	//TODO should be begin/end
	id_first_keymap,
	id_last_keymap=id_first_keymap+1000,

	id_drives_begin,
	id_drives_end=id_drives_begin+bbcFdd::num_drives*subid_drive_num,

	id_sound_record_start,
	id_sound_record_stop,

	id_screen_type_submenu,
	id_screen_types_begin,
	id_screen_type_colour,
	id_screen_type_bnw,
	id_screen_type_amber,
	id_screen_type_green,
	id_screen_types_end,

	id_scanlines_begin,
	id_windowed_scanlines_submenu,
	id_windowed_scanlines_single,
	id_windowed_scanlines_interlace,
	id_windowed_scanlines_double,
	id_windowed_scanlines_end,

	id_full_screen_scanlines_submenu,
	id_full_screen_scanlines_single,
	id_full_screen_scanlines_interlace,
	id_full_screen_scanlines_double,
	id_full_screen_scanlines_end,
	id_scanlines_end,

	id_hardware_dialog,
	id_video_dialog,
	
	id_file_menu,
	id_options_menu,
	id_special_menu,
#ifdef bbcDEBUG_ENABLE
	id_debug_menu,
#endif
#ifdef bbcENABLE_SAVE_STATE
	id_load_state,
	id_save_state,
#endif
	id_help_menu,

	id_special_effects_submenu,
	id_first_special_effect,
	id_last_special_effect=id_first_special_effect+25,//TODO should be a max value enum somewhere...
};

//////////////////////////////////////////////////////////////////////////
//
static wxString host_log_file;
static bool host_log_debug=false;
static bool host_log_lost=false;

static void HostLog(const char *str) {
	if(!host_log_file.empty()) {
		FILE *h=fopen(host_log_file,"a+t");
		if(!h) {
			host_log_lost=true;
		} else {
			if(host_log_lost) {
				fputs("*** some output was lost\n",h);
				host_log_lost=false;
			}

			fputs(str,h);
			fclose(h);
		}
	}

#ifdef bbcDEBUG_ENABLE
	OutputDebugStringA(str);
#endif
}

//////////////////////////////////////////////////////////////////////////
//
static void GeneratePalette(float palette[8][3],float rmul,float gmul,float bmul) {
	for(unsigned i=0;i<8;++i) {
		float r=i&1;
		float g=(i>>1)&1;
		float b=(i>>2)&1;
		float v=r*.3f+g*.59f+b*.11f;
		
		palette[i][0]=v*rmul;
		palette[i][1]=v*gmul;
		palette[i][2]=v*bmul;
	}
}

//////////////////////////////////////////////////////////////////////////
//a simple panel that ignores erase background and posts right click messages
//to its parent.
//the main emulator display is drawn on top of this.
class NullPanel:
public wxWindow
{
public:
	NullPanel(wxWindow *parent,wxWindowID id);
private:
	DECLARE_EVENT_TABLE();
	void OnEraseBackground(wxEraseEvent &event);
	void OnRightUp(wxMouseEvent &event);
};

BEGIN_EVENT_TABLE(NullPanel,wxWindow)
	EVT_ERASE_BACKGROUND(NullPanel::OnEraseBackground)
	EVT_RIGHT_UP(NullPanel::OnRightUp)
END_EVENT_TABLE()

NullPanel::NullPanel(wxWindow *parent,wxWindowID id):
wxWindow(parent,id,wxDefaultPosition,wxDefaultSize,0)
{
}

void NullPanel::OnEraseBackground(wxEraseEvent &event) {
	return;
}

void NullPanel::OnRightUp(wxMouseEvent &event) {
	wxWindow *parent=this->GetParent();
	if(parent) {
		parent->AddPendingEvent(event);
	}
}

//////////////////////////////////////////////////////////////////////////
//
mbMainFrame::SbState::SbState():
caps(false),
shift(false)
{
	for(unsigned i=0;i<bbcFdd::num_drives;++i) {
		this->drives[i].what=-1;//unloaded
	}
}

//////////////////////////////////////////////////////////////////////////
//
mbMainFrame::DriveUi::DriveUi():
//loaded(false),
last_filter_index(0)
{
}

//////////////////////////////////////////////////////////////////////////
//
bool mbMainFrame::DoCommandLine(const std::vector<wxString> &argv) {
	std::vector<twArg *> all_args;
	unsigned i;

	all_args.push_back(new twArgPresent("selectini",&cl_select_ini_));
	all_args.push_back(new twArgString("inifile",&cl_ini_file_));
	all_args.push_back(new twArgPresent("autoboot",&cl_autoboot_));
	all_args.push_back(new twArgMulti<twArgString>("inioverride",
		&cl_ini_override_files_));
	all_args.push_back(new twArgState("inisave",&cl_ini_save_));
	all_args.push_back(new twArgState("verbose",&cl_verbose_));
	for(i=0;i<4;++i) {
		all_args.push_back(new twArgString(wxString::Format("mount%u",i),
			&cl_drive_mounts_[i]));
	}
	all_args.push_back(new twArgString("hostlogfile",&host_log_file));
#if (defined bbcDEBUG_ENABLE)&&(defined bbcDEBUG_TRACE_EVENTS)
	all_args.push_back(new twArgPresent("withdisassembly",&cl_with_disassembly_));
#endif

	bool ok=twArg::ProcessAll(argv,all_args);
	for(i=0;i<all_args.size();++i) {
		delete all_args[i];
	}
	return ok;
}

//////////////////////////////////////////////////////////////////////////
//
mbMainFrame::mbMainFrame(const std::vector<wxString> &argv):
wxFrame(0,-1,window_title,wxDefaultPosition,wxDefaultSize,wxDEFAULT_FRAME_STYLE),
auto_paint_(false),
ok_(false),
//cfg_(cfg),
hostgfx_(0),
hostinp_(0),
hostsnd_(0),
num_frames_(0),
next_update_input_(0),
next_update_sys_(0),
next_update_sound_(0),
last_update_sys_time_(0),
debouncing_break_(false),
last_update_sound_hf_time_(0),
beeb_paused_(true),
//keymaps_submenu_(0),
screenshot_countdown_(0),
beeb_screen_(0),
status_bar_(0),
beeb_fastforward_(false),
cl_select_ini_(false),
cl_autoboot_(false),
cl_ini_save_(true),
//cl_verbose_ set up below
#if (defined bbcDEBUG_ENABLE)&&(defined bbcDEBUG_TRACE_EVENTS)
cl_with_disassembly_(false),
#endif
menu_bar_(0),
model_(0),
update_window_state_(false),
last_video_buffer_(0)
{
	int i;

	joystick_indexes_[0]=-1;
	joystick_indexes_[1]=-1;

#ifdef bbcDEBUG_ENABLE
	cl_verbose_=true;
#else
	cl_verbose_=false;
#endif

	Host_SetLogOutputCallback(&HostLog);

	if(!this->DoCommandLine(argv)) {
		return;
	}

#ifdef bbcHACKING_URIDIUM
	cl_drive_mounts_[0]="z:\\beeb\\src\\run\\uridium-test-version.ssd";
	cl_autoboot_=true;
#endif

	wxLog::GetActiveTarget()->SetVerbose(cl_verbose_);
	
	if(cl_select_ini_) {
		wxArrayString ini_names_arr;
		std::vector<wxFileName> ini_names;
		mbGetMatchingFileNames(wxFileName::DirName("."),"*.ini",&ini_names);
		if(ini_names.empty()) {
			wxLogError("No files matching \"*.ini\" for -selectini");
			return;
		}
		for(unsigned i=0;i<ini_names.size();++i) {
			ini_names_arr.Add(ini_names[i].GetName());
		}
		int index=wxGetSingleChoiceIndex("Select INI file to use","-selectini",
			ini_names_arr);
		if(index<0) {
			return;
		} else {
			cl_ini_file_=mbNormalizedFileName(ini_names[index].GetFullPath());
		}
	}

	if(cl_ini_file_.empty()) {
		cl_ini_file_="./beeb.ini";
	}
	cfg_.Load(cl_ini_file_);
	for(i=0;i<int(cl_ini_override_files_.size());++i) {
		cfg_.Load(cl_ini_override_files_[i]);
	}

	//By default, ini is readonly if overrides were specified. This stops override
	//settings leaking into the main file. However, this may be overridden with
	//-inisave.
	cfg_.SetFilename(cl_ini_file_,!cl_ini_override_files_.empty()&&!cl_ini_save_);

	//Child windows
	beeb_screen_=new NullPanel(this,-1);//,wxDefaultPosition,wxDefaultSize,wxNO_BORDER);
	beeb_screen_->SetBackgroundColour(*wxBLACK);
	beeb_screen_->Show(true);
	status_bar_=new mbStatusBar(this,id_status_bar);//,wxST_SIZEGRIP);
	wxBoxSizer *all_sz=new wxBoxSizer(wxVERTICAL);
	all_sz->Add(beeb_screen_,1,wxEXPAND);
	this->SetSizer(all_sz);
//	wxSlider *test=new wxSlider(status_bar_,-1,5,0,10);
//	test->Move(0,0);
//	test->SetSize(200,10);

	//host
	ok_=true;
	auto_paint_=true;
#ifdef __WXMSW__
	//grr why does GetHandle() not return an HWND???
	hostgfx_=HostGfx_Start(HWND(this->GetHandle()),HWND(beeb_screen_->GetHandle()));
#else
#error
#endif
	if(!hostgfx_) {
		wxMessageBox("Failed to initialise graphics.\n\nThe emulator will now exit.",
			"Graphics failed",wxOK|wxICON_ERROR,this);
		ok_=false;
		return;
	}

	hostinp_=HostInp_Start(this->GetHandle());
	if(!hostinp_) {
		wxMessageBox("Failed to initialise keyboard.\n\nThe emulator will now exit.",
			"Keyboard failed",wxOK|wxICON_ERROR,this);
		ok_=false;
		return;
	}
	HostTmr_Start();
	HostTmr_GetHfFrequency(&hf_freq_);
	
	this->InitMenuBar();

	bbcComputer::SetCmosContents(cfg_.cmos.contents);
	
	if(!this->ApplyConfig()) {
		ok_=false;
	} else {
		//Mount startup drives
		for(i=0;i<bbcFdd::num_drives;++i) {
			if(!cl_drive_mounts_[i].empty()) {
				this->UiLoadDisc(i,cl_drive_mounts_[i]);
			}
		}

		if(cl_autoboot_) {
			bbcKeyboardMatrix::SetForceShiftPressed(true);
		}
	}

#if (defined bbcDEBUG_ENABLE)&&(defined bbcDEBUG_TRACE_EVENTS)
	if(cl_with_disassembly_) {
		this->OnStartDisassembly(wxCommandEvent());
	}
#endif
}

//////////////////////////////////////////////////////////////////////////
bool mbMainFrame::Ok() const {
	return ok_;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::SetKeymap(const wxString &keymap_name) {
	mbKeymap::C::iterator src;

	for(src=cfg_.keyboard.keymaps.begin();src!=cfg_.keyboard.keymaps.end();++src) {
		if(keymap_name.CmpNoCase(src->name)==0) {
			break;
		}
	}

	for(unsigned i=0;i<256;++i) {
		keymap_[i].clear();
	}

	if(src==cfg_.keyboard.keymaps.end()) {
		wxLogWarning("SetKeymap: \"%s\": not found",keymap_name.c_str());
	}

	for(unsigned i=0;i<256;++i) {
		mbKeymap::HostKeys::const_iterator srckey;

		for(srckey=src->keymap[i].begin();srckey!=src->keymap[i].end();++srckey) {
			if(srckey->device.empty()) {
				unsigned host_code=HostInp_CodeFromKeyName(srckey->key);
				if(host_code==HostInp_bad_key) {
					wxLogWarning("SetKeymap: Unknown key \"%s\"",srckey->key.c_str());
				} else if(host_code>0xFFFF) {
					wxLogWarning("SetKeymap: Key \"%s\" cannot be used",srckey->key.c_str());
				} else {
					Key destkey;

					destkey.is_joystick=false;
					destkey.index=0;
					destkey.code=host_code;

					keymap_[i].push_back(destkey);

					//wxLogDebug(__FUNCTION__ ": %s -> code=%u",srckey->key.c_str(),destkey.code);
				}
			} else {
				int index;
				mbJoyKey code;

				mbJoyKeyGetIndexAndKey(hostinp_,*srckey,&index,&code);
				if(code==mbJOYKEY_INVALID) {
					wxLogWarning("SetKeymap: Unknown joystick key \"%s\"",srckey->key.c_str());
				} else if(index>=0) {
					Key destkey;

					destkey.is_joystick=true;
					destkey.index=index;
					destkey.code=code;

					keymap_[i].push_back(destkey);

					//wxLogDebug(__FUNCTION__ ": %s:%s -> index=%u code=%u",srckey->device.c_str(),srckey->key.c_str(),destkey.index,destkey.code);
				}
			}
		}
	}

	cfg_.keyboard.selected_keymap=keymap_name;
}

//////////////////////////////////////////////////////////////////////////
// Returns true if closing is OK.
//
// If there's no changes to be saved, return true; otherwise, pop up a
// dialog box asking for confirmation.
bool mbMainFrame::CloseConfirmed() {
	for(int i=0;i<bbcFdd::num_drives;++i) {
		if(bbcFdd::DriveStatus(i)!=bbcFdd::DRIVE_SIDE2&&
			!this->UiUnloadDrive(i))
		{
			return false;
		}
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnClose(wxCloseEvent &event) {
	if(event.CanVeto()) {
		if(!this->CloseConfirmed()) {
			event.Veto();
			return;
		}
	}

	//Retrieve CMOS contents
	bbcComputer::GetCmosContents(cfg_.cmos.contents);

	cfg_.Save();

	//the main frame can receive more idle events if any dialogs are
	//popped up whilst processing this next bit.
	//This stops anything being done with them.
	beeb_paused_=true;

	for(int i=0;i<bbcFdd::num_drives;++i) {
		bbcFdd::UnloadDiscImage(i);
	}
	HostSnd_Stop(hostsnd_);
	hostsnd_=0;
	HostInp_Stop(hostinp_);
	hostinp_=0;
	HostGfx_Stop(hostgfx_);
	hostgfx_=0;
	HostTmr_Stop();
//	this->Destroy();

	event.Skip();
}

/*
void mbMainFrame::UpdateStatusBar() {
	static const wxString caps_lock_on="CAPS LOCK";
	static const wxString shift_lock_on="SHIFT LOCK";
	wxStatusBar *sb=this->GetStatusBar();
	
	if(bbc_system_via.CapslockLedLit()) {
		sb->SetStatusText(caps_lock_on,0);
	} else {
		sb->SetStatusText("",0);
	}
	if(bbc_system_via.ShiftlockLedLit()) {

	}
}
*/

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::DoUpdateSys() {
	int this_time;
	HostTmr_ReadMilliseconds(&this_time);
	bbc1770::FdcAction action;
	int drive,side,track,sector;
	bbc1770::GetCurrentAction(&action,&drive,&side,&track,&sector);
	beeb_fastforward_=action!=bbc1770::ACTION_IDLE&&
		cfg_.misc.fast_forward_disc_access&&
		bbcKeyboardMatrix::NumKeysDown()==0;
//	if(this_time-last_update_sys_time_>100) 

#ifdef bbcDEBUG_PANELS
	mbDebugPanel *usr=static_cast<mbDebugPanel *>(this->FindWindow(id_dbgpanel_usrvia));
	mbDebugPanel *sys=static_cast<mbDebugPanel *>(this->FindWindow(id_dbgpanel_sysvia));
	mbDebugPanel *vid=static_cast<mbDebugPanel *>(this->FindWindow(id_dbgpanel_video));
	mbDebugPanel *adc=static_cast<mbDebugPanel *>(this->FindWindow(id_dbgpanel_adc));
	if(usr) {
		bbc_user_via.UpdateDebugPanel(usr->Panel());
		usr->UpdateFromPanel();
	}
	if(sys) {
		bbc_system_via.UpdateDebugPanel(sys->Panel());
		sys->UpdateFromPanel();
	}
	if(vid) {
		vid->UpdateFromPanel();
	}
	if(adc) {
		bbcAdc::UpdateDebugPanel(adc->Panel());
		adc->UpdateFromPanel();
	}
#endif


	BASSERT(this_time>=last_update_sys_time_);
	if(cfg_.misc.show_status_bar&&this_time-last_update_sys_time_>250) {
		//MHz
		static int last_time_cycles=0;
		int num_cycles=bbcComputer::cpucycles-last_time_cycles;
		float num_seconds=(this_time-last_update_sys_time_)/1000.f;
		float mhz=num_cycles/num_seconds/1000000.f;
		status_bar_->SetMhz(mhz);

#ifdef mbFAKE_KEYPRESSES
		this->SetTitle(wxString::Format("%s - %d",window_title,bbcComputer::cpucycles));
#endif

#ifdef bbcDEBUG_TRACE
		//Debug trace (bit of a hack at the mo')
		if(bbcComputer::trace) {
			status_bar_->SetDebugTraceSize(bbcComputer::trace->size());
		} else {
			status_bar_->SetDebugTraceSize(0);
		}
#endif

		//Screen
		int screen_w,screen_h;
		beeb_screen_->GetClientSize(&screen_w,&screen_h);
		status_bar_->SetScreenSize(screen_w,screen_h);

		//FDC status
		int real_drive=bbcFdd::DriveFromFdcAddress(drive,side);
		status_bar_->SetFdcStatus(action,real_drive,track,sector);

#ifdef bbcDEBUG_ENABLE
		//Program counter
		status_bar_->SetPc(bbcComputer::cpustate.pc);
#endif

		//
		last_time_cycles=bbcComputer::cpucycles;
		last_update_sys_time_=this_time;
	}
	next_update_sys_=bbcComputer::cpucycles+update_sys_cycles;
	//wxLogDebug("this_time=%u next_update_sys_=%d bbcComputer::cpucycles=%d\n",this_time,next_update_sys_,bbcComputer::cpucycles);
#ifdef __WXMSW__
	//This gives slightly smoother multitasking.
	//Windows will allegedly preemptively multitask GUI apps these days,
	//seems it does it very politely :)
	Sleep(0);
#endif
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnIdle(wxIdleEvent &event) {
	if(beeb_paused_) {
		event.Skip();
	} else {
		this->DoBeebTick();
		this->DoUpdateSys();
		event.RequestMore();//TODO shouldn't request more if we're being shut down?
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::DoUpdateInput() {
	BYTE key_states[256];
	int num_joysticks=HostInp_NumJoysticks(hostinp_);
    
	if(!hostinp_) {
		memset(key_states,0,sizeof key_states);
	} else {
		HostInp_Poll(hostinp_);
		HostInp_GetKeyboardState(hostinp_,key_states);

		if(num_joysticks!=joykey_states_.size()) {
			joykey_states_.resize(num_joysticks);
		}

		for(int i=0;i<num_joysticks;++i) {
			HostInp_JoyState state;

			if(HostInp_GetJoystickState(hostinp_,i,&state)) {
				mbJoyKeyGetKeyStates(&state,&joykey_states_[i]);
			} else {
				memset(&joykey_states_[i],0,sizeof joykey_states_[i]);
			}
		}
	}

#ifdef bbcHACKING_URIDIUM
	bool user_space=false;//whether user has pressed space
#endif

#ifndef mbFAKE_KEYPRESSES
	for(unsigned i=0;i<256;++i) {
		if(!keymap_[i].empty()) {
			bool key_state=false;

			unsigned n=keymap_[i].size();

			for(unsigned j=0;j<n;++j) {
				const Key *key=&keymap_[i][j];

				if(!key->is_joystick) {
					if(key_states[key->code]) {
						key_state=true;
						break;
					}
				} else {
					if(key->index<num_joysticks) {
						if(joykey_states_[key->index].states[key->code]) {
							key_state=true;
							break;
						}
					}
				}
			}
			bbcKeyboardMatrix::SetKeyState(bbcKey(i),key_state);
#ifdef bbcHACKING_URIDIUM
			if(i==bbcKEY_SPACE&&key_state) {
				user_space=true;
			}
#endif
		}
	}
#endif

#ifdef mbFAKE_KEYPRESSES
	for(unsigned i=0;i<num_fake_actions;++i) {
		const FakeAction *fa=&fake_actions[i];
		bool active=fa->cycles>=bbcComputer::cpucycles&&fa->cycles<bbcComputer::cpucycles+update_input_cycles;

		if(active) {
			bbcPrintf("fa->type=%d fa->cycles=%d fa->bbc_code=%s\n",fa->type,fa->cycles,mbKeyNameFromCode(fa->bbc_code));
			switch(fa->type) {
			case FAKE_KEY_DOWN:
				bbcKeyboardMatrix::SetKeyState(fa->bbc_code,true);
				break;

			case FAKE_KEY_UP:
				bbcKeyboardMatrix::SetKeyState(fa->bbc_code,false);
				break;

			case FAKE_START_DISASSEMBLY:
				bbcPrintf("faking start disassembly");
				this->OnStartDisassembly(wxCommandEvent());
				break;

			case FAKE_STOP_DISASSEMBLY:
				bbcPrintf("faking stop disassembly");
				this->OnStopDisassembly(wxCommandEvent());
				break;
			}
		}
	}
#endif

#ifdef bbcHACKING_URIDIUM
	static const int space_needed=6300000;
//	static const int down_for=5*400000;//N frames (5 cocks it)
	static const int down_for=4*400000;//N frames
	bbcKeyboardMatrix::SetKeyState(bbcKEY_SPACE,!user_space&&bbcComputer::cpucycles>=space_needed&&
		bbcComputer::cpucycles<=space_needed+down_for);

#endif

	bbc_system_via.DoKeyboardIrq();
	if(bbcKeyboardMatrix::KeyState(bbcKEY_BREAK)&&!debouncing_break_) {	
		bbcComputer::ResetSoft();
	} else if(!bbcKeyboardMatrix::KeyState(bbcKEY_BREAK)) {
		debouncing_break_=false;
	}

	//And now, joysticks.
	for(int i=0;i<2;++i) {
		mbJoysticksConfig::BeebStick *stick=&cfg_.joysticks.joysticks[i];
		bool button=false;
		float x=0.f,y=0.f;

		if(hostinp_&&stick->enabled&&joystick_indexes_[i]>=0) {
			HostInp_JoyState state;

			if(HostInp_GetJoystickState(hostinp_,joystick_indexes_[i],&state)) {

				if(stick->x_axis!=mbJoysticksConfig::INVALID_AXIS) {
					x=state.*mbJoysticksConfig::hostinp_axes[stick->x_axis];
				}
				if(stick->y_axis!=mbJoysticksConfig::INVALID_AXIS) {
					y=state.*mbJoysticksConfig::hostinp_axes[stick->y_axis];
				}

				if(stick->button>=0&&stick->button<HostInp_num_joystick_buttons&&
					state.buttons[stick->button])
				{
					button=true;
				}
			}
		}

		bbcAdc::SetJoystick(i,x,y);
		bbc_system_via.SetJoystickButtonState(i,button);
	}
}

//////////////////////////////////////////////////////////////////////////

wxMenu *CloneMenu(wxMenu *src) {
	wxMenu *dest=new wxMenu;
	wxMenuItemList &items=src->GetMenuItems();

	for(wxMenuItemList::Node *node=items.GetFirst();node;node=node->GetNext()) {
		wxMenuItem *item=node->GetData();
		wxMenu *submenu=item->GetSubMenu();
		int id=item->GetId();
		wxString text=item->GetText();
		wxString help=item->GetHelp();
		
		if(submenu) {
			dest->Append(id,text,CloneMenu(submenu),help);
		} else {
			switch(item->GetKind()) {
			case wxITEM_SEPARATOR:
				dest->AppendSeparator();
				break;

			case wxITEM_NORMAL:
				dest->Append(id,text,help);
				break;

			case wxITEM_CHECK:
				dest->AppendCheckItem(id,text,help);
				break;

			default:
				BASSERT(false);
				break;
			}
		}
	}

	return dest;
}

void mbMainFrame::InitMenuBar() {
	unsigned i;

	wxMenu *file=new wxMenu;
#ifdef ENABLE_TEST
	file->Append(id_test,"&Test...");
#endif
	file->Append(id_quickstart,"&Quickstart...","Select game to boot from menu");
	file->Append(id_hard_reset,"&Hard reset...","Reset BBC as if switched off then on");
#ifdef bbcENABLE_SAVE_STATE
	file->AppendSeparator();
	file->Append(id_load_state,"&Load state...");
	file->Append(id_save_state,"&Save state...");
#endif
	file->AppendSeparator();
	for(i=0;i<bbcFdd::num_drives;++i) {
		wxString drive_num=wxString::Format("%1.1d",i);
		wxString str;
		wxMenu *drive_menu;
		int base_id=id_drives_begin+i*subid_drive_num;

		drive_menu=new wxMenu;
		drive_menu->Append(base_id+subid_drive_filename,"");
		drive_menu->Enable(base_id+subid_drive_filename,false);
		drive_menu->AppendSeparator();
		drive_menu->Append(base_id+subid_drive_load,"&Load...",
			"Insert disc into drive "+drive_num);
		drive_menu->Append(base_id+subid_drive_save,"&Save",
			"Save disc currently in "+drive_num);
		drive_menu->Append(base_id+subid_drive_save_as,"Save &as...",
			"Save disc currently in "+drive_num+" under a new name");
		drive_menu->Append(base_id+subid_drive_unload,"&Unload",
			"Remove disc from "+drive_num);
		file->Append(base_id+subid_drive,"Drive &"+drive_num,drive_menu,
			"Controls for drive "+drive_num);
	}
	file->AppendSeparator();
	file->Append(id_exit,"E&xit","Exit the program");
	
	wxMenu *options=new wxMenu;
	options->AppendCheckItem(id_toggle_sound,"&Sound enabled",
		"Whether sound will be produced");
	options->AppendCheckItem(id_toggle_limit_speed,"&Limit speed",
		"Whether emulator is limited to max real BBC speed");
	options->AppendCheckItem(id_toggle_fast_forward_disc_access,
		"&Fast forward disc access",
		"Whether emulator should run flat out when acessing the disc");
//	options->AppendCheckItem(id_toggle_vertical_2x,"&Vertical doubling",
//		"Whether to double each scanline");
	options->AppendCheckItem(id_toggle_full_screen,"&Full screen");
	options->AppendCheckItem(id_toggle_status_bar,"&Show status bar",
		"Whether status bar is visible");
	wxMenu *keymaps_submenu=new wxMenu;
	options->Append(id_keymaps_submenu,"&Keymap",keymaps_submenu,
		"Select keyboard mapping");
	wxMenu *resolution_submenu=new wxMenu;
	for(i=0;i<4;++i) {
		resolution_submenu->Append(id_resolution_0+i,"Temp");
	}
	options->Append(id_resolution_submenu,"&Resolution",resolution_submenu,
		"Select window size");
	wxMenu *screen_type_submenu=new wxMenu;
	screen_type_submenu->AppendCheckItem(id_screen_type_colour,"&Colour",
		"Colour TV/monitor");
	screen_type_submenu->AppendCheckItem(id_screen_type_bnw,"&Black && White",
		"Black && white TV");
	screen_type_submenu->AppendCheckItem(id_screen_type_green,"&Green",
		"Green screen monitor (green only)");
	screen_type_submenu->AppendCheckItem(id_screen_type_amber,"&Amber",
		"Amber monitor");
	options->Append(id_screen_type_submenu,"S&creen type",screen_type_submenu,
		"Select screen type");

	{
		const char **sfxs=HostGfx_SpecialEffects(hostgfx_);
		unsigned n=0;
		while(sfxs[n]) {
			++n;
		}
		if(true) {//n>1) {
			wxMenu *sfx_menu=new wxMenu;
			for(unsigned i=0;i<n;++i) {
				sfx_menu->AppendCheckItem(id_first_special_effect+i,sfxs[i]);
			}
			options->Append(id_special_effects_submenu,"Video effects",sfx_menu);
		}
	}

	{
		wxMenu *wsc=new wxMenu;
		wsc->AppendCheckItem(id_windowed_scanlines_single,"&Single");
		wsc->AppendCheckItem(id_windowed_scanlines_interlace,"&Interlace");
		wsc->AppendCheckItem(id_windowed_scanlines_double,"&Double");
		wxMenu *fssc=new wxMenu;
		fssc->AppendCheckItem(id_full_screen_scanlines_single,"&Single");
		fssc->AppendCheckItem(id_full_screen_scanlines_interlace,"&Interlace");
		fssc->AppendCheckItem(id_full_screen_scanlines_double,"&Double");
		options->Append(id_windowed_scanlines_submenu,"&Windowed scanlines",wsc);
		options->Append(id_full_screen_scanlines_submenu,"&Full screen scanlines",
			fssc);
	}
	options->AppendSeparator();
	options->Append(id_roms_dialog,"&ROMs...","Configure sideways ROMs and OS ROM");
	options->Append(id_sound_dialog,"S&ound...","Configure sound output");
	options->Append(id_keymap_dialog,"K&eymaps...","Define keyboard mappings");
	
	if(HostInp_SupportsJoysticks(hostinp_)) {
		options->Append(id_joysticks_dialog,"&Joysticks...","Configure joysticks");
	}

	options->Append(id_hardware_dialog,"&Hardware...");
	options->Append(id_video_dialog,"&Video...");
//	options->AppendCheckItem(id_toggle_toolbar,"&Toolbar","Toggle toolbar display");

#ifdef bbcDEBUG_ENABLE
	wxMenu *debug=new wxMenu;
#if (defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS)
	debug->Append(id_start_disassembly,"St&art disassembly",
		"Start saving a running disassembly");
	debug->Append(id_stop_disassembly,"St&op disassembly",
		"Stop the running disassembly and save results to disc");
	debug->Append(id_reboot_with_disassembly,"Reboot with disassembly",
		"Reboot computer with disassembly running");
#endif
#ifdef BEEB_DEBUG_SAVE_CHALLENGER_RAM
	debug->Append(id_save_challenger_ram,"Save &CHALLENGER RAM",
		"Save CHALLENGER RAM to a file (if CHALLENGER enabled)");
#endif
#ifdef bbcDEBUG_ENABLE
	debug->Append(id_disassemble_ram,"&Disassemble RAM",
		"Save a disassembly of all of RAM");
#endif
#ifdef bbcDEBUG_PANELS
	debug->AppendCheckItem(id_toggle_dbgpanel_sysvia,"S&ystem VIA debug panel");
	debug->AppendCheckItem(id_toggle_dbgpanel_usrvia,"&User VIA debug panel");
	debug->AppendCheckItem(id_toggle_dbgpanel_video,"&Video debug panel");
	debug->AppendCheckItem(id_toggle_dbgpanel_adc,"&ADC debug panel");
#endif
#ifdef bbcDEBUG_VIDEO
	debug->AppendCheckItem(id_toggle_showall,"Show all");
	debug->Append(id_log_video_frames,"Log a few video frames");
#ifdef bbcDEBUG_TRACE
	debug->Append(id_log_video_frames_disasm,"Log a few video frames w/ disasm");
#endif//bbcDEBUG_TRACE
	debug->AppendCheckItem(id_toggle_show_t1_timeouts,"Show sys (R)/usr (G) T1 timeouts");
#endif//bbcDEBUG_VIDEO
#ifdef t65_COUNT_INSTRUCTION_FREQUENCIES
	debug->Append(id_show_instruction_frequencies,"Show instruction frequencies (by opcode)");
	debug->Append(id_show_instruction_frequencies_sorted,"Show instruction frequencies (by freq)");
#endif//t65_COUNT_INSTRUCTION_FREQUENCIES
#ifdef bbcDEBUG_COUNT_MMIO_ACCESSES
	debug->Append(id_show_sheila_access_counts_by_addr,"Show SHEILA access counts (by addr)");
	debug->Append(id_show_sheila_access_counts_by_count,"Show SHEILA access counts (by count)");
#endif
#endif//bbcDEBUG_ENABLE

	wxMenu *special=new wxMenu;
	special->Append(id_sound_record_start,"&Start recording sound");
	special->Append(id_sound_record_stop,"Stop &recording sound...");
	special->AppendSeparator();
	special->Append(id_screenshot_save_as,"&Save screenshot...",
		"Save screen contents to disc");

	wxMenu *help=new wxMenu;
	help->Append(id_show_about_box,"&About...","Show help box");

#ifdef NEW_MENU_STUFF
	int brk=0;
#else
	menu_bar_=new wxMenuBar;

	menu_bar_->Append(file,"File");
	popup_menu_.Append(id_file_menu,"File",CloneMenu(file));

	menu_bar_->Append(options,"Options");
	popup_menu_.Append(id_options_menu,"Options",CloneMenu(options));

	menu_bar_->Append(special,"Special");
	popup_menu_.Append(id_special_menu,"Special",CloneMenu(special));

#ifdef bbcDEBUG_ENABLE
	menu_bar_->Append(debug,"Debug");
	popup_menu_.Append(id_debug_menu,"Debug",CloneMenu(debug));
#endif

	menu_bar_->Append(help,"Help");
	popup_menu_.Append(id_help_menu,"Help",CloneMenu(help));
#endif

	this->SetMenuBar(menu_bar_);

	printf("id=%d\n",menu_bar_->FindItem(id_keymaps_submenu));

//	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::ModalBackground() {
	if(hostsnd_) {
		HostSnd_PlayStop(hostsnd_);
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::GetConfig(mbConfig *cfg) const {
	wxASSERT(cfg);
	*cfg=cfg_;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnMenuOpen(wxMenuEvent &event) {
	wxLogDebug("OnMenuOpen: this->ModalBackground()");
	this->ModalBackground();
	//HostGfx_SetFullScreen(hostgfx_,false);
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnMenuClose(wxMenuEvent &event) {
//	event.Skip();
	if(hostgfx_&&HostGfx_IsFullScreen(hostgfx_)) {
		//HostGfx_Cls(hostgfx_);
		wxLogDebug("OnMenuClose: clearing beeb screen.");
		//beeb_screen_->Clear();
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnExit(wxCommandEvent &event) {
	this->Close();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnToggleSound(wxCommandEvent &event) {
	this->SetSoundEnabled(!cfg_.sound.enabled);
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnToggleLimitSpeed(wxCommandEvent &event) {
	cfg_.misc.limit_speed=!cfg_.misc.limit_speed;
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnShowAboutBox(wxCommandEvent &event) {
	this->ModalBackground();
	wxString msg;
	
	msg+=wxString::Format("This is version %s\n",mb_version);
	msg+="\n";
	msg+="Tom Seddon <modelb@bbcmicro.com>\n";
	msg+="\n";
	msg+=wxString::Format("Compiled at %s on %s\n",__TIME__,__DATE__);
	wxMessageDialog about(this,msg,"Author",wxOK|wxCENTRE|wxICON_INFORMATION);
	about.ShowModal();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnPaint(wxPaintEvent &event) {
	wxPaintDC dc(this);
//	if(auto_paint_) {

	wxLogDebug(__FUNCTION__ ": painting.");
	if(!hostgfx_||!last_video_buffer_) {
		wxLogDebug(__FUNCTION__ ":     blanking; hostgfx_%p last_video_buffer_=%p",hostgfx_,last_video_buffer_);
		dc.SetBrush(*wxBLACK_BRUSH);
		dc.Clear();
		dc.SetBrush(wxNullBrush);
	} else {
		//Interlace won't look very good, because we're not drawing very often. So pretend
		//interlaced mode is double mode (which it is, kind of). Pass dummy frame number to
		//PresentLastVideoBuffer as it's not required.
		mbScanlinesMode scanlines_mode=HostGfx_IsFullScreen(hostgfx_)?cfg_.video.full_screen_scanlines:cfg_.video.windowed_scanlines;
		if(scanlines_mode==mbSCANLINES_INTERLACE) {
			scanlines_mode=mbSCANLINES_DOUBLE;
		}

		this->PresentLastVideoBuffer(scanlines_mode,HostGfx_PRESENT_HINT_TIDY,0);
		//wxLogDebug(__FUNCTION__ ":     doing nowt, this bit not done yet.");
		//HostGfx_Present()
//		if(HostGfx_IsFullScreen(hostgfx_)) {
//			wxLogDebug(__FUNCTION__ ":     full screen, last_video_buffer_=%p",last_video_buffer_);
//			if(last_video_buffer_) {
//				HostGfx_Present(hostgfx_,last_video_buffer_->data,last_video_buffer_->width,last_video_buffer_->height,
//					last_video_buffer_->pitch);
//			}
//		} else {
//			wxLogDebug(__FUNCTION__ ":     windowed.");
//			HostGfx_ShowWindowed(hostgfx_);
//		}
	}
//	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnEraseBackground(wxEraseEvent &event) {
	return;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnSize(wxSizeEvent &event) {
	this->ModalBackground();
//	wxSize new_size=event.GetSize();
//	this->SetSize(new_size.GetWidth(),400);
	this->RefreshCfgWindowState();
	event.Skip();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnMove(wxMoveEvent &event) {
	this->ModalBackground();
	this->RefreshCfgWindowState();
	event.Skip();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::SetAutoPaint(bool new_auto_paint) {
	auto_paint_=new_auto_paint;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::DoUpdateSound() {
	HostTmr_HfValue now_hf_time;

	HostTmr_ReadHf(&now_hf_time);
	
	if(cfg_.misc.limit_speed&&!beeb_fastforward_) {
		HostTmr_HfValue sound_tick_length=hf_freq_/update_sound_freq;
		while(now_hf_time-last_update_sound_hf_time_<sound_tick_length) {
#ifdef __WXMSW__
//			Sleep(0);
#endif
			HostTmr_ReadHf(&now_hf_time);
		}
/*
		wxString tmp;
		tmp.Printf("%I64d/%I64d",__int64(now_hf_time-last_update_sound_hf_time_),
			__int64(hf_freq_));
		this->SetTitle(tmp);
*/
	}

	if(!cfg_.sound.enabled||!hostsnd_) {
		BASSERT(!cfg_.sound.enabled||(cfg_.sound.enabled&&hostsnd_));
		//doesn't matter whether the 8 or 16 bit version is used here!
		bbcSound::Render16BitMono(0,0,0);
	} else {
		HostSnd_PlayStart(hostsnd_);
		void *ps[2];
		unsigned sizes[2];
		//sound_buffer_->GetCurrentPosition(&snd_play,&snd_write);
		int flag=HostSnd_GetWriteArea(hostsnd_,&ps[0],&sizes[0],&ps[1],&sizes[1]);
		if(flag<0) {
			bbcSound::Render16BitMono(0,0,0);
		} else if(flag>0) {
			static void (*(sound_renderers[]))(void **,unsigned *,unsigned)={
				&bbcSound::Render8BitMono,
				&bbcSound::Render8BitStereo,
				&bbcSound::Render16BitMono,
				&bbcSound::Render16BitStereo,
			};
			unsigned num_buffers=sizes[1]>0?2:1;
			unsigned idx=(cfg_.sound.stereo?1:0)|(cfg_.sound.bits==16?2:0);
			(*sound_renderers[idx])(ps,sizes,num_buffers);

			HostSnd_CommitWriteArea(hostsnd_);
		}
	}
	last_update_sound_hf_time_=now_hf_time;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::SetSoundEnabled(bool new_sound_enabled) {
	if(new_sound_enabled) {
		if(!hostsnd_) {
			OutputDebugStringA("mbMainFrame::SetSoundEnabled(true): HostSnd_Start\n");
			hostsnd_=HostSnd_Start(this->GetHandle());
			if(hostsnd_) {
				OutputDebugStringA("mbMainFrame::SetSoundEnabled(true): HostSnd_* init stuff\n");
				int len_samples=cfg_.sound.hz/update_sound_freq*cfg_.sound.len_frames;
				wxLogDebug("Sound: %dHz %d bits %s %d samples",
					cfg_.sound.hz,cfg_.sound.bits,cfg_.sound.stereo?"stereo":"mono",
					len_samples);
				HostSnd_SetSoundParameters(hostsnd_,cfg_.sound.hz,cfg_.sound.bits,
					cfg_.sound.stereo?2:1,len_samples);
				bbcSound::SetFrequency(cfg_.sound.hz);
				for(unsigned i=0;i<4;++i) {
					bbcSound::SetChannelStereoLeftRight(i,
						cfg_.sound.is_stereo_left[i],cfg_.sound.is_stereo_right[i]);
				}
				HostSnd_PlayStart(hostsnd_);
//				Sleep(10);
//				HostSnd_SetVolume(hostsnd_,cfg_.sound.volume);
			}
		}
	} else {
		if(hostsnd_) {
			OutputDebugStringA("mbMainFrame::SetSoundEnabled(false): HostSnd_Stop\n");
			HostSnd_Stop(hostsnd_);
			hostsnd_=0;
		}
	}
	cfg_.sound.enabled=new_sound_enabled;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::RefreshWindow() {
	wxLogDebug(__FUNCTION__ ": called.\n");
	//firstly, set up display

	if(hostgfx_) {
		wxLogDebug("RefreshWindow: full screen=%s",cfg_.video.full_screen?"yes":"no");
//		if(cfg_.video.full_screen) {
//			HostGfx_SetScanlinesMode(hostgfx_,cfg_.video.full_screen_scanlines);
//		} else {
//			HostGfx_SetScanlinesMode(hostgfx_,cfg_.video.windowed_scanlines);
//		}

		float palette[8][3];
		
		switch(cfg_.video.monitor_type) {
		default:
			BASSERT(false);
			cfg_.video.monitor_type=mbMONITOR_COLOUR;
		case mbMONITOR_COLOUR:
			for(unsigned i=0;i<8;++i) {
				palette[i][0]=i&1;
				palette[i][1]=(i>>1)&1;
				palette[i][2]=(i>>2)&1;
			}
			break;
		case mbMONITOR_BNW:
			GeneratePalette(palette,1.f,1.f,1.f);
			break;
		case mbMONITOR_AMBER:
			GeneratePalette(palette,1.f,.8f,.1f);
			break;
		case mbMONITOR_GREEN:
			GeneratePalette(palette,.2f,.9f,.1f);
			break;
		}
		HostGfx_SetPalette(hostgfx_,palette);
		
		bbcVideo::SetPixelFixedBits(cfg_.video.full_screen?
			HostGfx_FULL_SCREEN_PALETTE_BASE:0);
		//	status_bar_->SetResizeable()
		static const long fs_style=wxFULLSCREEN_NOBORDER|wxFULLSCREEN_NOCAPTION|
			wxFULLSCREEN_NOMENUBAR;

		//inhibit any updating of window states, as the window is mucked about with
		//when switching between windowed and full screen modes.
		bool old_update_window_state=update_window_state_;
		update_window_state_=false;
		bool full_screen_error=false;
		if(cfg_.video.full_screen) {
			//There's a bug in wxWindows; it saves the curent window rect as the unmaximized size
			//if the window is maximized.
			if(this->IsMaximized()) {
				this->Maximize(false);//but this won't update the current is_maximized setting
			}
			wxLogDebug(__FUNCTION__ ": setting full screen: ShowFullScreen then HostGfx_SetFullScreen.\n");
			this->ShowFullScreen(true,fs_style);
			HostGfx_SetFullScreen(hostgfx_,true,cfg_.video.full_screen_refresh_rate);
			if(!HostGfx_IsFullScreen(hostgfx_)) {
				cfg_.video.full_screen=false;
				full_screen_error=true;
			}
		}
		
		if(!cfg_.video.full_screen) {
			wxLogDebug(__FUNCTION__ ": HostGfx_SetFullScreen then ShowFullScreen.\n");
			HostGfx_SetFullScreen(hostgfx_,false,cfg_.video.full_screen_refresh_rate);
			this->ShowFullScreen(false,fs_style);
			
			//Coming out of full screen, we won't ever be maximized, because we don't go in
			//maximized. So if we were maximized before, do that again.
			if(cfg_.misc.window_is_maximized) {
				this->Maximize(true);
			}
		}

		if(full_screen_error) {
			wxLogError("Failed to set up full screen mode.\nThe emulator will continue to run windowed.");
		}
		update_window_state_=old_update_window_state;
	}
	if(hostsnd_) {
		HostSnd_SetVolume(hostsnd_,cfg_.sound.volume);
	}
	
	//tweak ui
	this->SetMenuItemCheck(id_toggle_sound,cfg_.sound.enabled);
	this->SetMenuItemCheck(id_toggle_limit_speed,cfg_.misc.limit_speed);
	this->SetMenuItemCheck(id_toggle_status_bar,cfg_.misc.show_status_bar);
	this->SetMenuItemCheck(id_toggle_fast_forward_disc_access,
		cfg_.misc.fast_forward_disc_access);
	//this->SetMenuItemCheck(id_toggle_vertical_2x,cfg_.video.vertical_2x);
	this->SetMenuItemCheck(id_toggle_full_screen,cfg_.video.full_screen);
	
#ifdef bbcDEBUG_PANELS
	this->SetMenuItemCheck(id_toggle_dbgpanel_sysvia,!!this->FindWindow(id_dbgpanel_sysvia));
	this->SetMenuItemCheck(id_toggle_dbgpanel_usrvia,!!this->FindWindow(id_dbgpanel_usrvia));
	this->SetMenuItemCheck(id_toggle_dbgpanel_video,!!this->FindWindow(id_dbgpanel_video));
#endif
#ifdef bbcDEBUG_VIDEO
	this->SetMenuItemCheck(id_toggle_showall,bbcVideo::show_all_);
	this->SetMenuItemCheck(id_toggle_show_t1_timeouts,bbcVideo::show_t1_timeouts_);
#endif

	for(unsigned i=0;i<4;++i) {
		int id=id_resolution_0+i;
		wxSize size=this->ResolutionFromId(id);
		wxString text=wxString::Format("%dx%d",size.GetWidth(),size.GetHeight());
		this->SetMenuItemText(id,text);
	}

//	HostGfx_BeebScreenType screen_type=HostGfx_COLOUR;
//	if(hostgfx_) {
//		screen_type=HostGfx_ScreenType(hostgfx_);
//	}
	this->SetMenuItemCheck(id_screen_type_colour,cfg_.video.monitor_type==mbMONITOR_COLOUR);
	this->SetMenuItemCheck(id_screen_type_bnw,cfg_.video.monitor_type==mbMONITOR_BNW);
	this->SetMenuItemCheck(id_screen_type_amber,cfg_.video.monitor_type==mbMONITOR_AMBER);
	this->SetMenuItemCheck(id_screen_type_green,cfg_.video.monitor_type==mbMONITOR_GREEN);

	this->SetMenuItemCheck(id_windowed_scanlines_single,
		cfg_.video.windowed_scanlines==mbSCANLINES_SINGLE);
	this->SetMenuItemCheck(id_windowed_scanlines_interlace,
		cfg_.video.windowed_scanlines==mbSCANLINES_INTERLACE);
	this->SetMenuItemCheck(id_windowed_scanlines_double,
		cfg_.video.windowed_scanlines==mbSCANLINES_DOUBLE);
	this->SetMenuItemCheck(id_full_screen_scanlines_single,
		cfg_.video.full_screen_scanlines==mbSCANLINES_SINGLE);
	this->SetMenuItemCheck(id_full_screen_scanlines_interlace,
		cfg_.video.full_screen_scanlines==mbSCANLINES_INTERLACE);
	this->SetMenuItemCheck(id_full_screen_scanlines_double,
		cfg_.video.full_screen_scanlines==mbSCANLINES_DOUBLE);

	//this isn't very efficient, but it doesn't make things slower :)
	for(int i=id_first_special_effect;i<=id_last_special_effect;++i) {
		this->SetMenuItemCheck(i,i-id_first_special_effect==cfg_.video.video_effect);
	}

	bool is_fs=hostgfx_&&HostGfx_IsFullScreen(hostgfx_);
//	this->SetMenuItemEnabled(id_resolution_0,!is_fs);
//	this->SetMenuItemEnabled(id_resolution_1,!is_fs);
//	this->SetMenuItemEnabled(id_resolution_2,!is_fs);
//	this->SetMenuItemEnabled(id_resolution_3,!is_fs);
	popup_menu_.Enable(id_resolution_submenu,!is_fs);//TODO doesn't work, don't know why.
	this->SetMenuItemEnabled(id_resolution_submenu,!is_fs);
	
	status_bar_->Show(false);
	wxASSERT(this->GetSizer());
	this->GetSizer()->Remove(status_bar_);
	if(cfg_.misc.show_status_bar) {
		status_bar_->Show(true);
		status_bar_->SetVolumeEnabled(cfg_.sound.enabled);
		status_bar_->SetVolume(cfg_.sound.volume);
		this->GetSizer()->Add(status_bar_,0,wxEXPAND);
	}
	this->GetSizer()->Layout();

	for(int i=0;i<bbcFdd::num_drives;++i) {
		bbcFdd::DriveStatusType st=bbcFdd::DriveStatus(i);
		int id_base=id_drives_begin+i*subid_drive_num;

		if(st==bbcFdd::DRIVE_SIDE2) {
			this->SetMenuItemEnabled(id_base+subid_drive,false);
		} else {
			const DriveUi *ui=&drive_uis_[i];
			this->SetMenuItemEnabled(id_base+subid_drive,true);
			bool loaded=st!=bbcFdd::DRIVE_EMPTY;
			//this->SetMenuItemEnabled(id_base+subid_drive_load,!loaded);
			this->SetMenuItemEnabled(id_base+subid_drive_save,loaded&&!ui->no_save);
			this->SetMenuItemEnabled(id_base+subid_drive_save_as,loaded&&!ui->no_save);
			this->SetMenuItemEnabled(id_base+subid_drive_unload,loaded);

			wxMenuItem *item=menu_bar_->FindItem(id_base+subid_drive_filename);
			wxASSERT(item);
			if(!loaded) {
				item->SetText("");
			} else {
				item->SetText(ui->disc_image.GetFullPath());
			}
		}
	}

	//Update keymaps
	this->RefreshKeymapsSubmenu(menu_bar_->FindItem(id_keymaps_submenu));
	this->RefreshKeymapsSubmenu(popup_menu_.FindItem(id_keymaps_submenu));

	//Update joystick indexes
	if(hostinp_) {
		for(int i=0;i<2;++i) {
			int num_joysticks=HostInp_NumJoysticks(hostinp_);
			for(int j=0;j<num_joysticks;++j) {
				const char *device_name=HostInp_JoystickName(hostinp_,j);;
				if(cfg_.joysticks.joysticks[j].device_name.CmpNoCase(device_name)==0) {
					break;
				}
			}

			joystick_indexes_[i]=j<num_joysticks?j:-1;
		}
	}

	//
	if(this->GetMenuBar()) {
		wxASSERT(this->GetMenuBar()==menu_bar_);
		menu_bar_->Refresh();
	}

	//Clear any rubbish that may have accumulated.
	//This is done too often. It doesn't matter.
	beeb_screen_->Clear();
}

//////////////////////////////////////////////////////////////////////////

void mbMainFrame::RefreshKeymapsSubmenu(wxMenuItem *item) {
	BASSERT(item);
	wxMenu *keymaps_submenu=item->GetSubMenu();
	BASSERT(keymaps_submenu);

	//take them out
	wxMenuItemList &menu_items=keymaps_submenu->GetMenuItems();
	if(menu_items.GetCount()>0) {
		//i'm a bit concerned about the deletion changing menu_items...
		std::vector<wxMenuItem *> items;
		for(wxMenuItemList::Node *node=menu_items.GetFirst();node;node=node->GetNext()) {
			items.push_back(node->GetData());
		}
		for(unsigned j=0;j<items.size();++j) {
			keymaps_submenu->Delete(items[j]);
		}
	}

	//put them in
	int j=id_first_keymap;
	mbKeymap::C::const_iterator km;
	for(km=cfg_.keyboard.keymaps.begin();km!=cfg_.keyboard.keymaps.end();++km) {
		if(j>=id_last_keymap) {
			//You don't get any more than that!!
			break;
		}
		keymaps_submenu->AppendCheckItem(j,km->name);
		keymaps_submenu->Check(j,km->name==cfg_.keyboard.selected_keymap);
		++j;
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::SetMenuItemEnabled(int menu_item_id,bool enabled) {
	wxMenuItem *menu_item;
	
	menu_item=menu_bar_->FindItem(menu_item_id);
	if(menu_item) {
		menu_item->Enable(enabled);
	}

	menu_item=popup_menu_.FindItem(menu_item_id);
	if(menu_item) {
		menu_item->Enable(enabled);
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::SetMenuItemCheck(int menu_item_id,bool checked) {
	wxMenuItem *menu_item;
	
	menu_item=menu_bar_->FindItem(menu_item_id);
	if(menu_item) {
		menu_item->Check(checked);
	}

	menu_item=popup_menu_.FindItem(menu_item_id);
	if(menu_item) {
		menu_item->Check(checked);
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::SetMenuItemText(int menu_item_id,const wxString &text) {
	wxMenuItem *menu_item;
	
	menu_item=menu_bar_->FindItem(menu_item_id);
	if(menu_item) {
		menu_item->SetText(text);
	}

	menu_item=popup_menu_.FindItem(menu_item_id);
	if(menu_item) {
		menu_item->SetText(text);
	}
}

//////////////////////////////////////////////////////////////////////////
static wxString WildcardsFromDiscFormat(const mbDiscFormat &fmt) {
	wxString wildcards;

	for(unsigned i=0;i<fmt.extensions.size();++i) {
		if(!wildcards.empty()) {
			wildcards+=";";
		}
		wildcards+="*."+fmt.extensions[i];
	}

	return wildcards;
}

void mbMainFrame::OnDriveLoadInternal(wxCommandEvent &event,int drive) {
	wxString dialog_wildcards;
	wxString all_wildcards;
	std::vector<mbDiscFormat> fmts;

	//
	mbDiscFormatGetAll(&fmts);
	for(unsigned i=0;i<fmts.size();++i) {
		wxString wildcards=WildcardsFromDiscFormat(fmts[i]);
		if(!all_wildcards.empty()) {
			all_wildcards+=";";
		}
		all_wildcards+=wildcards;
		dialog_wildcards+=fmts[i].description+" ("+wildcards+")|"+wildcards+"|";
//		wxString wildcards;
//		for(unsigned j=0;j<fmts[i].extensions.size();++j) {
//			if(!wildcards.empty()) {
//				wildcards+=";";
//			}
//			wildcards+="*."+fmts[i].extensions[j];
//		}
//		if(!all_wildcards.empty()) {
//			all_wildcards+=";";
//		}
//		all_wildcards+=wildcards;
//		dialog_wildcards+=fmts[i].description+" ("+wildcards+")|"+wildcards+"|";
	}
	dialog_wildcards="Autodetect ("+all_wildcards+")|"+all_wildcards+"|"+dialog_wildcards;

	//
	DriveUi *ui=&drive_uis_[drive];
	wxFileDialog fd(this,"Select disc",ui->last_path,"",dialog_wildcards,
		wxOPEN|wxHIDE_READONLY);//the read only flag doesn't do anything :(
	fd.SetFilterIndex(ui->last_filter_index);
	int r=fd.ShowModal();
	if(r==wxID_CANCEL) {
		return;
	}

	//
	ui->last_filter_index=fd.GetFilterIndex();
	ui->last_path=fd.GetPath();
	this->UiLoadDisc(drive,fd.GetPath());
}

//////////////////////////////////////////////////////////////////////////
// UiUnloadDrive -- does user interface for unloading a drive, if that's
// necessary. Does not handle side 2 drives...
bool mbMainFrame::UiUnloadDrive(int drive) {
	int specified_drive=drive;
	BASSERT(drive>=0);

	//Early out if drive empty.
	bbcFdd::DriveStatusType stdrive=bbcFdd::DriveStatus(drive);
	if(stdrive==bbcFdd::DRIVE_EMPTY) {
		return true;
	}

	//'drive' should be the top side.
	BASSERT(stdrive!=bbcFdd::DRIVE_SIDE2);

	int other=bbcFdd::other_side_of[drive];
	bbcFdd::DriveStatusType stother=bbcFdd::DriveStatus(other);
	bool is_ds=stother==bbcFdd::DRIVE_SIDE2;

	if(stdrive==bbcFdd::DRIVE_CHANGED) {
		wxString msg;
		
		msg+=wxString::Format("Drive %d contains \"%s\".\n",
			drive,drive_uis_[drive].disc_image.GetFullPath().c_str());
		if(is_ds) {
			msg+=wxString::Format("(Unloading drive %d will also unload drive %d.)\n\n",
				drive,other);
		}
		msg+="This disc image has changed.\n";
		msg+="If you continue, these changes will be lost.\n\n";
		msg+="Do you want to continue?";
		int result=wxMessageBox(msg,wxString::Format("Unload drive %d",drive),
			wxYES_NO,this);
		return result==wxYES;
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////
bool mbMainFrame::UiLoadDisc(int drive,wxFileName file_name) {
	wxString name=file_name.GetFullPath();

	//Load disc image
	std::vector<t65::byte> contents;
	OutputDebugStringA(name.c_str());
	OutputDebugStringA("\n");
	if(!file_name.IsOk()) {
		wxLogError("Invalid filename \"%s\"\n",name.c_str());
		return false;
	}
	bool loaded_ok=mbLoadFile(name,contents);
	if(!loaded_ok) {
		wxLogError("Failed to load \"%s\"\n",name.c_str());
		return false;
	}
	file_name.Normalize();

	//Determine the geometry of this disc image.
	mbDiscFormat fmt;
	if(!mbDiscFormatGetFromFile(file_name,contents.size(),&fmt)) {
		wxLogError("Cannot determine disc format of file \"%s\"\n",name.c_str());
		return false;
	}

	bool ok=false;
	switch(fmt.sides) {
	case 1:
		{
			if(!this->UiUnloadDrive(drive)) {
				return false;
			}
			bbcFdd::UnloadDiscImage(drive);
			ok=bbcFdd::LoadSsDiscImage(drive,fmt.sectors,fmt.density,contents);
			wxLogDebug("mbMainFrame::UiLoadDisc: load drive %d from \"%s\"",drive,name.c_str());
			if(!ok) {
				wxLogError("Failed to load drive %d from \"%s\"\n",drive,name.c_str());
			}
		}
		break;
	case 2:
		{
			int other=bbcFdd::other_side_of[drive];
			bool is_ds=bbcFdd::DriveStatus(other)==bbcFdd::DRIVE_SIDE2;
			if(!this->UiUnloadDrive(drive)||(!is_ds&&!this->UiUnloadDrive(other))) {
				return false;
			}
			//'drive' is top
			if(other<drive) {
				std::swap(other,drive);
			}
			bbcFdd::UnloadDiscImage(drive);
			bbcFdd::UnloadDiscImage(other);
			wxLogDebug("mbMainFrame::UiLoadDisc: load drive %d from \"%s\"",drive,name.c_str());
			ok=bbcFdd::LoadDsDiscImage(drive,fmt.sectors,fmt.density,contents);
			if(!ok) {
				wxLogError("Failed to load drives %d+%d from \"%s\"\n",
					drive,other,name.c_str());
			}
		}
		break;
	}
	if(ok) {
		//Assign to that drive
		drive_uis_[drive].disc_image=file_name;
		drive_uis_[drive].no_save=false;
	}
	this->RefreshWindow();
	return ok;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnDriveLoad(wxCommandEvent &event,int drive) {
	this->ModalBackground();
//	wxASSERT(bbcFdd::DriveStatus(drive)==bbcFdd::DRIVE_EMPTY);
	this->OnDriveLoadInternal(event,drive);
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnDriveSave(wxCommandEvent &event,int drive) {
	this->ModalBackground();
	bbcFdd::DriveStatusType status=bbcFdd::DriveStatus(drive);
	wxASSERT(status!=bbcFdd::DRIVE_EMPTY);
	wxASSERT(status!=bbcFdd::DRIVE_SIDE2);
	bool ok;
	std::vector<t65::byte> contents;
	if(bbcFdd::IsDs(drive)) {
		int other=bbcFdd::other_side_of[drive];
		ok=bbcFdd::GetDsDiscImage(drive,other,&contents);
	} else {
		ok=bbcFdd::GetSsDiscImage(drive,&contents);
	}
	//bbcFdd::GetDiscImage(drive,contents);
	wxString filename=drive_uis_[drive].disc_image.GetFullPath();
	if(!ok||!mbSaveFile(filename,contents)) {
		wxLogWarning("Failed to save file\n%s",filename.c_str());
	} else {
		bbcFdd::SetDriveChangedStatus(drive,false);
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnDriveSaveAs(wxCommandEvent &event,int drive) {
	this->ModalBackground();
	
	bbcFdd::DriveStatusType status=bbcFdd::DriveStatus(drive);
	wxASSERT(status!=bbcFdd::DRIVE_EMPTY);//neither of these should be...
	wxASSERT(status!=bbcFdd::DRIVE_SIDE2);//...possible to do from the ui

	wxString cur_ext=drive_uis_[drive].disc_image.GetExt().Lower();

	//disc must be saved in the same format. determine disc format
	//for the image in this drive.
	std::vector<mbDiscFormat> fmts;
	mbDiscFormatGetAll(&fmts);
	unsigned fmtidx;
	for(fmtidx=0;fmtidx<fmts.size();++fmtidx) {
		unsigned j;
		const mbDiscFormat *fmt=&fmts[fmtidx];
		for(j=0;j<fmt->extensions.size();++j) {
			if(cur_ext==fmt->extensions[j].Lower()) {
				break;
			}
		}
		if(j<fmt->extensions.size()) {
			break;
		}
	}
	
	if(fmtidx>=fmts.size()) {
		wxLogError("Drive %d somehow contains an unknown type of disc image.",drive);
	} else {
		const mbDiscFormat *fmt=&fmts[fmtidx];
		wxString wildcards=WildcardsFromDiscFormat(*fmt);
		wildcards=fmt->description+" ("+wildcards+")|"+wildcards;
		DriveUi *ui=&drive_uis_[drive];

		wxFileDialog fd(this,"Save disc",ui->last_path,"",wildcards,wxSAVE|wxOVERWRITE_PROMPT);
		int r=fd.ShowModal();
		if(r==wxID_OK) {
			bool ok;
			std::vector<t65::byte> contents;
			if(bbcFdd::IsDs(drive)) {
				int other=bbcFdd::other_side_of[drive];
				ok=bbcFdd::GetDsDiscImage(drive,other,&contents);
			} else {
				ok=bbcFdd::GetSsDiscImage(drive,&contents);
			}
			wxASSERT(ok);

			if(!mbSaveFile(fd.GetPath(),contents)) {
				wxLogError("Failed to save \"%s\"",fd.GetPath().c_str());
			} else {
				ui->disc_image=wxFileName(fd.GetPath());
				bbcFdd::SetDriveChangedStatus(drive,false);
				this->RefreshWindow();
			}
		}
	}

	//

}


//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnDriveXCommand(wxCommandEvent &event) {
	wxASSERT(event.GetId()>=id_drives_begin&&event.GetId()<id_drives_end);
	int id=event.GetId();
	id-=id_drives_begin;
	int subaction=id%subid_drive_num;
	int drive=id/subid_drive_num;
	if(drive_subactions[subaction]) {
		(this->*drive_subactions[subaction])(event,drive);
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnDriveUnload(wxCommandEvent &event,int drive) {
	bbcFdd::DriveStatusType status=bbcFdd::DriveStatus(drive);
	wxASSERT(status!=bbcFdd::DRIVE_EMPTY);
	if(status==bbcFdd::DRIVE_CHANGED) {
		static const wxString msg=
			"If this drive is unloaded, changes will be lost.\n"
			"\n"
			"Are you sure?";
		int r=wxMessageBox(msg,wxString::Format("Unload drive %d",drive),wxYES_NO);
		if(r!=wxYES) {
			return;
		}
	}
	bbcFdd::UnloadDiscImage(drive);
	//drive_uis_[drive].loaded=false;
	this->RefreshWindow();
	return;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnQuickstart(wxCommandEvent &event) {
	beeb_paused_=true;
	this->ModalBackground();
	mbQuickstartResult r;
	if(mbQuickstart(this,cfg_.quickstart,&r)) {
		if(this->UiLoadDisc(0,r.drive0)&&
			(!r.drive2.IsOk()||this->UiLoadDisc(2,r.drive2)))
		{
			bbcKeyboardMatrix::SetForceShiftPressed(true);
			this->PowerOnBeeb();// 7/2/04 fix for emulator losing input after quickstart (not resetting next_sys_update point)
		}
	}
	beeb_paused_=false;
}

//////////////////////////////////////////////////////////////////////////
#ifdef ENABLE_TEST
void mbMainFrame::OnTest(wxCommandEvent &event) {
	static const char *name="Z:\\beeb\\beebgames\\tapes\\Frak_B\\Frak_B.uef";
	std::vector<t65::byte> contents;
	if(mbLoadFile(name,contents)) {
		bbcUef test(contents);
	}
}
#endif
	
//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnKeymapDialog(wxCommandEvent &event) {
	beeb_paused_=true;
	this->ModalBackground();
	wxString old_selected_keymap=cfg_.keyboard.selected_keymap;
	mbKeyboardConfigDialog dlg(this,cfg_.keyboard);
	dlg.ShowModal();
	dlg.GetResult(&cfg_.keyboard);
	this->SetKeymap(old_selected_keymap);
	this->RefreshWindow();
	beeb_paused_=false;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnSoundDialog(wxCommandEvent &event) {
	beeb_paused_=true;
	this->ModalBackground();
	mbSoundConfigDialog snd_dlg(this,cfg_.sound);
	if(snd_dlg.ShowModal()==wxID_OK) {
		snd_dlg.GetResult(&cfg_.sound);
		bool old_enabled=cfg_.sound.enabled;
		this->SetSoundEnabled(false);
		this->SetSoundEnabled(old_enabled);
		this->RefreshWindow();
	}
	beeb_paused_=false;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnRomsDialog(wxCommandEvent &event) {
	BASSERT(model_);
	beeb_paused_=true;
	this->ModalBackground();

	mbRomsConfigDialog roms_dlg(this,-1,cfg_.roms,model_);
	
	int result=roms_dlg.ShowModal();
	if(result==wxID_OK||result==wxID_APPLY) {
		roms_dlg.GetResult(&cfg_.roms);
	}

	if(result==wxID_APPLY) {
		this->ApplyConfig();
	}

	beeb_paused_=false;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnKeymap(wxCommandEvent &event) {
	int id=event.GetId();
	wxMenuItem *keymap=menu_bar_->FindItem(id);
	if(keymap) {
		this->SetKeymap(keymap->GetText());
		this->RefreshWindow();
	}
}

//////////////////////////////////////////////////////////////////////////
#if (defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS)
void mbMainFrame::ResetDisassembly() {
#ifdef bbcDEBUG_TRACE
	delete bbcComputer::trace;
	bbcComputer::trace=0;
#elif (defined bbcDEBUG_TRACE_EVENTS)
	bbcComputer::event_record.Clear();
	bbcComputer::trace_cpu_events=false;
#endif
}

void mbMainFrame::OnStartDisassembly(wxCommandEvent &event) {
	this->ResetDisassembly();
#ifdef bbcDEBUG_TRACE
	bbcComputer::trace=new bbcDebugTrace;
#elif (defined bbcDEBUG_TRACE_EVENTS)
	bbcComputer::trace_cpu_events=true;
#endif
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnRebootWithDisassembly(wxCommandEvent &event) {
	this->ResetDisassembly();
	this->OnStartDisassembly(event);
	this->PowerOnBeeb();
}

//////////////////////////////////////////////////////////////////////////
static bool ProgressIncProgressDialog(int percent_done,unsigned amount_written,void *context) {
	wxProgressDialog *dlg=static_cast<wxProgressDialog *>(context);

	int mb=int(100.f/percent_done*amount_written)/1048576;
	wxString caption=wxString::Format("Estimated output size: %dMB",mb);
	dlg->Update(percent_done,caption);
	return true;
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnStopDisassembly(wxCommandEvent &event) {
	//anything to save?
#ifdef bbcDEBUG_TRACE
	if(!bbcComputer::trace)
#elif (defined bbcDEBUG_TRACE_EVENTS)
	if(!bbcComputer::trace_cpu_events)
#endif
	{
		return;
	}

	//
	FILE *h=0;
	for(;;) {
		wxString name=wxFileSelector("Save disassembly to file",0,0,0,
			wxFileSelectorDefaultWildcardStr,wxSAVE|wxOVERWRITE_PROMPT,this);
		if(name.empty()) {
			break;
		}
		h=fopen(name.c_str(),"wt");
		if(true) {//h) {
			//should have a 'try again' box here in case NAV goes mental as
			//usual.
			break;
		}
	}
	if(h) {
		setvbuf(h,0,32767,_IOFBF);//the default buffer is a ridiculous 512 bytes.
		wxProgressDialog progress("Please wait...","Saving disassembly.",100,this,
			wxPD_AUTO_HIDE|wxPD_APP_MODAL|wxPD_ESTIMATED_TIME|wxPD_ELAPSED_TIME);
#ifdef bbcDEBUG_TRACE
		bool b=bbcComputer::trace->WriteTrace(h,&ProgressIncProgressDialog,&progress);
#elif (defined bbcDEBUG_TRACE_EVENTS)
		bool b=bbcComputer::event_record.SaveEventsText(h,&ProgressIncProgressDialog,&progress);
#endif
		if(!b) {
			wxLogMessage("Save was cancelled.");
		}
		fclose(h);
	}

	wxProgressDialog progress("Please wait...","Freeing memory...",-1,this);
	this->ResetDisassembly();
}
#endif//BEEB_DEBUG_DISASSEMBLY

////////////////////////////////////////////////////////////////////////
//
#ifdef BEEB_DEBUG_SAVE_CHALLENGER_RAM
void mbMainFrame::OnSaveChallengerRam(wxCommandEvent &event) {
	wxString name=wxFileSelector("Save Challenger RAM to file",0,0,0,
		wxFileSelectorDefaultWildcardStr,wxSAVE|wxOVERWRITE_PROMPT,this);
	if(!name.empty()) {
		if(!mbSaveFile(name,bbc1770OpusChallenger::ram_)) {
			wxLogError("Failed to save \"%s\"",name.c_str());
		}
	}
}
#endif

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnResolution(wxCommandEvent &event) {
	this->SetBeebScreenSize(this->ResolutionFromId(event.GetId()));
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::SetBeebScreenSize(const wxSize &screen_size) {
	if(hostgfx_&&HostGfx_IsFullScreen(hostgfx_)) {
		return;
	}
	wxSize deltas=this->GetSize()-beeb_screen_->GetSize();
	wxASSERT(deltas.GetWidth()>=0&&deltas.GetHeight()>=0);
	wxSize size=deltas+screen_size;
	wxLogDebug("mbMainFrame::OnResolution: beeb screen=%dx%d",
		screen_size.GetWidth(),screen_size.GetHeight());
	wxLogDebug("mbMainFrame::OnResolution: new window=%dx%d",
		size.GetWidth(),size.GetHeight());
	this->SetSize(size);
}

//////////////////////////////////////////////////////////////////////////
wxSize mbMainFrame::ResolutionFromId(int id) {
	wxASSERT(id>=id_resolution_0&&id<=id_resolution_3);
	id-=id_resolution_0;
	wxSize s(cfg_.video.width,cfg_.video.height);
	if(!(id&2)) {
		s.x/=2;
	}
	if(id&1) {
		s.y*=2;
	}
	return s;
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnScreenshotSaveAs(wxCommandEvent &event) {
	screenshot_countdown_=0;
	wxString wildcards;
	wildcards+="Auto (*.*)|*.*";
	wxFileDialog fd(this,"Select file","","",wildcards,wxSAVE);
	int r=fd.ShowModal();
	if(r==wxID_CANCEL) {
		return;
	}
	screenshot_countdown_=2;
	screenshot_name_=fd.GetPath();
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::DoScreenshot() {
	bool error=true;

	if(last_video_buffer_) {
		wxImage *image=new wxImage(last_video_buffer_->width,last_video_buffer_->height);
		const t65::byte *src=last_video_buffer_->data;
		for(int y=0;y<last_video_buffer_->height;++y) {
			for(int x=0;x<last_video_buffer_->width;++x) {
				unsigned char r=*src&1?255:0;
				unsigned char g=*src&2?255:0;
				unsigned char b=*src&4?255:0;
				++src;
				image->SetRGB(x,y,r,g,b);
			}
		}
		error=!image->SaveFile(screenshot_name_);
		delete image;
	}

	if(error) {
		wxLogError("Failed to save image\n\"%s\"\n",screenshot_name_.c_str());
	}
}
//	wxImage *image=new wxImage(bbcVideo::buffer_width,bbcVideo::buffer_height);
//	const t65::byte *src=bbcVideo::buffer;
//	for(int y=0;y<bbcVideo::buffer_height;++y) {
//		for(int x=0;x<bbcVideo::buffer_width;++x) {
//			unsigned char r=*src&1?255:0;
//			unsigned char g=*src&2?255:0;
//			unsigned char b=*src&4?255:0;
//			++src;
//			image->SetRGB(x,y,r,g,b);
//		}
//	}
//	if(!image->SaveFile(screenshot_name_)) {
//		wxLogError("Failed to save image\n\"%s\"\n",screenshot_name_.c_str());
//	}
//	delete image;
//}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnToggleStatusBar(wxCommandEvent &event) {
	cfg_.misc.show_status_bar=!cfg_.misc.show_status_bar;
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnToggleFastForwardDiscAccess(wxCommandEvent &event) {
	cfg_.misc.fast_forward_disc_access=!cfg_.misc.fast_forward_disc_access;
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
#ifdef bbcDEBUG_PANELS
static mbDebugPanel *ViaPanelInit(wxFrame *frame,long id,const char *t,bbcVIA *via) {
	mbDebugPanel *panel=static_cast<mbDebugPanel *>(frame->FindWindow(id));
	if(panel) {
		panel->Destroy();
		panel=0;
	} else {
		panel=new mbDebugPanel(frame,id,t);
		via->InitDebugPanel(panel->Panel());
	}
	return panel;
}

void mbMainFrame::OnToggleDbgpanelSysvia(wxCommandEvent &event) {
	ViaPanelInit(this,id_dbgpanel_sysvia,"System VIA",&bbc_system_via);
}

void mbMainFrame::OnToggleDbgpanelUsrvia(wxCommandEvent &event) {
	ViaPanelInit(this,id_dbgpanel_usrvia,"User VIA",&bbc_user_via);
}

void mbMainFrame::OnToggleDbgpanelVideo(wxCommandEvent &event) {
	mbDebugPanel *panel=
		static_cast<mbDebugPanel *>(this->FindWindow(id_dbgpanel_video));

	if(panel) {
		panel->Destroy();
		panel=0;
	} else {
		panel=new mbDebugPanel(this,id_dbgpanel_video,"Video System");
		panel->Panel()->SetSize(52,16);
	}
}
#endif

#ifdef bbcDEBUG_VIDEO
void mbMainFrame::OnToggleShowAll(wxCommandEvent &event) {
	bbcVideo::SetShowAll(!bbcVideo::show_all_);
	this->RefreshWindow();
}

void mbMainFrame::OnLogVideoFrames(wxCommandEvent &event) {
#ifdef bbcDEBUG_TRACE
	bool with_disassembly=event.GetId()==id_log_video_frames_disasm;
#else
	bool with_disassembly=false;
#endif
	wxString caption="Log frames";
	if(with_disassembly) {
		caption+=" w/ disasm";
	}
	wxString name=wxFileSelector(caption,0,0,0,
		"Text file (*.txt)|*.txt|",wxSAVE|wxOVERWRITE_PROMPT,this);
	if(!name.empty()) {
		bbcVideo::LogFrames(name.c_str(),5,with_disassembly);
	}
}

void mbMainFrame::OnToggleShowT1Timeouts(wxCommandEvent &event) {
	bbcVideo::show_t1_timeouts_=!bbcVideo::show_t1_timeouts_;
}
#endif

void mbMainFrame::OnSoundRecordStart(wxCommandEvent &event) {
	bbcSound::StartRecording();
	this->RefreshWindow();
}

void mbMainFrame::OnSoundRecordStop(wxCommandEvent &event) {
	if(!bbcSound::IsRecording()) {
		return;
	}
	bbcSound::StopRecording();
	wxString name=wxFileSelector("Save VGM file",0,0,0,
		"Video Game Music file (*.vgm)|*.vgm|",wxSAVE|wxOVERWRITE_PROMPT,this);
	if(!name.empty()) {
		std::vector<t65::byte> binary;
		bbcSound::GetRecordedVgmData(&binary);
		if(!mbSaveFile(name,binary)) {
			wxLogError("Failed to save sound file \"%s\".\n",name.c_str());
		}
	}
	bbcSound::ClearRecording();
}

//////////////////////////////////////////////////////////////////////////
// Hard reset -- power on type thing.
void mbMainFrame::OnHardReset(wxCommandEvent &event) {
	int result=wxMessageBox("This will totally reset the computer\n\nAre you sure?",
		"Hard reset",wxYES_NO|wxICON_QUESTION,this);
	if(result==wxYES) {
		this->PowerOnBeeb();
#ifdef bbcHACKING_URIDIUM
		if(cl_autoboot_) {
			bbcKeyboardMatrix::SetForceShiftPressed(true);
		}
#endif
	}
}

//////////////////////////////////////////////////////////////////////////
void mbMainFrame::OnSetScreenType(wxCommandEvent &event) {
	switch(event.GetId()) {
	default:
	case id_screen_type_colour:
		cfg_.video.monitor_type=mbMONITOR_COLOUR;
		break;
	case id_screen_type_bnw:
		cfg_.video.monitor_type=mbMONITOR_BNW;
		break;
	case id_screen_type_amber:
		cfg_.video.monitor_type=mbMONITOR_AMBER;
		break;
	case id_screen_type_green:
		cfg_.video.monitor_type=mbMONITOR_GREEN;
		break;
	}

	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnVolume(wxCommandEvent &event) {
	//printf("mbMainFrame::OnVolume: volume=%ld\n",event.m_extraLong);
	wxASSERT(event.m_extraLong>=0&&event.m_extraLong<=100);
	cfg_.sound.volume=event.m_extraLong;
	HostSnd_SetVolume(hostsnd_,cfg_.sound.volume);
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnDropFile(wxCommandEvent &event) {
	this->UiLoadDisc(event.m_commandInt,event.m_commandString);
}

//////////////////////////////////////////////////////////////////////////
//
/*
void mbMainFrame::OnToggleVertical2x(wxCommandEvent &event) {
	cfg_.video.vertical_2x=!cfg_.video.vertical_2x;
	this->RefreshWindow();
}
*/
//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnActivate(wxActivateEvent &event) {
#ifdef __WXMSW__
	if(!event.GetActive()) {
		HWND cur=GetForegroundWindow();
		HWND window=0;

		while(cur) {
//			char text[2000];
//
//			GetWindowText(cur,text,sizeof text);
//			wxLogDebug("cur=%08X text=\"%s\"\n",cur,text);
			window=cur;
			cur=::GetParent(cur);
		}

		if(window!=HWND(this->GetHandle())) {
			cfg_.video.full_screen=false;
		}
//		wxWindow *cur=wxGetActiveWindow();
//		wxWindow *window=0;
//
//		while(cur) {
//			wxLogDebug(__FUNCTION__ ": cur title=%s\n",cur->GetT)
//			window=cur;
//			cur=cur->GetParent();
//		}
//
//		if(window!=this) {
//			cfg_.video.full_screen=false;
//		}
	}

	wxLogDebug(__FUNCTION__ ": event.GetActive()=%d; cfg_.video.full_screen=%d\n",event.GetActive(),cfg_.video.full_screen);

#else
	//TODO for now this bit of code is a bit dumb.
	cfg_.video.full_screen=event.GetActive()&&cfg_.video.full_screen;
#endif
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnToggleFullScreen(wxCommandEvent &event) {
	cfg_.video.full_screen=!cfg_.video.full_screen;
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnScanlines(wxCommandEvent &event) {
	switch(event.GetId()) {
	case id_windowed_scanlines_single:
		cfg_.video.windowed_scanlines=mbSCANLINES_SINGLE;
		break;

	case id_windowed_scanlines_interlace:
		cfg_.video.windowed_scanlines=mbSCANLINES_INTERLACE;
		break;

	case id_windowed_scanlines_double:
		cfg_.video.windowed_scanlines=mbSCANLINES_DOUBLE;
		break;

	case id_full_screen_scanlines_single:
		cfg_.video.full_screen_scanlines=mbSCANLINES_SINGLE;
		break;

	case id_full_screen_scanlines_interlace:
		cfg_.video.full_screen_scanlines=mbSCANLINES_INTERLACE;
		break;

	case id_full_screen_scanlines_double:
		cfg_.video.full_screen_scanlines=mbSCANLINES_DOUBLE;
		break;
	}
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnJoysticksDialog(wxCommandEvent &event) {
	beeb_paused_=true;
	this->ModalBackground();
//	wxString old_selected_keymap=cfg_.keyboard.selected_keymap;
//	mbKeyboardConfigDialog dlg(this,cfg_.keyboard);
//	dlg.ShowModal();
//	dlg.GetResult(&cfg_.keyboard);
//	this->SetKeymap(old_selected_keymap);

	mbJoysticksConfigDialog dlg(this,cfg_.joysticks);
	if(!dlg.IsOk()) {
		wxLogError("Failed to initialise joystick configuration dialog");
	} else {
		if(dlg.ShowModal()==wxID_OK) {
			dlg.GetResult(&cfg_.joysticks);
		}
	}

	this->RefreshWindow();
	beeb_paused_=false;
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnRightUp(wxMouseEvent &event) {
//	if(hostgfx_&&HostGfx_IsFullScreen(hostgfx_)) {
	this->PopupMenu(&popup_menu_,event.GetPosition());
//	}
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnHardwareDialog(wxCommandEvent &event) {
	beeb_paused_=true;
	this->ModalBackground();
	
	mbHardwareDialog dlg(this,cfg_.misc,cfg_.roms);
	if(dlg.ShowModal()==wxID_OK) {
		dlg.GetResult(&cfg_.misc,&cfg_.roms);
		this->ApplyConfig();
	}
	
	this->RefreshWindow();
	beeb_paused_=false;
}

#ifdef bbcDEBUG_PANELS
//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::OnToggleDbgpanelAdc(wxCommandEvent &event) {
	mbDebugPanel *panel=static_cast<mbDebugPanel *>(this->FindWindow(id_dbgpanel_adc));
	if(panel) {
		panel->Destroy();
		panel=0;
	} else {
		panel=new mbDebugPanel(this,id_dbgpanel_adc,"ADC");
		bbcAdc::InitDebugPanel(panel->Panel());
	}
}
#endif

//////////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::PresentLastVideoBuffer(mbScanlinesMode scanlines_mode,int present_hints,int frame_number) {
	int line_height=1;
	int gap_height=0;
	int y_first=0;

	if(last_video_buffer_) {
		switch(scanlines_mode) {
		case mbSCANLINES_SINGLE:
			gap_height=HostGfx_IsFullScreen(hostgfx_)?1:0;
			break;

		case mbSCANLINES_INTERLACE:
			y_first=frame_number&1;
			gap_height=1;
			break;

		case mbSCANLINES_DOUBLE:
			line_height=2;
			break;
		}

		HostGfx_Present(hostgfx_,cfg_.video.video_effect,last_video_buffer_->data,last_video_buffer_->width,last_video_buffer_->height,
			last_video_buffer_->pitch,y_first,line_height,gap_height,present_hints);
	}
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::DoBeebTick() {
	this->SetAutoPaint(false);
	for(;;) {
		bbcComputer::Run6502();
		
		//This is to cater for overflow, which takes about 15 mins to happen at 2MHz.
		//No update will cause the cycles counter to jump by 2,000,000.
		bbcComputer::next_stop+=2000000;//=INT_MAX;
		
		bbcComputer::Update();
		
		//Do this first, so it can change the screen buffer to indicate stuff
		if(int(bbcComputer::cpucycles-next_update_sound_)>0) {//bbcComputer::cpucycles>next_update_sound_) {
			this->DoUpdateSound();
			next_update_sound_=bbcComputer::cpucycles+update_sound_cycles;
		}

		const bbcVideoBuffer *new_video_buffer=bbcVideo::DisplayBuffer();
		if(new_video_buffer!=last_video_buffer_) {
			last_video_buffer_=new_video_buffer;

			if(screenshot_countdown_==1) {
				this->DoScreenshot();
			}
			
			int freq=beeb_fastforward_?50:cfg_.video.frame_skip;
			if(num_frames_%freq==0) {
				int present_hints=0;
				bool is_full_screen=!!HostGfx_IsFullScreen(hostgfx_);

				if(is_full_screen&&cfg_.video.full_screen_vsync) {
					present_hints|=HostGfx_PRESENT_HINT_SYNC;
				}

				mbScanlinesMode scanlines_mode=is_full_screen?cfg_.video.full_screen_scanlines:cfg_.video.windowed_scanlines;
				this->PresentLastVideoBuffer(scanlines_mode,present_hints,num_frames_/freq);

				if(is_full_screen) {
					bbcVideo::SetPixelFixedBits(HostGfx_FULL_SCREEN_PALETTE_BASE);
				} else {
					bbcVideo::SetPixelFixedBits(0);
				}

				bbcVideo::SetBufferDimensions(cfg_.video.width,cfg_.video.height);
			}
			//bbcVideo::buffer_dirty=false;

			if(screenshot_countdown_>0||num_frames_%cfg_.video.cleanup_frequency==0) {
				//memset(bbcVideo::buffer,bbcVideo::blank_pixel,bbcVideo::buffer_size);
				bbcVideo::ClearBuffer();
			}
			if(screenshot_countdown_>0) {
				--screenshot_countdown_;
			}

			++num_frames_;
		}
		if(int(bbcComputer::cpucycles-next_update_input_)>0) {//bbcComputer::cpucycles>next_update_keys_) {
			this->DoUpdateInput();
			next_update_input_=bbcComputer::cpucycles+update_input_cycles;
			//			if(bbcKeyboardMatrix::KeyState(mbKEY_QUICKQUIT)) {
			//				break;
			//			}
		}
		if(int(bbcComputer::cpucycles-next_update_sys_)>0) {//bbcComputer::cpucycles>next_update_sys_) {
			break;
		}
#ifdef bbcHACKING_URIDIUM
		if(bbcComputer::trace&&bbcComputer::trace->size()>=5000000) {
			this->OnStopDisassembly(wxCommandEvent());
		}
#endif
	}
	this->SetAutoPaint(true);
}

//////////////////////////////////////////////////////////////////////////
//
bool mbMainFrame::ApplyConfig() {
	std::vector<wxByte> rom;
	
	wxLogDebug("ApplyConfig...");

	this->SetSoundEnabled(cfg_.sound.enabled);

	if(hostsnd_) {
		HostSnd_SetVolume(hostsnd_,cfg_.sound.volume);
	}
	
	wxLogDebug("ApplyConfig: Keymap: keymap \"%s\"",cfg_.keyboard.selected_keymap.c_str());
	this->SetKeymap(cfg_.keyboard.selected_keymap);
	
	//If there is no window size stored in the INI file, resize the BBC screen
	//to the size specified in the INI file.
	this->Show();
	if(cfg_.misc.window_rect.GetWidth()<0) {
		wxLogDebug("ApplyConfig: no saved window rect: BBC screen is %dx%d",
			cfg_.video.width,cfg_.video.height);
		beeb_screen_->SetSize(cfg_.video.width,cfg_.video.height);
		this->GetSizer()->Layout();
	} else {
		wxLogDebug("ApplyConfig: saved window rect is @(%d,%d) +(%d,%d)",
			cfg_.misc.window_rect.GetLeft(),cfg_.misc.window_rect.GetTop(),
			cfg_.misc.window_rect.GetWidth(),cfg_.misc.window_rect.GetHeight());
		this->SetSize(cfg_.misc.window_rect);
	}
	
	if(cfg_.misc.window_is_maximized) {
		wxLogDebug("ApplyConfig: Maximizing window");
		this->Maximize();
	}
	
	this->RefreshWindow();
	
	this->PowerOnBeeb();
	
	beeb_paused_=false;
	update_window_state_=true;
	
	wxLogDebug("ApplyConfig: done.");
	
	return true;
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::PowerOnBeeb() {
	HostTmr_GetHfFrequency(&hf_freq_);
	
#if (defined bbcHACKING_URIDIUM)&&((defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS))
	this->OnStartDisassembly(wxCommandEvent());
#endif
	//Beeb setup
	{
		const bbc1770Interface *disc_interface=mbDiscInterfaceFromShortName(cfg_.misc.disc_interface)->iface;
		
		model_=mbModelFromShortName(cfg_.misc.model);
		if(!model_) {
			wxLogWarning("Unknown BBC model '%s'\nBBC model set to B",cfg_.misc.model);
			cfg_.misc.model="b";
			model_=mbModelFromShortName(cfg_.misc.model);
			wxASSERT(model_);
		}

		if(!model_->LoadRoms(cfg_.roms)) {
			wxLogWarning("Failed to load ROMs\nBBC will fail to start properly until they are configured!\n");
		}
		
		bbcComputer::SetModel(model_->model);
		bbcComputer::SetDiscInterface(disc_interface);
		bbcVideo::SetBufferDimensions(cfg_.video.width,cfg_.video.height);
	}
	
	bbcSound::SetFrequency(cfg_.sound.hz);
	
	if(!(model_->model->flags&bbcMODEL_NO_KEYBOARD_LINKS)) {
		for(unsigned i=0;i<8;++i) {
			bbcKeyboardMatrix::ForceSetKeyState(bbcKEY_KEYLINKS+i,cfg_.keyboard.keylinks[i]);
		}
	}

	bbcComputer::ResetHard();

	num_frames_=0;

	HostTmr_ReadMilliseconds(&last_update_sys_time_);
	HostTmr_ReadHf(&last_update_sound_hf_time_);
	next_update_input_=bbcComputer::cpucycles;
	next_update_sys_=bbcComputer::cpucycles;
	next_update_sound_=bbcComputer::cpucycles;
}

//////////////////////////////////////////////////////////////////////////
//
#ifdef bbcDEBUG_ENABLE
void mbMainFrame::OnDisassembleRam(wxCommandEvent &event) {
	wxString name=wxFileSelector("Save RAM disassembly",0,0,0,wxFileSelectorDefaultWildcardStr,wxSAVE|wxOVERWRITE_PROMPT,this);
	if(!name.empty()) {
		FILE *h=fopen(name.c_str(),"wt");
		if(!h) {
			wxLogError("Failed to save to \"%s\"",name.c_str());
		} else {
			bbcComputer::WriteRamDisassembly(h);
			fclose(h);
		}
	}
}
#endif//bbcDEBUG_ENABLE

void mbMainFrame::OnVideoDialog(wxCommandEvent &event) {
	beeb_paused_=true;
	this->ModalBackground();

	mbVideoConfigDialog dlg(this,cfg_.video,HostGfx_FullScreenRefreshRates(hostgfx_));
	if(dlg.ShowModal()==wxID_OK) {
		dlg.GetResult(&cfg_.video);
	}

	this->RefreshWindow();
	beeb_paused_=false;
}

//////////////////////////////////////////////////////////////////////////
//
void mbMainFrame::RefreshCfgWindowState() {
	//If running full screen, don't save window size.
	//Also, don't update it if we haven't done an ApplyConfig yet -- otherwise,
	//the window size gets stuck at the arbitrary default as a size/move msg is
	//sent on window creation.
	if(!update_window_state_) {
		return;
	}

	//must check a few different things, because full screen doesn't happen straight away.
	//*sigh* i'm not sure whether all of these are actually required...
	//
	//************************************************************
	//* Note to self: Don't EVER do things that way again. EVER. *
	//************************************************************
	//
	if(cfg_.video.full_screen||(hostgfx_&&HostGfx_IsFullScreen(hostgfx_))||this->IsFullScreen()) {
		wxLogDebug(__FUNCTION__ ": Full screen.\n");
	} else {
		if(!cfg_.video.full_screen) {
			if(this->IsIconized()) {
				//If the window is iconized, don't save its size.
				wxLogDebug(__FUNCTION__ ": Iconized.\n");
			} else if(this->IsMaximized()) {
				wxLogDebug(__FUNCTION__ ": Maximized.\n");
				//it is maximized, don't save its window rect
				cfg_.misc.window_is_maximized=true;
			} else {
				//not maximized, save window rect.
				cfg_.misc.window_is_maximized=false;
				cfg_.misc.window_rect=this->GetRect();

				wxLogDebug(__FUNCTION__ ": Window @(%d,%d) +(%d,%d).\n",
					cfg_.misc.window_rect.GetLeft(),cfg_.misc.window_rect.GetTop(),
					cfg_.misc.window_rect.GetWidth(),cfg_.misc.window_rect.GetHeight());
			}
		}
	}
}

#ifdef t65_COUNT_INSTRUCTION_FREQUENCIES
//////////////////////////////////////////////////////////////////////////
//
struct InstrFreq {
	t65::byte op;
	const char *mnem;
	const char *mode;
	unsigned count;
};

//note > -- want highest first
static bool InstrFreqLessThanByCount(const InstrFreq &a,const InstrFreq &b) {
	return a.count>b.count;
}

void mbMainFrame::OnShowInstructionFrequencies(wxCommandEvent &event) {
	InstrFreq instrs[256];

	for(unsigned i=0;i<256;++i) {
		InstrFreq *instr=&instrs[i];
		instr->op=i;
		bbcComputer::Model()->GetInstrInfo(instr->op,&instr->mnem,&instr->mode,&instr->count);
	}

	if(event.GetId()==id_show_instruction_frequencies_sorted) {
		std::stable_sort(instrs,instrs+256,&InstrFreqLessThanByCount);
	}

	printf("Instruction frequencies:\n");

	//4 cols, 64 rows
	for(unsigned i=0;i<64;++i) {
		for(unsigned j=0;j<4;++j) {
			const InstrFreq *instr=&instrs[i+j*64];

			//0123456789012345678901234
			//XX: YYY? ZZZ: 2147483648
			printf("%02X: %-4s %s: %-10u ",instr->op,instr->mnem,instr->mode,instr->count);
		}
		printf("\n");
	}
}
#endif

#ifdef bbcDEBUG_COUNT_MMIO_ACCESSES
//////////////////////////////////////////////////////////////////////////
//
struct MmioAccessCount {
	t65::byte offset;
	unsigned count;
};

//note > -- want highest first
static bool MmioAccessCountLessThanByCount(const MmioAccessCount &a,const MmioAccessCount &b) {
	return a.count>b.count;
}

static void ShowMmioPageCounts(const unsigned *counts_in,bool sorted,int num_columns) {
	MmioAccessCount counts[256];

	for(unsigned i=0;i<256;++i) {
		counts[i].offset=i;
		counts[i].count=counts_in[i];
	}

	if(sorted) {
		std::stable_sort(counts,counts+256,&MmioAccessCountLessThanByCount);
	}

	int num_rows=256/num_columns;
	for(int r=0;r<num_rows;++r) {
		printf(" ");
		for(int c=0;c<num_columns;++c) {
			const MmioAccessCount *count=&counts[r+c*num_rows];

			printf("%02X: %-10u ",count->offset,count->count);
		}
		printf("\n");
	}
}

void mbMainFrame::OnShowSheilaAccessCounts(wxCommandEvent &event) {
	printf("Reads:\n");
	ShowMmioPageCounts(&bbcComputer::mmio_reads_count_[0x200],event.GetId()==id_show_sheila_access_counts_by_count,8);
	printf("Writes:\n");
	ShowMmioPageCounts(&bbcComputer::mmio_writes_count_[0x200],event.GetId()==id_show_sheila_access_counts_by_count,8);
}
#endif

#ifdef bbcENABLE_SAVE_STATE
void mbMainFrame::OnLoadState(wxCommandEvent &event) {
	wxFileDialog fd(this,"Load state","","","State files (*.UEF)|*.UEF",
		wxOPEN);
	int r=fd.ShowModal();
	if(r==wxID_CANCEL) {
		return;
	}
}

void mbMainFrame::OnSaveState(wxCommandEvent &event) {
	wxFileDialog fd(this,"Save state","","","State files (*.UEF)|*.UEF",
		wxSAVE|wxOVERWRITE_PROMPT);
	int r=fd.ShowModal();
	if(r==wxID_CANCEL) {
		return;
	}
}
#endif

void mbMainFrame::OnSpecialEffect(wxCommandEvent &event) {
	wxASSERT(event.GetId()>=id_first_special_effect&&event.GetId()<=id_last_special_effect);
	cfg_.video.video_effect=event.GetId()-id_first_special_effect;
	this->RefreshWindow();
}

//////////////////////////////////////////////////////////////////////////
//
BEGIN_EVENT_TABLE(mbMainFrame,wxFrame)
	EVT_IDLE(mbMainFrame::OnIdle)
	EVT_SIZE(mbMainFrame::OnSize)
	EVT_MOVE(mbMainFrame::OnMove)
	EVT_CLOSE(mbMainFrame::OnClose)
	EVT_ERASE_BACKGROUND(mbMainFrame::OnEraseBackground)
	EVT_PAINT(mbMainFrame::OnPaint)
	EVT_MENU_OPEN(mbMainFrame::OnMenuOpen)
	EVT_MENU_CLOSE(mbMainFrame::OnMenuClose)
	EVT_MENU(id_exit,mbMainFrame::OnExit)
	EVT_MENU(id_quickstart,mbMainFrame::OnQuickstart)
#ifdef ENABLE_TEST
	EVT_MENU(id_test,mbMainFrame::OnTest)
#endif
	EVT_MENU(id_toggle_sound,mbMainFrame::OnToggleSound)
	EVT_MENU(id_toggle_limit_speed,mbMainFrame::OnToggleLimitSpeed)
	EVT_MENU(id_toggle_status_bar,mbMainFrame::OnToggleStatusBar)
	EVT_MENU(id_toggle_fast_forward_disc_access,mbMainFrame::OnToggleFastForwardDiscAccess)
	EVT_MENU(id_show_about_box,mbMainFrame::OnShowAboutBox)
	EVT_MENU(id_roms_dialog,mbMainFrame::OnRomsDialog)
	EVT_MENU(id_sound_dialog,mbMainFrame::OnSoundDialog)
	EVT_MENU(id_keymap_dialog,mbMainFrame::OnKeymapDialog)

#if (defined bbcDEBUG_TRACE)||(defined bbcDEBUG_TRACE_EVENTS)
	EVT_MENU(id_start_disassembly,mbMainFrame::OnStartDisassembly)
	EVT_MENU(id_stop_disassembly,mbMainFrame::OnStopDisassembly)
	EVT_MENU(id_reboot_with_disassembly,mbMainFrame::OnRebootWithDisassembly)
#endif

#ifdef BEEB_DEBUG_SAVE_CHALLENGER_RAM
	EVT_MENU(id_save_challenger_ram,mbMainFrame::OnSaveChallengerRam)
#endif
#ifdef bbcDEBUG_ENABLE
	EVT_MENU(id_disassemble_ram,mbMainFrame::OnDisassembleRam)
#endif
#ifdef bbcDEBUG_PANELS
	EVT_MENU(id_toggle_dbgpanel_sysvia,mbMainFrame::OnToggleDbgpanelSysvia)
	EVT_MENU(id_toggle_dbgpanel_usrvia,mbMainFrame::OnToggleDbgpanelUsrvia)
	EVT_MENU(id_toggle_dbgpanel_video,mbMainFrame::OnToggleDbgpanelVideo)
	EVT_MENU(id_toggle_dbgpanel_adc,mbMainFrame::OnToggleDbgpanelAdc)
#endif
#ifdef bbcDEBUG_VIDEO
	EVT_MENU(id_toggle_showall,mbMainFrame::OnToggleShowAll)
	EVT_MENU(id_log_video_frames,mbMainFrame::OnLogVideoFrames)
#ifdef bbcDEBUG_TRACE
	EVT_MENU(id_log_video_frames_disasm,mbMainFrame::OnLogVideoFrames)
#endif
	EVT_MENU(id_toggle_show_t1_timeouts,mbMainFrame::OnToggleShowT1Timeouts)
#endif
	EVT_MENU(id_screenshot_save_as,mbMainFrame::OnScreenshotSaveAs)
	EVT_MENU_RANGE(id_first_keymap,id_last_keymap,mbMainFrame::OnKeymap)

	EVT_MENU_RANGE(id_resolution_0,id_resolution_3,mbMainFrame::OnResolution)

	EVT_MENU_RANGE(id_drives_begin,id_drives_end,mbMainFrame::OnDriveXCommand)

	EVT_MENU(id_sound_record_start,mbMainFrame::OnSoundRecordStart)
	EVT_MENU(id_sound_record_stop,mbMainFrame::OnSoundRecordStop)
	EVT_MENU(id_hard_reset,mbMainFrame::OnHardReset)
	EVT_MENU_RANGE(id_screen_types_begin,id_screen_types_end,mbMainFrame::OnSetScreenType)
	EVT_COMMAND(id_status_bar,mbEVT_VOLUME,mbMainFrame::OnVolume)
	EVT_COMMAND(id_status_bar,mbEVT_DROPFILE,mbMainFrame::OnDropFile)
//	EVT_MENU(id_toggle_vertical_2x,mbMainFrame::OnToggleVertical2x)
	EVT_MENU(id_toggle_full_screen,mbMainFrame::OnToggleFullScreen)
	EVT_ACTIVATE(mbMainFrame::OnActivate)
	EVT_MENU_RANGE(id_scanlines_begin,id_scanlines_end,mbMainFrame::OnScanlines)
	EVT_MENU(id_joysticks_dialog,mbMainFrame::OnJoysticksDialog)
	EVT_RIGHT_UP(mbMainFrame::OnRightUp)
	EVT_MENU(id_hardware_dialog,mbMainFrame::OnHardwareDialog)
	EVT_MENU(id_video_dialog,mbMainFrame::OnVideoDialog)
#ifdef t65_COUNT_INSTRUCTION_FREQUENCIES
	EVT_MENU(id_show_instruction_frequencies,mbMainFrame::OnShowInstructionFrequencies)
	EVT_MENU(id_show_instruction_frequencies_sorted,mbMainFrame::OnShowInstructionFrequencies)
#endif
#ifdef bbcDEBUG_COUNT_MMIO_ACCESSES
	EVT_MENU(id_show_sheila_access_counts_by_addr,mbMainFrame::OnShowSheilaAccessCounts)
	EVT_MENU(id_show_sheila_access_counts_by_count,mbMainFrame::OnShowSheilaAccessCounts)
#endif
#ifdef bbcENABLE_SAVE_STATE
	EVT_MENU(id_load_state,mbMainFrame::OnLoadState)
	EVT_MENU(id_save_state,mbMainFrame::OnSaveState)
#endif
	EVT_MENU_RANGE(id_first_special_effect,id_last_special_effect,mbMainFrame::OnSpecialEffect)
END_EVENT_TABLE()


