#include "pch.h"
#include "mbMisc.h"

//This is the largest file mbLoadFile will load.
//I put this in because the validity of a disc is determined after its
//contents have loaded. So if the user clicks on a massive file by accident,
//it will be loaded first... this should prevent anything nasty happening.
static const off_t max_load_file_size=16777216;

bool mbSaveFile(const wxString &filename,const std::vector<wxByte> &contents) {
	wxFileOutputStream out(filename);
	if(!out.Ok()) {
		return false;
	}
	out.Write(&contents[0],contents.size());
	if(!out.Ok()) {
		return false;
	}
	return true;
}

bool mbLoadFile(const wxString &filename,std::vector<wxByte> &contents) {
#ifdef _WIN32
	//For some reason, the wx system had trouble loading the " start" file from the 6502 test
	//suite. So on Win32 I'm using CreateFile and friends, which seem to do the trick.
	HANDLE h;
	bool ok=false;

	h=CreateFile(filename.c_str(),GENERIC_READ,0,0,OPEN_EXISTING,0,0);
	if(h!=INVALID_HANDLE_VALUE) {
		DWORD size;

		size=GetFileSize(h,0);
		if(size!=INVALID_FILE_SIZE) {
			DWORD num_read;

			contents.resize(size);
			if(ReadFile(h,&contents[0],size,&num_read,0)&&num_read==size) {
				ok=true;
			}
		}
		CloseHandle(h);
	}
	return ok;
#else
	contents.clear();
	wxFileSystem fs;
	wxFSFile *f=fs.OpenFile(filename);
	if(!f) {
		return false;
	}
	wxInputStream *stream=f->GetStream();
	if(!stream) {
		delete f;
		return false;
	}
	stream->SeekI(0,wxFromEnd);
	off_t length=stream->TellI();
	if(length>=max_load_file_size) {
		delete f;
		return false;
	}
	stream->SeekI(0,wxFromStart);
	contents.resize(length);
	if(!stream->Read(&contents[0],length).IsOk()) {
		delete f;
		return false;
	}
	delete f;
	return true;
#endif
}

void mbGetMatchingFileNames(const wxFileName &root,const wxString &name_spec,
	std::vector<wxFileName> *file_names)
{
	wxASSERT(file_names);
	wxFileName spec=root;
	spec.SetName(name_spec);
//	wxLogDebug("mbGetMatchingFileNames: files: \"%s\"...",spec.GetFullPath().c_str());
	wxFileSystem fs;
	wxString name=fs.FindFirst(spec.GetFullPath(),wxFILE);
	while(!name.empty()) {
		file_names->push_back(name);
		name=fs.FindNext();
	}
//	wxLogDebug("mbGetMatchingFileNames: files: %u found",file_names->size());
}

void mbGetDirectoryNames(const wxFileName &root,std::vector<wxFileName> *dir_names) {
	wxASSERT(dir_names);
	wxFileName spec=root;
	spec.SetName("*");
//	wxLogDebug("mbGetDirectoryNames: \"%s\"...",spec.GetFullPath().c_str());
	wxFileSystem fs;
	wxString name=fs.FindFirst(spec.GetFullPath(),wxDIR);
	while(!name.empty()) {
		if(name!="."&&name!="..") {
			dir_names->push_back(wxFileName::DirName(name));
		}
		name=fs.FindNext();
	}
//	wxLogDebug("mbGetDirectoryNames: %u found",dir_names->size());
}

wxString mbNormalizedFileName(const wxString &file_name) {
	wxFileName f=wxFileName::FileName(file_name);
	f.Normalize();
	return f.GetFullPath();
}

wxString mbRelativeFileName(const wxString &file_name,const wxString &base_dir) {
	wxFileName f=wxFileName::FileName(file_name);
	f.MakeRelativeTo(base_dir);
	return f.GetFullPath();
}

//A bit dumb, but there you go -- it has been tested a lot, though.
void mbTokenizeString(const wxString &source,const wxString &seperators,
	std::vector<wxString> &tokensOutput,bool handleQuotes)
{
	static const char QUOTE_CHAR='"';
	std::vector<wxString>::iterator currentToken;
	wxString::const_iterator currentChar;
	bool inQuotes=false;

	if(handleQuotes&&seperators.find_first_of(QUOTE_CHAR)!=wxString::npos) {
		handleQuotes=false;
	}
	tokensOutput.clear();
	currentToken=tokensOutput.end();
	for(currentChar=source.begin();currentChar!=source.end();currentChar++) {
		if(handleQuotes&&*currentChar==QUOTE_CHAR) {
			inQuotes=!inQuotes;
			continue;
		}
		if(!inQuotes&&seperators.find_first_of(*currentChar)!=wxString::npos) {
			// this is a seperator. 
			currentToken=tokensOutput.end();
			continue;
		}
		// this is not a seperator.
		if(currentToken==tokensOutput.end()) {
			currentToken=tokensOutput.insert(tokensOutput.end(),wxString());
		}
		currentToken->append(1,*currentChar);
	}
}

