#include "pch.h"
#include "mbModel.h"
#include "mbMisc.h"
#include "bbcComputer.h"
#include <bbcComputer.h>
#include <bbcSY6502A.h>
#include <bbcModelB.h>
#include <bbcModelBPlus.h>
#include <bbcModelMaster128.h>
#include <bbcModelBWatford.h>

//couldn't work out the syntax for arrays of pointers to members.
//(perhaps c++ doesn't support this for items in an array?)
const unsigned mb_b_roms_offsets[16]={
	offsetof(mbRomsConfig,roms[0]),
	offsetof(mbRomsConfig,roms[1]),
	offsetof(mbRomsConfig,roms[2]),
	offsetof(mbRomsConfig,roms[3]),
	offsetof(mbRomsConfig,roms[4]),
	offsetof(mbRomsConfig,roms[5]),
	offsetof(mbRomsConfig,roms[6]),
	offsetof(mbRomsConfig,roms[7]),
	offsetof(mbRomsConfig,roms[8]),
	offsetof(mbRomsConfig,roms[9]),
	offsetof(mbRomsConfig,roms[10]),
	offsetof(mbRomsConfig,roms[11]),
	offsetof(mbRomsConfig,roms[12]),
	offsetof(mbRomsConfig,roms[13]),
	offsetof(mbRomsConfig,roms[14]),
	offsetof(mbRomsConfig,roms[15]),
};

const unsigned mb_m128_roms_offsets[16]={
	offsetof(mbRomsConfig,roms[0]),
	offsetof(mbRomsConfig,roms[1]),
	offsetof(mbRomsConfig,roms[2]),
	offsetof(mbRomsConfig,roms[3]),
	offsetof(mbRomsConfig,roms[4]),
	offsetof(mbRomsConfig,roms[5]),
	offsetof(mbRomsConfig,roms[6]),
	offsetof(mbRomsConfig,roms[7]),
	offsetof(mbRomsConfig,roms[8]),
	offsetof(mbRomsConfig,master128_roms[0]),
	offsetof(mbRomsConfig,master128_roms[1]),
	offsetof(mbRomsConfig,master128_roms[2]),
	offsetof(mbRomsConfig,master128_roms[3]),
	offsetof(mbRomsConfig,master128_roms[4]),
	offsetof(mbRomsConfig,master128_roms[5]),
	offsetof(mbRomsConfig,master128_roms[6]),
};

static const char *all_disc_interfaces[]={
	"none",
	"acorn1770",
	"watford1770",
	"opus1770",
	"chal256",
	"chal512",
	0,
};

static const char *master_disc_interface[]={
	"master1770",
	0,
};

static const mbModel mb_models[]={
	// B
	mbModel("BBC Model B","b",&bbcModelB::model,0xFFFF,mb_b_roms_offsets,&mbRomsConfig::os_b,0,all_disc_interfaces),

	// B+ and all its variants
	mbModel("BBC Model B with Watford ROM/RAM board","bwatford",&bbcModelBWatford::model,0xFFFF,mb_b_roms_offsets,&mbRomsConfig::os_b,0,
		all_disc_interfaces),
	mbModel("BBC Model B+","bplus",&bbcModelBPlus::model,0xFFFF,mb_b_roms_offsets,&mbRomsConfig::os_bplus,0,all_disc_interfaces),
	mbModel("BBC Model B+96","bplus96",&bbcModelBPlus::model,0xFFFF,mb_b_roms_offsets,&mbRomsConfig::os_bplus,(1<<12)|(1<<13),all_disc_interfaces),
	mbModel("BBC Model B+128","bplus128",&bbcModelBPlus::model,0xFFFF,mb_b_roms_offsets,&mbRomsConfig::os_bplus,(1<<12)|(1<<13)|(1<<0)|(1<<1),
		all_disc_interfaces),

	// M128
#ifdef mbENABLE_M128
	mbModel("BBC Master 128","m128",&bbcModelMaster128::model,0x01FF,mb_m128_roms_offsets,&mbRomsConfig::os_master128,(1<<7)|(1<<6)|(1<<5)|(1<<4),
		master_disc_interface),
#endif
};
static const unsigned mb_num_models=sizeof mb_models/sizeof mb_models[0];

mbModel::mbModel(const char *long_name_in,const char *short_name_in,const bbcModel *model_in,
	t65::word b_rom_mask_in,const unsigned *filenames_offsets,wxString mbRomsConfig::*os_name_in,
	t65::word force_swram_mask_in,const char **valid_disc_interfaces_in):
long_name(long_name_in),
short_name(short_name_in),
model(model_in),
b_rom_mask(b_rom_mask_in),
force_swram_mask(force_swram_mask_in),
os_name(os_name_in),
filenames_offsets_(filenames_offsets),
valid_disc_interfaces(valid_disc_interfaces_in)
{
}

bool mbModel::LoadRom(int slot,const mbRomsConfig::Rom *rom) const {
	std::vector<t65::byte> contents;
	
	if(!rom->filename.empty()&&!mbLoadFile(rom->filename,contents)) {
		wxLogWarning("Slot %d: failed to load \"%s\"",slot,rom->filename.c_str());
		return false;
	}
	
	wxLogDebug("LoadRom: slot %d: %s: %u bytes",slot,rom->filename.c_str(),contents.size());
	bbcComputer::SetRomContents(slot,&contents[0],contents.size());

	bool is_ram=rom->ram&&(this->b_rom_mask&(1<<slot));
	bbcComputer::SetRomWriteable(slot,is_ram);
	
	return true;
}

mbRomsConfig::Rom *mbModel::CfgRomPtr(mbRomsConfig &cfg,int slot) const {
	BASSERT(slot>=0&&slot<16);
	char *base=reinterpret_cast<char *>(&cfg);
	return reinterpret_cast<mbRomsConfig::Rom *>(base+filenames_offsets_[slot]);
}

const mbRomsConfig::Rom *mbModel::CfgRomPtr(const mbRomsConfig &cfg,int slot) const {
	BASSERT(slot>=0&&slot<16);
	const char *base=reinterpret_cast<const char *>(&cfg);
	return reinterpret_cast<const mbRomsConfig::Rom *>(base+filenames_offsets_[slot]);
}

//void mbModel::GetRomsPtrs(mbRomsConfig &cfg,mbRomsConfig::Rom **roms_array) const {
//	for(unsigned i=0;i<16;++i) {
//		unsigned d=filenames_offsets_[i];
//		char *pcfg=reinterpret_cast<char *>(&cfg);
//		roms_array[i]=reinterpret_cast<mbRomsConfig::Rom *>(pcfg+d);
//	}
//}
//
//void mbModel::GetRomsPtrs(const mbRomsConfig &cfg,const mbRomsConfig::Rom **roms_array) const {
//	for(unsigned i=0;i<16;++i) {
//		unsigned d=filenames_offsets_[i];
//		const char *pcfg=reinterpret_cast<const char *>(&cfg);
//		roms_array[i]=reinterpret_cast<const mbRomsConfig::Rom *>(pcfg+d);
//	}
//}

bool mbModel::LoadRoms(const mbRomsConfig &cfg) const {
	const wxString &os_rom=cfg.*os_name;

	//Reset ROMs first
	bbcComputer::SetOsRomContents(0,0);
	for(int i=0;i<16;++i) {
		bbcComputer::SetRomContents(i,0,0);
		bbcComputer::SetRomWriteable(i,false);
	}
	
	std::vector<t65::byte> rom;
	if(os_rom.empty()||!mbLoadFile(os_rom,rom)) {
		wxLogWarning("OS ROM: failed to load \"%s\"",os_rom.c_str());
		return false;
	}
	bbcComputer::SetOsRomContents(&rom[0],rom.size());

	bool ok=true;
	for(int i=0;i<16;++i) {
		const mbRomsConfig::Rom *rom=this->CfgRomPtr(cfg,i);
		
		if(!this->LoadRom(i,rom)) {
			ok=false;
		}

		if(this->force_swram_mask&(1<<i)) {
			bbcComputer::SetRomWriteable(i,true);
		}
	}
		
	if(!ok) {
		wxLogError("Failed to load ROMs");
		return false;
	}
	
	return true;
}

const mbModel *mbModelFromShortName(const wxString &short_name) {
	for(unsigned i=0;i<mb_num_models;++i) {
		if(short_name.CmpNoCase(mb_models[i].short_name)==0) {
			return &mb_models[i];
		}
	}
	
	return 0;
}

void mbModelGetAll(std::vector<const mbModel *> *all_models) {
	BASSERT(all_models);
	for(unsigned i=0;i<mb_num_models;++i) {
		all_models->push_back(&mb_models[i]);
	}
}
