#include "pch.h"
#include "mbVideoConfigDialog.h"
#include <bbcVideo.h>

//Width=640
//Height=272
//FrameSkip=1
//CleanupFrequency=1
//FullScreen=0
//FullScreenScanlinesMode=interlace
//WindowedScanlinesMode=single
//FullScreenRefreshRate=75
//MonitorType=0

static const int border_size=5;
static const int border_flags=wxTOP|wxRIGHT|wxLEFT;
static const int left_proportion=1;
static const int right_proportion=1;

wxBoxSizer *mbVideoConfigDialog::NumberEntryBox(const wxString &caption,wxString *var) {
	wxTextCtrl *textentry=new wxTextCtrl(this,-1,*var);
	textentry->SetValidator(wxTextValidator(wxFILTER_NUMERIC,var));
	
	wxBoxSizer *sz=new wxBoxSizer(wxHORIZONTAL);
	sz->Add(new wxStaticText(this,-1,caption),left_proportion,border_flags,border_size);
	sz->Add(textentry,right_proportion,border_flags,border_size);

	return sz;
}

wxStaticBoxSizer *mbVideoConfigDialog::StaticBox(const wxString &caption,
	const std::vector<wxObject *> &contents)
{
	wxStaticBox *box=new wxStaticBox(this,-1,caption);
	wxStaticBoxSizer *sz=new wxStaticBoxSizer(box,wxVERTICAL);
	for(unsigned i=0;i<contents.size();++i) {
		wxObject *obj=contents[i];

		if(obj->IsKindOf(CLASSINFO(wxSizer))) {
			sz->Add(static_cast<wxSizer *>(obj),1,wxALL|wxGROW,border_size);
		} else if(obj->IsKindOf(CLASSINFO(wxWindow))) {
			sz->Add(static_cast<wxWindow *>(obj),1,wxALL|wxGROW,border_size);
		} else {
			BASSERT(false);//ooops...
		}
	}

	return sz;
}

mbVideoConfigDialog::mbVideoConfigDialog(wxWindow *parent,const mbVideoConfig &cfg,
	const int *hostgfx_rates):
wxDialog(parent,-1,"Configure video",wxDefaultPosition,wxDefaultSize,
	wxDEFAULT_DIALOG_STYLE),
cfg_(cfg)
{
	width_=wxString::Format("%d",cfg_.width);
	height_=wxString::Format("%d",cfg_.height);
	cleanup_freq_=wxString::Format("%d",cfg_.cleanup_frequency);
	display_freq_=wxString::Format("%d",cfg_.frame_skip);

	std::vector<wxObject *> windowed,full_screen,settings;

	//Display box
	windowed.push_back(this->NumberEntryBox("&Width",&width_));
	windowed.push_back(this->NumberEntryBox("&Height",&height_));

	wxBoxSizer *hz_sz=new wxBoxSizer(wxHORIZONTAL);
	{
		hz_cb_=new wxComboBox(this,-1,"",wxDefaultPosition,wxDefaultSize,0,0,
			wxCB_DROPDOWN|wxCB_SORT|wxCB_READONLY);
		hz_sz->Add(new wxStaticText(this,-1,"&Refresh rate"),left_proportion,border_flags,border_size);
		hz_sz->Add(hz_cb_,right_proportion,border_flags,border_size);

		int selection=-1;
		for(int i=0;hostgfx_rates[i]!=0;++i) {
			int hz=hostgfx_rates[i];
			hz_cb_->Append(wxString::Format("%dHz",hz),reinterpret_cast<void *>(hz));//urgh!!
			if(cfg.full_screen_refresh_rate==hz) {
				selection=i;
			}
		}
		hz_cb_->Append("Default",reinterpret_cast<void *>(0));
		if(selection<0) {
			//which also caters for the case when cfg.full_screen_refresh_rate==0.
			selection=hz_cb_->GetCount()-1;
		}

		hz_cb_->SetSelection(selection);
	}
	full_screen.push_back(hz_sz);

	wxCheckBox *vsync=new wxCheckBox(this,-1,"Sync to refresh rate");
	vsync->SetValidator(wxGenericValidator(&cfg_.full_screen_vsync));
	full_screen.push_back(vsync);

	//Settings box
	settings.push_back(this->NumberEntryBox("Cleanup frequency",&cleanup_freq_));
	settings.push_back(this->NumberEntryBox("Display frequency",&display_freq_));

	//Buttons
	wxBoxSizer *btns_sz=new wxBoxSizer(wxHORIZONTAL);
	wxButton *ok=new wxButton(this,wxID_OK,"OK");
	wxButton *cancel=new wxButton(this,wxID_CANCEL,"Cancel");
	ok->SetDefault();
	btns_sz->Add(ok,0,wxALL|wxALIGN_RIGHT,border_size);
	btns_sz->Add(cancel,0,wxALL|wxALIGN_RIGHT,border_size);

	//Altogether now...
	wxBoxSizer *all_sz=new wxBoxSizer(wxVERTICAL);
	all_sz->Add(this->StaticBox("Windowed",windowed),0,wxGROW|border_flags,border_size);
	all_sz->Add(this->StaticBox("Full screen",full_screen),0,wxGROW|border_flags,border_size);
	all_sz->Add(this->StaticBox("Options",settings),0,wxGROW|border_flags,border_size);
	all_sz->Add(btns_sz,1,wxALIGN_RIGHT,border_size);

	//
	this->SetSizer(all_sz);
	all_sz->SetSizeHints(this);
	this->Layout();

	this->Move(cfg_.dlg_rect.GetLeft(),cfg_.dlg_rect.GetTop());
}

void mbVideoConfigDialog::GetResult(mbVideoConfig *cfg) {
	BASSERT(cfg);
	*cfg=cfg_;
}

static void GetInt(const char *name,const wxString &str,int *dest,int mini,int maxi) {
	long value;
	bool bad=false;

	BASSERT(mini>=0);
	if(!str.ToLong(&value,0)) {
		bad=true;
		value=mini;
	} else if(mini>=0&&value<mini) {
		bad=true;
		value=mini;
	} else if(maxi>=0&&value>maxi) {
		bad=true;
		value=maxi;
	}

	if(bad) {
		wxLogWarning("Invalid %s \"%s\" -- reset to %ld\n",name,str.c_str(),value);
	}

	*dest=value;
}

void mbVideoConfigDialog::OnOk(wxCommandEvent &event) {
	int rate=reinterpret_cast<int>(hz_cb_->GetClientData(hz_cb_->GetSelection()));
	this->TransferDataFromWindow();
	
	cfg_.full_screen_refresh_rate=rate;
	GetInt("width",width_,&cfg_.width,bbcVideo::min_buffer_width,bbcVideo::max_buffer_width);
	GetInt("height",height_,&cfg_.height,bbcVideo::min_buffer_height,bbcVideo::max_buffer_height);
	GetInt("cleanup frequency",cleanup_freq_,&cfg_.cleanup_frequency,1,-1);
	GetInt("display frequency",display_freq_,&cfg_.frame_skip,1,mbVideoConfig::max_frame_skip);

	cfg_.dlg_rect=this->GetRect();

	this->EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(mbVideoConfigDialog,wxDialog)
	EVT_BUTTON(wxID_OK,mbVideoConfigDialog::OnOk)
END_EVENT_TABLE()
